//##################################################################################################
//
//  Ceetron Desktop Components
//  QtDemoApp
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QWidget>

#include <vector>

#include "CeeCore/PtrRef.h"
#include "CeeCore/Vec3d.h"


namespace Ui {
class ParticleTracesWidget;
}

class MouseClickEventFilter;

namespace cee {
    namespace vis { class View; }
    namespace ug { class UnstructGridModel; }
}


class QModelIndex;
class QStandardItem;
class QStandardItemModel;


//==================================================================================================
//
// 
//
//==================================================================================================
class ParticleTracesWidget : public QWidget
{
    Q_OBJECT

public:
    ParticleTracesWidget(QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowFlags());
    virtual ~ParticleTracesWidget();

    void setup(cee::vis::View* view, cee::ug::UnstructGridModel* model);
    void addSeedPoint(const cee::Vec3d& seedPoint);

private:
    void updatePanel();
    void applyScaleFactor();
    void enableDisableItems();

private slots:
    void applySelection(const QModelIndex& index);
    void setParticleTracesVisible(bool show);
    void setVectorResult(int index);
    void setMappedScalarResult(int index);
    void setVisualizationStyle(int index);
    void applyNumberOfPulses();
    void enableRelativeScaling(bool enable);
    void applyPrimitiveRadius();
    void selectFixedColor();
    void addSeedPoint();
    void removeSeedPoint();
    void removeAllSeedPoints();
    void setSeedPoint(QStandardItem* item);

private:
    Ui::ParticleTracesWidget*               m_ui;

    QStandardItemModel*                     m_listModel;

    cee::PtrRef<cee::ug::UnstructGridModel> m_unstructModel;
    cee::PtrRef<cee::vis::View>             m_view;

    MouseClickEventFilter*                  m_mouseClickEventFilter;
};

