//##################################################################################################
//
//  Ceetron Desktop Components
//  QtDemoApp
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QWidget>

#include "CeeCore/PtrRef.h"

#include "CeePlot2d/OverlayPlot.h"
#include "MouseClickEventFilter.h"

namespace Ui {
class PlotWidget;
}

namespace cee {
    namespace vis { class View; }
    namespace ug { class UnstructGridModel; }
}


class QModelIndex;
class QStandardItemModel;


//==================================================================================================
//
// 
//
//==================================================================================================
class PlotWidget : public QWidget
{
    Q_OBJECT

public:
    PlotWidget(QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowFlags());
    virtual ~PlotWidget();

    void    setup(cee::vis::View* view, cee::ug::UnstructGridModel* model);

private:
    void    updatePanel();
    void    createDummyPlotData();
    bool    createPlotFromProperties();
    void    updateCurveList();

private slots:
    void    showPlot(bool show);
    void    setPlotTitle(const QString& title);
    void    showLegend(bool show);
    void    enableAutoRange(bool use);
    void    applyRange();

    void    updateCurveSettings();
    void    addCurve();
    void    editCurve();
    void    removeCurve();
    void    setCurveTitle(const QString& title);
    void    selectCurveColor();
private:
    Ui::PlotWidget*                         m_ui;

    cee::PtrRef<cee::ug::UnstructGridModel> m_unstructModel;
    cee::PtrRef<cee::vis::View>             m_view;
    cee::PtrRef<cee::plt::OverlayPlot>      m_overlayPlot;

    MouseClickEventFilter*                  m_mouseClickEventFilter;
};
