//##################################################################################################
//
//  Ceetron Desktop Components
//  QtDemoApp
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QWidget>

#include "CeeCore/PtrRef.h"
#include "CeeReport/Repository.h"

namespace Ui {
class ReportWidget;
}

namespace cee {
    namespace vis { class View; }
    namespace ug { class UnstructGridModel; }
}


class QModelIndex;
class QStandardItemModel;
class QProgressDialog;


//==================================================================================================
//
// 
//
//==================================================================================================
class ReportWidget : public QWidget
{
    Q_OBJECT

public:
    ReportWidget(QWidget* parent = 0, Qt::WindowFlags f = Qt::WindowFlags());
    virtual ~ReportWidget();

    void    setup(cee::vis::View* view, cee::ug::UnstructGridModel* model);

private:
    void    updatePanel();
    void    updateRepositoryList();
    void    showReport(cee::Str reportName) const;
    static QString parseTopLevelJsonValue(const QString json, const QString key);

private slots:
    void    captureSnapshotFromView();
    void    deleteSnapshot();
    void    loadRepository();
    void    saveRepository();

    void    updateFromCurrent();

    void    sendModelsToCloud();
    void    uploadNextSnapshot();
    void    createReport();

    void    sendToCloud(const cee::rep::Snapshot* snapshot);
    void    cloudUploadFinished();
    void    updateUploadProgress(qint64 bytesSent, qint64 bytesTotal);

private:
    Ui::ReportWidget*                       m_ui;

    cee::PtrRef<cee::rep::Repository>       m_repository;
    cee::PtrRef<cee::ug::UnstructGridModel> m_unstructModel;
    cee::PtrRef<cee::vis::View>             m_view;

    size_t                                  m_snapshotIndexToUploadToCloud;
    QProgressDialog*                        m_progressDialog;

};

