//##################################################################################################
//
//  Ceetron Desktop Components
//  QtDemoApp
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "SendToCloudDialogs.h"

#include <QLineEdit>
#include <QPushButton>
#include <QLabel>
#include <QTextBrowser>
#include <QVBoxLayout>

//==================================================================================================
///
/// Get the user's Ceetron Cloud upload ID
/// 
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
EnterUploadIDDialog::EnterUploadIDDialog(QWidget* parent) : QDialog(parent)
{
    const QString ceetronCloudUrl = "https://cloud.ceetron.com/";

    setWindowTitle("Enter your CeetronCloud Upload ID");
    QTextBrowser* textBrowser = new QTextBrowser(this);

    QVBoxLayout* layout = new QVBoxLayout();
    layout->addWidget(textBrowser);
    setLayout(layout);

    QSpacerItem* vSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
    layout->addItem(vSpacer);

    {
        // The bottom part with the OK button and
        // possibly text label indicating that we're running a debug build
        QHBoxLayout* inputLayout = new QHBoxLayout;

        // Add OK button
        //QSpacerItem* hSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
        //bottomLayout->addItem(hSpacer);

        QLabel* label = new QLabel("My Upload ID:");
        inputLayout->addWidget(label);

        m_uploadIdLineEdit = new QLineEdit();
        inputLayout->addWidget(m_uploadIdLineEdit);
        m_uploadIdLineEdit->setFocus();

        layout->addLayout(inputLayout);
    }

    // The bottom part with the OK button and
    // possibly text label indicating that we're running a debug build
    QHBoxLayout* bottomLayout = new QHBoxLayout;

    // Add OK button
    QSpacerItem* hSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    bottomLayout->addItem(hSpacer);

    QPushButton* buttonCancel = new QPushButton("&Cancel", this);
    connect(buttonCancel, SIGNAL(clicked()), this, SLOT(reject()) );
    bottomLayout->addWidget(buttonCancel);

    QPushButton* buttonOk = new QPushButton("&OK", this);
    buttonOk->setAutoDefault(true);
    buttonOk->setDefault(true);
    connect(buttonOk, SIGNAL(clicked()), this, SLOT(accept()) );
    bottomLayout->addWidget(buttonOk);

    layout->addLayout(bottomLayout);

    textBrowser->setOpenExternalLinks(true);

    QString htmlText = "<h1>Welcome to CeetronCloud</h1>";
    htmlText += "<p>Please enter your CeetronCloud Upload ID in the field below.<p>";
    htmlText += "<p>You find this value in your profile page on <a href=\"" + ceetronCloudUrl + "myAccount\">" + ceetronCloudUrl +"myAccount</a></p>";
    htmlText += "<p>If you do not have an account on CeetronCloud, please sign up at <br><a href=\"" + ceetronCloudUrl + "signup\">" + ceetronCloudUrl +"signup</a> for a free account.";
    textBrowser->setHtml(htmlText);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
QString EnterUploadIDDialog::uploadId() const 
{
    return m_uploadIdLineEdit->text();
}


//==================================================================================================
///
/// Display upload information when it is done
/// 
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
UploadResultDialog::UploadResultDialog(QWidget* parent, QString htmlText, QString title): QDialog(parent)
{
    setWindowTitle(title);
    QTextBrowser* textBrowser = new QTextBrowser(this);

    QVBoxLayout* layout = new QVBoxLayout();
    layout->addWidget(textBrowser);
    setLayout(layout);

    QSpacerItem* vSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
    layout->addItem(vSpacer);

    // The bottom part with the OK button and
    // possibly text label indicating that we're running a debug build
    QHBoxLayout* bottomLayout = new QHBoxLayout;

    // Add OK button
    QSpacerItem* hSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    bottomLayout->addItem(hSpacer);

    QPushButton* buttonOk = new QPushButton("&OK", this);
    buttonOk->setAutoDefault(true);
    buttonOk->setDefault(true);
    buttonOk->setFocus();
    connect(buttonOk, SIGNAL(clicked()), this, SLOT(accept()) );
    bottomLayout->addWidget(buttonOk);

    layout->addLayout(bottomLayout);

    textBrowser->setOpenExternalLinks(true);
    textBrowser->setHtml(htmlText);
}

