//##################################################################################################
//
//  Ceetron Desktop Components
//  QtDemoApp
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "MainWindow.h"

#include "CeeCore/CoreComponent.h"
#include "CeeCore/LogManager.h"
#include "CeeCore/LogDestinationFile.h"
#include "CeeCore/AppLogging.h"
#include "CeeVisualization/VisualizationComponent.h"
#include "CeeVisualization/OpenGLContextGroup.h"
#include "CeeUnstructGrid/UnstructGridComponent.h"
#include "CeeExport/ExportComponent.h"
#include "CeeReport/ReportComponent.h"
#ifdef CEE_USE_IMPORT_CAE
#include "CeeImportCae/ImportCaeComponent.h"
#endif

#include "ceeqtUtilsCore.h"
#include "ceeqtOpenGLFunctionsBackend_qt5.h"

#include <QApplication>
#include <QDesktopServices>
#include <QStandardPaths>
#include <QDir>
#include <QTime>
#include <QDate>

#include <locale.h>


//--------------------------------------------------------------------------------------------------
/// The main entry point for the application
//--------------------------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    // Initialize the parts of Ceetron Desktop Components that we'll use.
    // Please contact support@ceetron.com if you don't have a valid license.
    unsigned int licenseKeyA = 0x00;    // Todo: Set by developer 
    unsigned int licenseKeyB = 0x00;    // Todo: Set by developer
    cee::PtrRef<cee::Instance> compInst = cee::CoreComponent::initialize(licenseKeyA, licenseKeyB);
    cee::vis::VisualizationComponent::initialize(compInst.get());
    cee::exp::ExportComponent::initialize(compInst.get());
    cee::ug::UnstructGridComponent::initialize(compInst.get());
    cee::rep::ReportComponent::initialize(compInst.get());
#ifdef CEE_USE_IMPORT_CAE
    cee::imp::cae::ImportCaeComponent::initialize(compInst.get());
#endif


    QApplication app(argc, argv);
    QApplication::setOrganizationName("Ceetron");
    QApplication::setApplicationName("Qt Demo Application");

    // On Linux, Qt will use the system locale, force number formatting settings back to "C" locale
    setlocale(LC_NUMERIC,"C");

    // Write info/warnings error to a log file
    // The logfile will be put in %APPDATA%\Local\<appname>
    QDir logDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
    if (!logDir.exists())
    {
        logDir.mkpath(logDir.absolutePath());
    }

    QString logFile = logDir.absoluteFilePath("CeetronDesktopComponentsQtDemoLog.txt");
    cee::CoreComponent::logManager()->setDestination(new cee::LogDestinationFile(cee::qt::UtilsCore::toStr(logFile)));
    
    cee::AppLogging::initialize("cee.demo");
    //cee::AppLogging::setFullDebugLogLevels();  

    cee::CoreComponent::logVersionInfo();
    QTime time = QTime::currentTime();
    QDate date = QDate::currentDate();
    CEE_LOG_APP_INFO_QSTR("Log for Ceetron Desktop Components Qt Demo App. Started: " + date.toString() + " " + time.toString());
 
    cee::vis::OpenGLFunctionsBackend* functionsBackend = NULL;
#if QT_VERSION >= 0x050600
    // Use back-end implemented on top of Qt
    functionsBackend = new cee::qt::OpenGLFunctionsBackend_qt5;
#endif
    cee::PtrRef<cee::vis::OpenGLContextGroup> contextGroup = cee::vis::VisualizationComponent::createOpenGLContextGroup(functionsBackend);
    MainWindow mainWindow(contextGroup.get());
    mainWindow.show();
    mainWindow.openDefaultModel();

    app.exec();

    return EXIT_SUCCESS;
}
