//##################################################################################################
//
//  Ceetron Desktop Components
//  QtMinimal
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "QMMainWindow.h"

#include "CeeVisualization/View.h"

#include "CeeVisualization/VisualizationComponent.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "CeeGeometry/GeometryModel.h"
#include "CeeGeometry/DataGenerator.h"
#include "CeeGeometry/Part.h"
#include "CeeGeometry/PartSettings.h"
#include "CeeGeometry/EffectColor.h"

#include <QFrame>
#include <QHBoxLayout>
#include <QAction>
#include <QMenu>
#include <QMenuBar>




//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
QMMainWindow::QMMainWindow(cee::vis::OpenGLContextGroup* contextGroup)
{
    QFrame* mainFrame = new QFrame;
    QHBoxLayout* frameLayout = new QHBoxLayout(mainFrame);
    setCentralWidget(mainFrame);

    m_viewer = new cee::qt::ViewerOpenGLWidgetDemo(contextGroup, mainFrame, this);
    
    m_viewer->setFocus();
    frameLayout->addWidget(m_viewer);

    QMenu* menu = menuBar()->addMenu("&Scenes");
    menu->addAction("Default Scene", this, SLOT(slotCreateDefaultScene()));
    menu->addAction("Clear Scene", this, SLOT(slotClearScene()));

    cee::vis::View* gcView = new cee::vis::View;
    m_viewer->setView(gcView);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
QMMainWindow::~QMMainWindow()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void QMMainWindow::onViewerOpenGLReady(cee::qt::ViewerOpenGLWidget* /*viewer*/)
{
    // Wait until openGL setup is ready
    slotCreateDefaultScene();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void QMMainWindow::slotCreateDefaultScene()
{
    if (!m_viewer) return;

    cee::vis::View* gcView = m_viewer->view();
    gcView->removeAllModels();

    cee::PtrRef<cee::geo::GeometryModel> boxModel = new cee::geo::GeometryModel;
    gcView->addModel(boxModel.get());
    {
        cee::geo::Part* part = new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(0, 0, 0), cee::Vec3d(1, 0.75f, 0.5)).get());
        part->settings().addEffect(new cee::geo::EffectColor(cee::Color3f(0.00f, 0.53f, 0.58f)));
        boxModel->addPart(part);
    }
    {
        cee::geo::Part* part = new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(1.1, 0, 0), cee::Vec3d(1, 0.75f, 0.5)).get());
        part->settings().addEffect(new cee::geo::EffectColor(cee::Color3f(0.83f, 0.62f, 0.38f)));
        boxModel->addPart(part);
    }
    {
        cee::geo::Part* part = new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(0, 1.1, 0), cee::Vec3d(1, 0.75f, 0.5)).get());
        part->settings().addEffect(new cee::geo::EffectColor(cee::Color3f(0.85f, 0.32f, 0.11f)));
        boxModel->addPart(part);
    }

    cee::PtrRef<cee::geo::GeometryModel> sphereModel = new cee::geo::GeometryModel;
    gcView->addModel(sphereModel.get());
    {
        cee::geo::Part* part = new cee::geo::Part(cee::geo::DataGenerator::createSphere(cee::Vec3d(1.1, 1.1, 0), 0.5, 10).get());
        part->settings().addEffect(new cee::geo::EffectColor(cee::Color3f(0.00f, 0.53f, 0.58f)));
        sphereModel->addPart(part);
    }

    cee::BoundingBox bb = gcView->boundingBox();
    gcView->camera().fitView(bb, cee::Vec3d(0, 0, -1), cee::Vec3d(0, 1, 0));
    gcView->camera().inputHandler()->setRotationPoint(bb.center());

    m_viewer->update();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void QMMainWindow::slotClearScene()
{
    if (!m_viewer) return;

    cee::vis::View* view = m_viewer->view();
    view->removeAllModels();

    m_viewer->update();
}


