//##################################################################################################
//
//  Ceetron Desktop Components
//  QtMultiView
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QMainWindow>
#include <QPointer>

#include "CeeCore/PtrRef.h"
#include "CeeVisualization/OpenGLContextGroup.h"
#include "ceeqtViewerOpenGLWidgetDemo.h"


//==================================================================================================
//
// 
//
//==================================================================================================
class QMVMainWindow : public QMainWindow
{
    Q_OBJECT

private:
    enum ViewCreation
    {
        NEW_VIEWS,
        RECYCLE_VIEWS
    };

public:
    QMVMainWindow(cee::vis::OpenGLContextGroup* contextGroup);

public slots:
    void    slotCreate1Viewer();
    void    slotCreate2Viewers();
    void    slotCreate3Viewers();
    void    slotCreate2ViewersRecycleViews();
    void    slotCreate3ViewersRecycleViews();
    void    slotPopulateViewsWithSameModel();
    void    slotPopulateViewsWithIndividualModels();
    void    slotClearAllViews();
    void    slotDeleteAllViews();
    void    slotDeleteAllViewers();

private:
    void            createViewers(size_t numViewers, ViewCreation viewCreation);
    void            deleteViewers();
    void            populateViewsWithSingleModel();
    void            populateViewsWithMixedModels();
    void            populateView_sphereAndBoxesHalo(cee::vis::View* view);
    void            populateView_unstructGridModel(cee::vis::View* view);
    void            populateView_cylinder(cee::vis::View* view);
    size_t          viewCount();
    cee::vis::View* view(size_t index);
    void            clearViewContents();
    void            redrawAllViewers();
    void	        closeEvent(QCloseEvent* event);

private:
    cee::PtrRef<cee::vis::OpenGLContextGroup>   m_openGLContextGroup;
    static const size_t                         MAX_NUM_VIEWERS = 3;
    QPointer<cee::qt::ViewerOpenGLWidgetDemo>   m_viewers[MAX_NUM_VIEWERS];
};

