//##################################################################################################
//
//  Ceetron Desktop Components
//  QtMinimal
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "CeeCore/CoreComponent.h"
#include "CeeCore/Version.h"
#include "CeeCore/LogDestinationFile.h"

#include "CeeVisualization/VisualizationComponent.h"
#include "CeeVisualization/OpenGLContextGroup.h"

#include "CeeUnstructGrid/UnstructGridComponent.h"
#include "CeeReport/ReportComponent.h"
#include "CeeExport/ExportComponent.h"

#include "ceeqtUtilsCore.h"

#include "QRMMainWindow.h"

#include <QApplication>
#include <QDateTime>
#include <QDir>
#include <QDesktopServices>
#include <QStandardPaths>

#include <locale.h>




//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    // Initialize the parts of Ceetron Desktop Components that we'll use.
    // Please contact support@ceetron.com if you don't have a valid license.
    unsigned int licenseKeyA = 0x00;    // Todo: Set by developer 
    unsigned int licenseKeyB = 0x00;    // Todo: Set by developer
    cee::PtrRef<cee::Instance> compInst = cee::CoreComponent::initialize(licenseKeyA, licenseKeyB);
    cee::vis::VisualizationComponent::initialize(compInst.get());
    cee::ug::UnstructGridComponent::initialize(compInst.get());
    cee::rep::ReportComponent::initialize(compInst.get());
    cee::exp::ExportComponent::initialize(compInst.get());

    QApplication::setAttribute(Qt::AA_ShareOpenGLContexts);

    QApplication app(argc, argv);
    QApplication::setOrganizationName("Ceetron");
    QApplication::setApplicationName("Qt Report Manager");
    // On Linux, Qt will use the system locale, force number formatting settings back to "C" locale
    setlocale(LC_NUMERIC,"C");

    // Write info/warnings error to a log file
    // The logfile will be put in %APPDATA%\Local\<appname>
    QDir logDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
    if (!logDir.exists())
    {
        logDir.mkpath(logDir.absolutePath());
    }

    QString logFile = logDir.absoluteFilePath("CDCQtRepositoryManager.txt");
    cee::CoreComponent::logManager()->setDestination(new cee::LogDestinationFile(cee::qt::UtilsCore::toStr(logFile)));

    cee::AppLogging::initialize("cee.demo");
    //cee::AppLogging::setFullDebugLogLevels();  

    cee::CoreComponent::logVersionInfo();

    QTime time = QTime::currentTime();
    QDate date = QDate::currentDate();
    CEE_LOG_APP_INFO_QSTR("Log for CDC QtReportRepositoryManager Started: " + date.toString() + " " + time.toString());

    cee::PtrRef<cee::vis::OpenGLContextGroup> contextGroup = cee::vis::VisualizationComponent::createOpenGLContextGroup();
    QRMMainWindow window(contextGroup.get());
    QString verInfo = QString("CDC %1, Qt %2").arg(CEE_VERSION_STRING).arg(QT_VERSION_STR);
    window.setWindowTitle("QtReportRepositoryManager " + verInfo);
    window.resize(1000, 800);
    window.show();

    app.exec();

    return EXIT_SUCCESS;
}
