//##################################################################################################
//
//  Ceetron Report
//  QtReportRepositoryViewer
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QMainWindow>

#include "CeeCore/PtrRef.h"
#include "CeeCore/Str.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeReport/Repository.h"

#include "ceeqtViewerOpenGLWidgetDemo.h"

namespace Ui {
    class QRMMainWindow;
}

class QProgressDialog;

//==================================================================================================
//
// 
//
//==================================================================================================
class QRMMainWindow : public QMainWindow
{
    Q_OBJECT

public:
    QRMMainWindow(cee::vis::OpenGLContextGroup* contextGroup);
    ~QRMMainWindow();

protected:
    void resizeEvent(QResizeEvent* event);

private:
    void loadRepository(const cee::Str& filename);
    void updateRepositoryList();

    void showSnapshot(size_t snapshotIndex);
    void showVTFxSnapshot(size_t snapshotIndex);
    void showImageSnapshot(size_t snapshotIndex);
    bool showTableSnapshot(size_t snapshotIndex);

    void resizeImage();

    void showReport(cee::Str reportName) const;

    void sendToCloud(const cee::rep::Snapshot* snapshot);
    static QString parseTopLevelJsonValue(const QString json, const QString key);

public slots:
    void slotAddSnapshot(cee::rep::Snapshot* snapshot);

protected slots:
    void slotOpen();
    void slotSave();
    void slotUpdateFromCurrent();
    void slotOpenModel();

    void slotCreateWordReport();
    void slotCreatePowerPointReport();
    void slotCreateHtmlReport();

    void slotViewReportSummary();
    void slotViewReportValidation();
    void slotClearSnapshotList();

    void slotSendToCloud();
    void cloudUploadFinished();
    void updateUploadProgress(qint64 bytesSent, qint64 bytesTotal);
    void clearCloudUploadId();

private:
    Ui::QRMMainWindow*                      m_ui;
    cee::vis::OpenGLContextGroup*           m_contextGroup;
    cee::PtrRef<cee::rep::Repository>       m_repository;
    cee::qt::ViewerOpenGLWidgetDemo*        m_viewer;
    cee::PtrRef<cee::ug::UnstructGridModel> m_unstructModel;
    QProgressDialog*                        m_progressDialog;
};

