//##################################################################################################
//
//  Ceetron Desktop Components
//  QtSendToCloud
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QObject>
#include <QNetworkReply>
#include <iostream>

//--------------------------------------------------------------------------------------------------
/// Small class to track upload progress and completion
//--------------------------------------------------------------------------------------------------
class CloudUploadTracker : public QObject
{
    Q_OBJECT

public:
    CloudUploadTracker(QObject *parent, QString cloudURL) : QObject(parent), m_cloudURL(cloudURL) {}

public slots:
    void cloudUploadFinished()
    {
        QNetworkReply* reply = (QNetworkReply*)sender();
        const int httpStatusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();

        QString message;
        if (httpStatusCode > 0)
        {
            QString responseJson = reply->readAll();

            if (httpStatusCode == 200)
            {
                const QString modelKey = parseTopLevelJsonValue(responseJson, "modelKey");
                const QString viewerUrl = m_cloudURL + "/v/" + modelKey;
                message = "#SUCCESS;" + viewerUrl + "; model key:" + modelKey;
            }
            else 
            {
                const QString failReason = parseTopLevelJsonValue(responseJson, "message");
                const QString apiErrorCode = parseTopLevelJsonValue(responseJson, "apiErrorCode");
                message = "#FAIL; Error: " + QString("%1 - httpStatusCode:%2 - apiErrorCode:%3").arg(failReason).arg(httpStatusCode).arg(apiErrorCode);
            }
        }
        else
        {
            message = QString("#ERROR; Network Error During Upload - %1").arg(reply->errorString());
        }

        std::cout << message.toStdString() << std::endl;

        reply->deleteLater();
        emit finished();
    }

    void updateUploadProgress(qint64 bytesSent, qint64 bytesTotal)
    {
        if (bytesTotal > 0)
        {
            float pctDone = 100.0f*(float)bytesSent/(float)bytesTotal;
            std::cout << "#PROGRESS; " << pctDone << "; " << bytesSent << " of " << bytesTotal << " bytes. " << pctDone << "% done." << std::endl;
        }
    }

signals:
    void finished();

private:
    /// Static helper function to do crude parsing of top level JSON values
    QString parseTopLevelJsonValue(const QString json, const QString key)
    {
        QString valString = "";

        const QString quotedKey = QString("\"%1\"").arg(key);
        const int keyIdx = json.indexOf(quotedKey);
        if (keyIdx >= 0)
        {
            int startQuote = json.indexOf("\"", keyIdx + quotedKey.length());
            int endQuote = json.indexOf("\"", startQuote + 1);

            if (startQuote >= 0 && endQuote > (startQuote + 1))
            {
                valString = json.mid(startQuote + 1, endQuote - startQuote - 1);
            }
        }

        return valString;
    }

private:
    QString m_cloudURL;
};
