//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "ceeqtOpenGLFunctionsBackend_qt53_deprecated.h"

#include "CeeCore/CoreComponent.h"
#include "CeeCore/LogManager.h"

#include <QtGlobal>

#if QT_VERSION < 0x050300
#error This Qt Backend requires Qt 5.3 or later
#endif

#include <QOpenGLContext>
#include <QOpenGLFunctions>
#include <QOpenGLFunctions_2_1>
#include <QOpenGLFunctions_3_0>
#include <QOpenGLFunctions_3_2_Core>


namespace cee {
namespace qt {


    
//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
OpenGLFunctionsBackend_qt53_deprecated::OpenGLFunctionsBackend_qt53_deprecated()
 :  m_qtFuncs(NULL),
    m_qtFuncs_21(NULL),
    m_qtFuncs_30(NULL),
    m_qtFuncs_32c(NULL),
    m_loggerName("cee.comp.qt")
{
    m_logMgr = cee::CoreComponent::logManager();
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
OpenGLFunctionsBackend_qt53_deprecated::~OpenGLFunctionsBackend_qt53_deprecated()
{
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
bool OpenGLFunctionsBackend_qt53_deprecated::initializeBackend(cee::vis::OpenGLBackendInfo* backendInfo)
{
    QOpenGLContext* currQtContext = QOpenGLContext::currentContext();
    Q_ASSERT(currQtContext);

    const QSurfaceFormat surfFormat = currQtContext->format();
    const QSurfaceFormat::RenderableType renderableType = surfFormat.renderableType();
    const QSurfaceFormat::OpenGLContextProfile profile = surfFormat.profile();
    const int majorVersion = surfFormat.majorVersion();
    const int minorVersion = surfFormat.minorVersion();

    cee::Str sRenderableType = "UNKNOWN";
    if      (renderableType == QSurfaceFormat::DefaultRenderableType) sRenderableType = "DefaultRenderableType";
    else if (renderableType == QSurfaceFormat::OpenGL)                sRenderableType = "OpenGL";
    else if (renderableType == QSurfaceFormat::OpenGLES)              sRenderableType = "OpenGLES";
    else if (renderableType == QSurfaceFormat::OpenVG)                sRenderableType = "OpenVG";

    cee::Str sProfile = "UNKNOWN";
    if      (profile == QSurfaceFormat::NoProfile)            sProfile = "NoProfile";
    else if (profile == QSurfaceFormat::CoreProfile)          sProfile = "CoreProfile";
    else if (profile == QSurfaceFormat::CompatibilityProfile) sProfile = "CompatibilityProfile";

    // Write debug info about what we discover to log
    m_logMgr->logDebug(m_loggerName, "Using OpenGLFunctionsBackend_qt53");
    m_logMgr->logDebug(m_loggerName, cee::Str("  qt.OpenGL version %1.%2").arg(majorVersion).arg(minorVersion));
    m_logMgr->logDebug(m_loggerName, "  qt.RenderableType: " + sRenderableType);
    m_logMgr->logDebug(m_loggerName, "  qt.Profile: " + sProfile);


    // Reset pointers in case we fail to get the main functions
    m_qtFuncs_21 = NULL;
    m_qtFuncs_30 = NULL;
    m_qtFuncs_32c = NULL;

    m_qtFuncs = currQtContext->functions();
    if (!m_qtFuncs)
    {
        return false;
    }

    // The versioned functions cannot be used on OpenGL ES
    if (renderableType != QSurfaceFormat::OpenGLES)
    {
        // There seems to be some subtle differences between Qt versions going on here. According to the doc 
        // of Qt 5.5.1 the function objects returned below are ready initialized and ready to use, BUT the Qt 5.4.2 doc 
        // states that we must initialize them ourselves. Always do the initialize calls to be certain
        m_qtFuncs_21 = currQtContext->versionFunctions<QOpenGLFunctions_2_1>();
        if (m_qtFuncs_21 && !m_qtFuncs_21->initializeOpenGLFunctions())
        {
            m_qtFuncs_21 = NULL;
        }

        m_qtFuncs_30 = currQtContext->versionFunctions<QOpenGLFunctions_3_0>();
        if (m_qtFuncs_30 && !m_qtFuncs_30->initializeOpenGLFunctions())
        {
            m_qtFuncs_30 = NULL;
        }

        m_qtFuncs_32c = currQtContext->versionFunctions<QOpenGLFunctions_3_2_Core>();
        if (m_qtFuncs_32c && !m_qtFuncs_32c->initializeOpenGLFunctions())
        {
            m_qtFuncs_32c = NULL;
        }
    }


    backendInfo->oglMajorVersion = majorVersion;
    backendInfo->oglMinorVersion = minorVersion;
    backendInfo->isOpenGLES = renderableType == QSurfaceFormat::OpenGLES ? true : false;

    if (m_qtFuncs->hasOpenGLFeature(QOpenGLFunctions::Framebuffers))
    {
        backendInfo->capabilityFlags |= cee::vis::OpenGLBackendInfo::FRAMEBUFFER_OBJECT;
    }

    if (majorVersion >= 3 || currQtContext->hasExtension("GL_ARB_texture_float"))
    {
        backendInfo->capabilityFlags |= cee::vis::OpenGLBackendInfo::TEXTURE_FLOAT;
    }

    if (majorVersion >= 3 || currQtContext->hasExtension("GL_ARB_texture_rg"))
    {
        backendInfo->capabilityFlags |= cee::vis::OpenGLBackendInfo::TEXTURE_RG;
    }

    if (majorVersion >= 4 || (majorVersion == 3 && minorVersion >= 1) || currQtContext->hasExtension("GL_ARB_texture_rectangle"))
    {
        backendInfo->capabilityFlags |= cee::vis::OpenGLBackendInfo::TEXTURE_RECTANGLE;
    }

    if (majorVersion >= 3 || currQtContext->hasExtension("GL_EXT_gpu_shader4"))
    {
        backendInfo->capabilityFlags |= cee::vis::OpenGLBackendInfo::FLAT_INTERPOLATION_QUALIFIER;
    }


    cee::Str sVersion(reinterpret_cast<const char*>(m_qtFuncs->glGetString(GL_VERSION)));
    cee::Str sVendor(reinterpret_cast<const char*>(m_qtFuncs->glGetString(GL_VENDOR)));
    cee::Str sRenderer(reinterpret_cast<const char*>(m_qtFuncs->glGetString(GL_RENDERER)));
    cee::Str sShadingLangVersion(reinterpret_cast<const char*>(m_qtFuncs->glGetString(GL_SHADING_LANGUAGE_VERSION)));
    m_logMgr->logDebug(m_loggerName, "  GL_VENDOR: " + sVendor);
    m_logMgr->logDebug(m_loggerName, "  GL_RENDERER: " + sRenderer);
    m_logMgr->logDebug(m_loggerName, "  GL_VERSION: " + sVersion);
    m_logMgr->logDebug(m_loggerName, "  GL_SHADING_LANGUAGE_VERSION: " + sShadingLangVersion);

    backendInfo->oglVersionString = sVersion;
    backendInfo->rendererString = sRenderer;
    backendInfo->vendorString = sVendor;
    backendInfo->backendDetailsString = "Qt backed, " + sRenderableType + ", " + sProfile + ", ceeqt_53_depr (Qt ver: " + QT_VERSION_STR + ")";

    return true;
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDepthFunc(cee::vis::GLenum func)
{
    m_qtFuncs->glDepthFunc(func);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDepthMask(cee::vis::GLboolean flag)
{
    m_qtFuncs->glDepthMask(flag);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glEnable(cee::vis::GLenum cap)
{
    m_qtFuncs->glEnable(cap);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDisable(cee::vis::GLenum cap)
{
    m_qtFuncs->glDisable(cap);
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBindBuffer(cee::vis::GLenum target, cee::vis::GLuint buffer)
{
    m_qtFuncs->glBindBuffer(target, buffer);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glViewport(cee::vis::GLint x, cee::vis::GLint y, cee::vis::GLsizei width, cee::vis::GLsizei height)
{
    m_qtFuncs->glViewport(x, y, width, height);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glClearColor(cee::vis::GLclampf red, cee::vis::GLclampf green, cee::vis::GLclampf blue, cee::vis::GLclampf alpha)
{
    m_qtFuncs->glClearColor(red, green, blue, alpha);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glClearDepth(GLclampd depth)
{
    m_qtFuncs->glClearDepthf(depth);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glStencilMask(cee::vis::GLuint mask)
{
    m_qtFuncs->glStencilMask(mask);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glClearStencil(cee::vis::GLint s)
{
    m_qtFuncs->glClearStencil(s);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glScissor(cee::vis::GLint x, cee::vis::GLint y, cee::vis::GLsizei width, cee::vis::GLsizei height)
{
    m_qtFuncs->glScissor(x, y, width, height);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glClear(cee::vis::GLbitfield mask)
{
    m_qtFuncs->glClear(mask);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDrawBuffer(cee::vis::GLenum mode)
{
    Q_ASSERT(m_qtFuncs_21);
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glDrawBuffer(mode);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glDrawBuffer()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDrawBuffers(cee::vis::GLsizei n, const cee::vis::GLenum* bufs)
{
    Q_ASSERT(m_qtFuncs_21);
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glDrawBuffers(n, bufs);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glDrawBuffers()");
    }
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlendFunc(cee::vis::GLenum sfactor, cee::vis::GLenum dfactor)
{
    m_qtFuncs->glBlendFunc(sfactor, dfactor);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlendFuncSeparate(cee::vis::GLenum srcRGB, cee::vis::GLenum dstRGB, cee::vis::GLenum srcAlpha, cee::vis::GLenum dstAlpha)
{
    m_qtFuncs->glBlendFuncSeparate(srcRGB, dstRGB, srcAlpha, dstAlpha);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlendEquation(cee::vis::GLenum mode)
{
    m_qtFuncs->glBlendEquation(mode);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlendEquationSeparate(cee::vis::GLenum modeRGB, cee::vis::GLenum modeAlpha)
{
    m_qtFuncs->glBlendEquationSeparate(modeRGB, modeAlpha);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlendColor(cee::vis::GLfloat red, cee::vis::GLfloat green, cee::vis::GLfloat blue, cee::vis::GLfloat alpha)
{
    m_qtFuncs->glBlendColor(red, green, blue, alpha);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glColorMask(cee::vis::GLboolean red, cee::vis::GLboolean green, cee::vis::GLboolean blue, cee::vis::GLboolean alpha)
{
    m_qtFuncs->glColorMask(red, green, blue, alpha);
}
   
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glCullFace(cee::vis::GLenum mode)
{
    m_qtFuncs->glCullFace(mode);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glFrontFace(cee::vis::GLenum mode)
{
    m_qtFuncs->glFrontFace(mode);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glLineWidth(cee::vis::GLfloat width)
{
    m_qtFuncs->glLineWidth(width);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glPointSize(cee::vis::GLfloat size)
{
    Q_ASSERT(m_qtFuncs_21);
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glPointSize(size);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glPointSize()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glPointParameteri(cee::vis::GLenum pname, cee::vis::GLint param)
{
    Q_ASSERT(m_qtFuncs_21);
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glPointParameteri(pname, param);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glPointParameteri()");
    }
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glPolygonMode(cee::vis::GLenum face, cee::vis::GLenum mode)
{
    Q_ASSERT(m_qtFuncs_21);
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glPolygonMode(face, mode);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glPolygonMode()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glPolygonOffset(cee::vis::GLfloat factor, cee::vis::GLfloat units)
{
    m_qtFuncs->glPolygonOffset(factor, units);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glStencilFunc(cee::vis::GLenum func, cee::vis::GLint ref, cee::vis::GLuint mask)
{
    m_qtFuncs->glStencilFunc(func, ref, mask);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glStencilOp(cee::vis::GLenum sfail, cee::vis::GLenum dpfail, cee::vis::GLenum dppass)
{
    m_qtFuncs->glStencilOp(sfail, dpfail, dppass);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glActiveTexture(cee::vis::GLenum texture)
{
    m_qtFuncs->glActiveTexture(texture);
}


//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glTexImage2D(cee::vis::GLenum target, cee::vis::GLint level, cee::vis::GLint internalFormat, cee::vis::GLsizei width, cee::vis::GLsizei height, cee::vis::GLint border, cee::vis::GLenum format, cee::vis::GLenum type, const cee::vis::GLvoid* data)
{
    m_qtFuncs->glTexImage2D(target, level, internalFormat, width, height,border, format, type, data);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGenerateMipmap(cee::vis::GLenum target)
{
    m_qtFuncs->glGenerateMipmap(target);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glTexParameteri(cee::vis::GLenum target, cee::vis::GLenum pname, cee::vis::GLint param)
{
    m_qtFuncs->glTexParameteri(target, pname, param);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glTexParameterfv(cee::vis::GLenum target, cee::vis::GLenum pname, const cee::vis::GLfloat * params)
{
    m_qtFuncs->glTexParameterfv(target, pname, params);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glTexImage2DMultisample(cee::vis::GLenum target, cee::vis::GLsizei samples, cee::vis::GLint internalformat, cee::vis::GLsizei width, cee::vis::GLsizei height, cee::vis::GLboolean fixedsamplelocations)
{
    Q_ASSERT(m_qtFuncs_32c);
    if (m_qtFuncs_32c)
    {
        m_qtFuncs_32c->glTexImage2DMultisample(target, samples, internalformat, width, height, fixedsamplelocations);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glTexImage2DMultisample()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBindTexture(cee::vis::GLenum target, cee::vis::GLuint texture)
{
    m_qtFuncs->glBindTexture(target, texture);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glPixelStorei(cee::vis::GLenum pname, cee::vis::GLint param)
{
    m_qtFuncs->glPixelStorei(pname, param);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glVertexAttribPointer(cee::vis::GLuint index, cee::vis::GLint size, cee::vis::GLenum type, cee::vis::GLboolean normalized, cee::vis::GLsizei stride, const cee::vis::GLvoid* pointer)
{
    m_qtFuncs->glVertexAttribPointer(index, size, type, normalized, stride, pointer);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glEnableVertexAttribArray(cee::vis::GLuint index)
{
    m_qtFuncs->glEnableVertexAttribArray(index);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDisableVertexAttribArray(cee::vis::GLuint index)
{
    m_qtFuncs->glDisableVertexAttribArray(index);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDrawArrays(cee::vis::GLenum mode, cee::vis::GLint first, cee::vis::GLsizei count)
{
    m_qtFuncs->glDrawArrays(mode, first, count);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDrawElements(cee::vis::GLenum mode, cee::vis::GLsizei count, cee::vis::GLenum type, const cee::vis::GLvoid* indices)
{
    m_qtFuncs->glDrawElements(mode, count, type, indices);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDrawRangeElements(cee::vis::GLenum mode, cee::vis::GLuint start, cee::vis::GLuint end, cee::vis::GLsizei count, cee::vis::GLenum type, const cee::vis::GLvoid* indices)
{
    if (m_qtFuncs_21)
    {
        m_qtFuncs_21->glDrawRangeElements(mode, start, end, count, type, indices);
    }
    else
    {
        m_qtFuncs->glDrawElements(mode, count, type, indices);
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform1f(cee::vis::GLint location, cee::vis::GLfloat v0)
{
    m_qtFuncs->glUniform1f(location, v0);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform1i(cee::vis::GLint location, cee::vis::GLint v0)
{
    m_qtFuncs->glUniform1i(location, v0);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform1fv(cee::vis::GLint location, cee::vis::GLsizei count, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniform1fv(location, count, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform1iv(cee::vis::GLint location, cee::vis::GLsizei count, const cee::vis::GLint* value)
{
    m_qtFuncs->glUniform1iv(location, count, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform2fv(cee::vis::GLint location, cee::vis::GLsizei count, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniform2fv(location, count, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform3fv(cee::vis::GLint location, cee::vis::GLsizei count, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniform3fv(location, count, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniform4fv(cee::vis::GLint location, cee::vis::GLsizei count, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniform4fv(location, count, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniformMatrix3fv(cee::vis::GLint location, cee::vis::GLsizei count, cee::vis::GLboolean transpose, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniformMatrix3fv(location, count, transpose, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUniformMatrix4fv(cee::vis::GLint location, cee::vis::GLsizei count, cee::vis::GLboolean transpose, const cee::vis::GLfloat* value)
{
    m_qtFuncs->glUniformMatrix4fv(location, count, transpose, value);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGenBuffers(cee::vis::GLsizei n, cee::vis::GLuint* buffers)
{
    m_qtFuncs->glGenBuffers(n, buffers);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteBuffers(cee::vis::GLsizei n, const cee::vis::GLuint* buffers)
{
    m_qtFuncs->glDeleteBuffers(n, buffers);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBufferData(cee::vis::GLenum target, cee::vis::GLsizeiptr size, const cee::vis::GLvoid* data, cee::vis::GLenum usage)
{
    m_qtFuncs->glBufferData(target, size, data, usage);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLuint OpenGLFunctionsBackend_qt53_deprecated::glCreateShader(cee::vis::GLenum shaderType)
{
    return m_qtFuncs->glCreateShader(shaderType);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteShader(cee::vis::GLuint shader)
{
    m_qtFuncs->glDeleteShader(shader);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glShaderSource(cee::vis::GLuint shader, cee::vis::GLsizei count, const GLchar** string, const cee::vis::GLint* length)
{
    m_qtFuncs->glShaderSource(shader, count, string, length);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glCompileShader(cee::vis::GLuint shader)
{
    m_qtFuncs->glCompileShader(shader);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLboolean OpenGLFunctionsBackend_qt53_deprecated::glIsShader(cee::vis::GLuint shader)
{
    return m_qtFuncs->glIsShader(shader);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetShaderInfoLog(cee::vis::GLuint shader, cee::vis::GLsizei maxLength, cee::vis::GLsizei* length, GLchar* infoLog)
{
    m_qtFuncs->glGetShaderInfoLog(shader, maxLength, length, infoLog);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetShaderiv(cee::vis::GLuint shader, cee::vis::GLenum pname, cee::vis::GLint* params)
{
    m_qtFuncs->glGetShaderiv(shader, pname, params);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLuint OpenGLFunctionsBackend_qt53_deprecated::glCreateProgram()
{
    return m_qtFuncs->glCreateProgram();
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteProgram(cee::vis::GLuint program)
{
    m_qtFuncs->glDeleteProgram(program);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glAttachShader(cee::vis::GLuint program, cee::vis::GLuint shader)
{
    m_qtFuncs->glAttachShader(program, shader);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glLinkProgram(cee::vis::GLuint program)
{
    m_qtFuncs->glLinkProgram(program);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glValidateProgram(cee::vis::GLuint program)
{
    m_qtFuncs->glValidateProgram(program);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLboolean OpenGLFunctionsBackend_qt53_deprecated::glIsProgram(cee::vis::GLuint program)
{
    return m_qtFuncs->glIsProgram(program);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glUseProgram(cee::vis::GLuint program)
{
    m_qtFuncs->glUseProgram(program);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetProgramInfoLog(cee::vis::GLuint program, cee::vis::GLsizei maxLength, cee::vis::GLsizei* length, GLchar* infoLog)
{
    m_qtFuncs->glGetProgramInfoLog(program, maxLength, length, infoLog);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetProgramiv(cee::vis::GLuint program, cee::vis::GLenum pname, cee::vis::GLint* params)
{
    m_qtFuncs->glGetProgramiv(program, pname, params);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLint OpenGLFunctionsBackend_qt53_deprecated::glGetUniformLocation(cee::vis::GLuint program, const GLchar* name)
{
    return m_qtFuncs->glGetUniformLocation(program, name);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetActiveUniform(cee::vis::GLuint program, cee::vis::GLuint index, cee::vis::GLsizei bufSize, cee::vis::GLsizei* length, cee::vis::GLint* size, cee::vis::GLenum* type, GLchar* name)
{
    m_qtFuncs->glGetActiveUniform(program, index, bufSize, length, size, type, name);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLint OpenGLFunctionsBackend_qt53_deprecated::glGetAttribLocation(cee::vis::GLuint program, const GLchar* name)
{
    return m_qtFuncs->glGetAttribLocation(program, name);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBindAttribLocation(cee::vis::GLuint program, cee::vis::GLuint index, const GLchar* name)
{
    m_qtFuncs->glBindAttribLocation(program, index, name);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGenTextures(cee::vis::GLsizei n, cee::vis::GLuint* textures)
{
    m_qtFuncs->glGenTextures(n, textures);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteTextures(cee::vis::GLsizei n, const cee::vis::GLuint* textures)
{
    m_qtFuncs->glDeleteTextures(n, textures);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLenum OpenGLFunctionsBackend_qt53_deprecated::glGetError()
{
    return m_qtFuncs->glGetError();
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLboolean OpenGLFunctionsBackend_qt53_deprecated::glIsEnabled(cee::vis::GLenum cap)
{
    return m_qtFuncs->glIsEnabled(cap);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGetIntegerv(cee::vis::GLenum pname, cee::vis::GLint* params)
{
    m_qtFuncs->glGetIntegerv(pname, params);
}
    
//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGenRenderbuffers(cee::vis::GLsizei n, cee::vis::GLuint* renderbuffers)
{
    m_qtFuncs->glGenRenderbuffers(n, renderbuffers);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteRenderbuffers(cee::vis::GLsizei n, cee::vis::GLuint* renderbuffers)
{
    m_qtFuncs->glDeleteRenderbuffers(n, renderbuffers);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBindRenderbuffer(cee::vis::GLenum target, cee::vis::GLuint renderbuffer)
{
    m_qtFuncs->glBindRenderbuffer(target, renderbuffer);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glRenderbufferStorage(cee::vis::GLenum target, cee::vis::GLenum internalformat, cee::vis::GLsizei width, cee::vis::GLsizei height)
{
    m_qtFuncs->glRenderbufferStorage(target, internalformat, width, height);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glRenderbufferStorageMultisample(cee::vis::GLenum target, cee::vis::GLsizei samples, cee::vis::GLenum internalformat, cee::vis::GLsizei width, cee::vis::GLsizei height)
{    
    Q_ASSERT(m_qtFuncs_30);
    if (m_qtFuncs_30)
    {
        m_qtFuncs_30->glRenderbufferStorageMultisample(target, samples, internalformat, width, height);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glRenderbufferStorageMultisample()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glGenFramebuffers(cee::vis::GLsizei n, cee::vis::GLuint* ids)
{
    m_qtFuncs->glGenFramebuffers(n, ids);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glDeleteFramebuffers(cee::vis::GLsizei n, cee::vis::GLuint* framebuffers)
{
    m_qtFuncs->glDeleteFramebuffers(n, framebuffers);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glFramebufferRenderbuffer(cee::vis::GLenum target, cee::vis::GLenum attachment, cee::vis::GLenum renderbuffertarget, cee::vis::GLuint renderbuffer)
{
    m_qtFuncs->glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glFramebufferTexture2D(cee::vis::GLenum target, cee::vis::GLenum attachment, cee::vis::GLenum textarget, cee::vis::GLuint texture, cee::vis::GLint level)
{
    m_qtFuncs->glFramebufferTexture2D(target, attachment, textarget, texture, level);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBindFramebuffer(cee::vis::GLenum target, cee::vis::GLuint framebuffer)
{
    m_qtFuncs->glBindFramebuffer(target, framebuffer);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
cee::vis::GLenum OpenGLFunctionsBackend_qt53_deprecated::glCheckFramebufferStatus(cee::vis::GLenum target)
{
    return m_qtFuncs->glCheckFramebufferStatus(target);
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glBlitFramebuffer(cee::vis::GLint srcX0, cee::vis::GLint srcY0, cee::vis::GLint srcX1, cee::vis::GLint srcY1, cee::vis::GLint dstX0, cee::vis::GLint dstY0, cee::vis::GLint dstX1, cee::vis::GLint dstY1, cee::vis::GLbitfield mask, cee::vis::GLenum filter)
{
    Q_ASSERT(m_qtFuncs_30);
    if (m_qtFuncs_30)
    {
        m_qtFuncs_30->glBlitFramebuffer(srcX0, srcY0, srcX1, srcY1, dstX0, dstY0, dstX1, dstY1, mask, filter);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glBlitFramebuffer()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glVertexAttribIPointer(cee::vis::GLuint index, cee::vis::GLint size, cee::vis::GLenum type, cee::vis::GLsizei stride, const cee::vis::GLvoid* pointer)
{
    Q_ASSERT(m_qtFuncs_30);
    if (m_qtFuncs_30)
    {
        m_qtFuncs_30->glVertexAttribIPointer(index, size, type, stride, pointer);
    }
    else
    {
        m_logMgr->logDebug(m_loggerName, "OpenGL function not available: glVertexAttribIPointer()");
    }
}

//--------------------------------------------------------------------------------------------------
///
//--------------------------------------------------------------------------------------------------
void OpenGLFunctionsBackend_qt53_deprecated::glReadPixels(cee::vis::GLint x, cee::vis::GLint y, cee::vis::GLsizei width, cee::vis::GLsizei height, cee::vis::GLenum format, cee::vis::GLenum type, cee::vis::GLvoid* data)
{
    m_qtFuncs->glReadPixels(x, y, width, height, format, type, data);
}



}
}


