//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Color3f.h"
#include "CeeCore/Str.h"
#include "CeeCore/AppLogging.h"

#include <QString>
#include <QVector>

namespace cee {
namespace qt {



//==================================================================================================
//
// Static helper class for interop between Ceetron Desktop Components and Qt 
//
//==================================================================================================
class UtilsCore
{
public:
    //--------------------------------------------------------------------------------------------------
    /// 
    //--------------------------------------------------------------------------------------------------
    static QString toQString(const Str& str)
    {
        if (str.isEmpty())
        {
            return "";
        }

        if (sizeof(wchar_t) == 2)
        {
            const unsigned short* strPtr = reinterpret_cast<const unsigned short*>(str.unspecifiedWCharPointer());
            return QString::fromUtf16(strPtr);
        }
        else if (sizeof(wchar_t) == 4)
        {
            const unsigned int* strPtr = reinterpret_cast<const unsigned int*>(str.unspecifiedWCharPointer());
            return QString::fromUcs4(strPtr);
        }

        return QString("Unexpected sizeof wchar_t");
    }

    //--------------------------------------------------------------------------------------------------
    /// 
    //--------------------------------------------------------------------------------------------------
    static Str toStr(const QString& qtString)
    {
        if (qtString.length() == 0)
        {
            return Str();
        }

        if (sizeof(wchar_t) == 2)
        {
            const wchar_t* strPtr = reinterpret_cast<const wchar_t*>(qtString.utf16());
            return Str::fromUnspecifiedWCharPointer(strPtr);
        }
        else if (sizeof(wchar_t) == 4)
        {
            QVector<uint> ucs4Str = qtString.toUcs4();
            ucs4Str.push_back(0);
            const wchar_t* strPtr = reinterpret_cast<const wchar_t*>(ucs4Str.data());
            return Str::fromUnspecifiedWCharPointer(strPtr);
        }
        return Str("Unexpected sizeof wchar_t");
    }
};

} // namespace qt
} // namespace cee



// Helper macros that allow passing QStrings directly to the app log
#define CEE_LOG_APP_ERROR_QSTR(theMessage)        CEE_LOG_APP_ERROR(cee::qt::UtilsCore::toStr(theMessage))  
#define CEE_LOG_APP_WARNING_QSTR(theMessage)      CEE_LOG_APP_WARNING(cee::qt::UtilsCore::toStr(theMessage))  
#define CEE_LOG_APP_INFO_QSTR(theMessage)         CEE_LOG_APP_INFO(cee::qt::UtilsCore::toStr(theMessage))  
#define CEE_LOG_APP_DEBUG_QSTR(theMessage)        CEE_LOG_APP_DEBUG(cee::qt::UtilsCore::toStr(theMessage))  
