//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "ceeqtViewerGLWidgetDemo.h"
#include "ceeqtUtilsGui.h"

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"
#include "CeeVisualization/OpenGLContextGroup.h"

namespace cee {
namespace qt {



//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Creates a Qt viewer widget. Use this for the first or only viewer widget in your application.
//--------------------------------------------------------------------------------------------------
ViewerGLWidgetDemo::ViewerGLWidgetDemo(cee::vis::OpenGLContextGroup* contextGroup, QWidget* parent, const QGLFormat& format)
:   ViewerGLWidget(contextGroup, parent, format)
{
}


//--------------------------------------------------------------------------------------------------
/// Creates a Qt viewer widget sharing the OpenGL resources with the specified widget.
/// 
/// This is useful when doing multiple windows with their own OpenGL widget. By specifying the 
/// shareWidget, all OpenGL resources can be shared between the widgets and thus the same Models
/// can be shown in multiple views at the same time.
//--------------------------------------------------------------------------------------------------
ViewerGLWidgetDemo::ViewerGLWidgetDemo(cee::vis::OpenGLContextGroup* contextGroup, QWidget* parent, ViewerGLWidgetDemo* shareWidget)
:   ViewerGLWidget(contextGroup, parent, shareWidget)
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ViewerGLWidgetDemo::~ViewerGLWidgetDemo()
{
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse move events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerGLWidgetDemo::mouseMoveEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        if (inputHandler->mouseMoveEvent(me))
        {
            update();
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse press events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerGLWidgetDemo::mousePressEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonPressed = UtilsGui::toMouseButton(event->button());
        
        inputHandler->mousePressEvent(buttonPressed, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse release events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerGLWidgetDemo::mouseReleaseEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonReleased = UtilsGui::toMouseButton(event->button());
        
        inputHandler->mouseReleaseEvent(buttonReleased, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse wheel events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerGLWidgetDemo::wheelEvent(QWheelEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::WheelEvent wEvent = UtilsGui::toWheelEvent(*this, *event);
        
        inputHandler->wheelEvent(wEvent);
        update();
    }
}


} // namespace qt
} // namespace cee



