//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "ceeqtViewerGraphicsViewAdapter.h"

#include <QGraphicsView>
#include <QGLWidget>

namespace cee {
namespace qt {



//==================================================================================================
//
// Helper class for using Ceetron Desktop Components in a QGraphicsView/QGraphicsScene environment.
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ViewerGraphicsViewAdapter::ViewerGraphicsViewAdapter(vis::OpenGLContextGroup* contextGroup, QGraphicsView* graphicsView, QGLWidget* viewportGLWidget)
:   vis::ViewerUIFramework(contextGroup)
{
    Q_ASSERT(contextGroup);
    Q_ASSERT(graphicsView);
    Q_ASSERT(viewportGLWidget);
    Q_ASSERT(viewportGLWidget == graphicsView->viewport());

    m_graphicsView = graphicsView;
    m_viewportGLWidget = viewportGLWidget;

    m_viewportGLWidget->makeCurrent();
    initializeComponentInternalOpenGL();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ViewerGraphicsViewAdapter::~ViewerGraphicsViewAdapter()
{
    Q_ASSERT(m_viewportGLWidget);
    m_viewportGLWidget->makeCurrent();
    shutdownComponentInternalOpenGL();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void ViewerGraphicsViewAdapter::doRequestRedraw()
{
    Q_ASSERT(m_graphicsView);
    m_graphicsView->update();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void ViewerGraphicsViewAdapter::doRequestImmediateRedraw()
{
    Q_ASSERT(m_graphicsView);
    m_graphicsView->repaint();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool ViewerGraphicsViewAdapter::doMakeCurrent()
{
    Q_ASSERT(m_viewportGLWidget);
    m_viewportGLWidget->makeCurrent();
    return true;
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
bool ViewerGraphicsViewAdapter::doIsCurrent()
{
    Q_ASSERT(m_viewportGLWidget);
    if (QGLContext::currentContext() == m_viewportGLWidget->context())
    {
        return true;
    }
    else
    {
        return false;
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
unsigned int ViewerGraphicsViewAdapter::doGetDefaultFramebufferObject()
{
    return 0;
}

} // namespace qt
} // namespace cee

