//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/ViewerUIFramework.h"
#include "CeeVisualization/View.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/LogManager.h"

#include <QOpenGLWidget>

namespace cee {
namespace qt {



//==================================================================================================
//
// 
//
//==================================================================================================
class ViewerOpenGLWidget : public QOpenGLWidget, public vis::ViewerUIFramework
{
public:
    ViewerOpenGLWidget(vis::OpenGLContextGroup* contextGroup, QWidget* parent);
    virtual ~ViewerOpenGLWidget();

    void            setView(vis::View* aView);
    vis::View*      view();

    bool            isViewerInitialized() const;

    void            logOpenGLInfo();

protected:
    virtual void    initializeGL();
    virtual void    resizeGL(int width, int height);
    virtual void    paintGL();

private:
    virtual void    onViewerOpenGLReady();

private:
    virtual void            doRequestRedraw();
    virtual void            doRequestImmediateRedraw();
    virtual bool            doMakeCurrent();
    virtual bool            doIsCurrent();
    virtual unsigned int    doGetDefaultFramebufferObject();

    void                    qtOpenGLContextAboutToBeDestroyed();

private:
    enum InitializationState
    {
        UNINITIALIZED,
        PENDING_REINITIALIZATION,
        IS_INITIALIZED
    };

    InitializationState     m_viewerInitializationState;
    PtrRef<vis::View>       m_view;
    const cee::Str          m_loggerName;
    PtrRef<cee::LogManager> m_logMgr;
    int                     m_instanceNumber;
};


}
}
