//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for Qt
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "ceeqtViewerOpenGLWidgetDemo.h"
#include "ceeqtUtilsGui.h"

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"
#include "CeeVisualization/OpenGLContextGroup.h"

namespace cee {
namespace qt {



//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// Creates a Qt viewer widget. 
//--------------------------------------------------------------------------------------------------
ViewerOpenGLWidgetDemo::ViewerOpenGLWidgetDemo(cee::vis::OpenGLContextGroup* contextGroup, QWidget* parent, InitializationObserver* initializationObserver)
:   ViewerOpenGLWidget(contextGroup, parent),
    m_initializationObserver(initializationObserver)
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ViewerOpenGLWidgetDemo::~ViewerOpenGLWidgetDemo()
{
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse move events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerOpenGLWidgetDemo::mouseMoveEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        if (inputHandler->mouseMoveEvent(me))
        {
            update();
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse press events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerOpenGLWidgetDemo::mousePressEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonPressed = UtilsGui::toMouseButton(event->button());
        
        inputHandler->mousePressEvent(buttonPressed, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse release events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerOpenGLWidgetDemo::mouseReleaseEvent(QMouseEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonReleased = UtilsGui::toMouseButton(event->button());
        
        inputHandler->mouseReleaseEvent(buttonReleased, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// Virtual function called on mouse wheel events.
/// 
/// Translate event and passes it to the view's input manager.
//--------------------------------------------------------------------------------------------------
void ViewerOpenGLWidgetDemo::wheelEvent(QWheelEvent* event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::WheelEvent wEvent = UtilsGui::toWheelEvent(*this, *event);
        
        inputHandler->wheelEvent(wEvent);
        update();
    }
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void ViewerOpenGLWidgetDemo::onViewerOpenGLReady()
{
    logOpenGLInfo();

    if (m_initializationObserver)
    {
        m_initializationObserver->onViewerOpenGLReady(this);
    }
}

} // namespace qt
} // namespace cee



