//##################################################################################################
//
//  Ceetron Desktop Components
//  Platform integration for WxWidgets
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "ceewxViewerCanvasDemo.h"

#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

namespace cee {
namespace wx {




//==================================================================================================
//
// 
//
//==================================================================================================

BEGIN_EVENT_TABLE(ViewerCanvasDemo, ViewerCanvas)
    EVT_MOTION(ViewerCanvasDemo::onMouseMove)
END_EVENT_TABLE()

//--------------------------------------------------------------------------------------------------
/// Creates a viewer canvas. 
/// Use this constructor for the first or only ViewerCanvas in your application.
//--------------------------------------------------------------------------------------------------
ViewerCanvasDemo::ViewerCanvasDemo(cee::vis::OpenGLContextGroup* contextGroup, wxWindow* parent, int* attribList)
:   ViewerCanvas(contextGroup, parent, attribList)
{
}


//--------------------------------------------------------------------------------------------------
/// Creates a viewer sharing the OpenGL resources with the specified viewer canvas.
/// 
/// By specifying the shareCanvas, all OpenGL resources can be shared between the viewers and thus 
/// the same Models can be shown in multiple views at the same time.
//--------------------------------------------------------------------------------------------------
ViewerCanvasDemo::ViewerCanvasDemo(cee::vis::OpenGLContextGroup* contextGroup, wxWindow* parent, int* attribList, ViewerCanvas* shareCanvas)
:   ViewerCanvas(contextGroup, parent, attribList, shareCanvas)
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
ViewerCanvasDemo::~ViewerCanvasDemo()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void ViewerCanvasDemo::onMouseMove(wxMouseEvent& event)
{
    vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = toMouseEvent(event);
        if (inputHandler->mouseMoveEvent(me))
        {
            Refresh();
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::MouseEvent ViewerCanvasDemo::toMouseEvent(const wxMouseEvent& event)
{
    const int clientHeight = GetClientSize().GetY();
    const int xPos = event.GetX();
    const int yPos = clientHeight - event.GetY();

    cee::vis::MouseButtons buttonsDown = cee::vis::NoButton;
    if (event.LeftIsDown())     buttonsDown |= cee::vis::LeftButton;
    if (event.MiddleIsDown())   buttonsDown |= cee::vis::MiddleButton;
    if (event.RightIsDown())    buttonsDown |= cee::vis::RightButton;

    return cee::vis::MouseEvent(xPos, yPos, buttonsDown, cee::vis::NoModifier);
}


} // namespace wx
} // namespace gc

