//##################################################################################################
//
//  Ceetron Desktop Components
//  Tutorials
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "QtTutorialRunner/TutorialRunnerMainWindow.h"

#include "TutorialUtils.h"

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "CeeUnstructGrid/ModelSpec.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/DataSourceVTFx.h"
#include "CeeUnstructGrid/PartSettingsIterator.h"
#include "CeeUnstructGrid/PartSettings.h"

#include <QTimer>



//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::setupAnimation()
{
    //--------------------------------------------------------------------------
    // Load a UnstructGridModel from file
    // Apply some part settings
    //--------------------------------------------------------------------------

    // Create the model and data source
    cee::PtrRef<cee::ug::UnstructGridModel> ugModel = new cee::ug::UnstructGridModel();
    cee::PtrRef<cee::ug::DataSourceVTFx> source = new cee::ug::DataSourceVTFx(42);

    // Open the vtfx file
    cee::Str vtfxFile = TutorialUtils::testDataDir() + "contact.vtfx";
    if (!source->open(vtfxFile, 0))
    {
        // VTFx file not found
        return;
    }

    // Add the data source to the model
    ugModel->setDataSource(source.get());

    // Set all states as current
    std::vector<cee::ug::StateInfo> stateInfos = source->directory()->stateInfos();
    std::vector<int> stateIds;
    for (size_t i = 0; i < stateInfos.size(); i++)
    {
        int stateId = stateInfos[i].id();
        stateIds.push_back(stateId);
    }

    ugModel->modelSpec().setStateIds(stateIds);

    // Show first displacement result on the model
    std::vector<cee::ug::ResultInfo> dispResultInfos = source->directory()->displacementResultInfos();
    if (dispResultInfos.size() > 0)
    {
        int dispId = dispResultInfos[0].id();
        ugModel->modelSpec().setDisplacementResultId(dispId);
    }

    // Set result visibility for all parts
    cee::ug::PartSettingsIterator it(ugModel.get());
    while (it.hasNext())
    {
        cee::ug::PartSettings* partSettings = it.next();
        partSettings->setDisplacementVisible(true);
    } 

    // Add model to view. Ensure that old models are removed first
    cee::vis::View* gcView = getTutorialView();
    gcView->removeAllModels();
    gcView->addModel(ugModel.get());

    ugModel->updateVisualization();

    cee::BoundingBox bb = gcView->boundingBox();
    gcView->camera().fitView(bb, cee::Vec3d(0, 0, -1), cee::Vec3d(0, 1, 0));
    gcView->camera().inputHandler()->setRotationPoint(bb.center());

    ugModel->setCurrentFrameIndex(0);
    m_animationRunning = true;
    update3dView();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::update3dView()
{
    cee::ug::UnstructGridModel* ugModel = dynamic_cast<cee::ug::UnstructGridModel*>(getTutorialView()->model(0));
    if (ugModel == NULL)
    {
        return;
    }

    if (m_animationRunning)
    {
        size_t newIndex = ugModel->currentFrameIndex() + 1;
        if (newIndex >= ugModel->frameCount())
        {
            newIndex = 0;
        }

        ugModel->setCurrentFrameIndex(newIndex);

        getTutorialView()->requestRedraw();

        QTimer::singleShot(100, this, SLOT(update3dView()));
    }
}
