//##################################################################################################
//
//  Ceetron Desktop Components
//  Tutorials
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#if MFC_TUTORIAL
#include "MfcTutorialRunner/TutorialRunnerMainWindow.h"
#elif WX_TUTORIAL
#include "WxTutorialRunner/TutorialRunnerMainWindow.h"
#else
#include "QtTutorialRunner/TutorialRunnerMainWindow.h"
#endif

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "CeeGeometry/GeometryModel.h"
#include "CeeGeometry/DataGenerator.h"
#include "CeeGeometry/Part.h"
#include "CeeGeometry/PartSettings.h"
#include "CeeGeometry/EffectHalo.h"
#include "CeeGeometry/HitItem.h"



//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::highlighting()
{
    cee::vis::View* gcView = getTutorialView();

    cee::PtrRef<cee::geo::GeometryModel> geometryModel = new cee::geo::GeometryModel;
    gcView->setModelOpenGLCapabilitiesFromView(geometryModel.get());


    // Add four boxes
    geometryModel->addPart(new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(1.0, 1.0, 1.0), cee::Vec3d(1.0, 1.0, 1.0)).get()));
    geometryModel->addPart(new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(3.0, 1.0, 1.0), cee::Vec3d(1.0, 1.0, 1.0)).get()));
    geometryModel->addPart(new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(1.0, 3.0, 1.0), cee::Vec3d(1.0, 1.0, 1.0)).get()));
    geometryModel->addPart(new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(3.0, 3.0, 1.0), cee::Vec3d(1.0, 1.0, 1.0)).get()));

    // Add model to view. Ensure that old models are removed first
    gcView->removeAllModels();
    gcView->addModel(geometryModel.get());

    cee::BoundingBox bb = gcView->boundingBox();
    gcView->camera().fitView(bb, cee::Vec3d(0, 0, -1), cee::Vec3d(0, 1, 0));
    gcView->camera().inputHandler()->setRotationPoint(bb.center());

    gcView->requestRedraw();
}

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::pick(const cee::Ray& ray)
{
    cee::vis::View* gcView = getTutorialView();

    // Get the current model as a geometry model
    cee::geo::GeometryModel* geometryModel = dynamic_cast<cee::geo::GeometryModel*>(gcView->model(0));
    if (!geometryModel)
    {
        return;
    }

    // Do the picking
    cee::geo::HitItem hitItem;
    if (!geometryModel->rayIntersect(ray, *gcView, &hitItem))
    {
        // No parts were hit, return without doing any changes
        return;
    }

    // Loop all parts. Toggle off highlighting for all parts except the currently hit
    for (size_t i = 0; i < geometryModel->partCount(); i++)
    {
        cee::geo::Part* part = geometryModel->part(i);

        part->settings().removeAllEffectsOfType<cee::geo::EffectHalo>();

        if (part == hitItem.part())
        {
            part->settings().addEffect(new cee::geo::EffectHalo);
        }
    }

    gcView->requestRedraw();
}
