//##################################################################################################
//
//  Ceetron Desktop Components
//  QtTutorialRunner
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "TutorialRunnerMainWindow.h"
#include "TutorialRunnerViewer.h"

#include "CeeVisualization/VisualizationComponent.h"
#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "CeeUnstructGrid/DataPart.h"
#include "CeeUnstructGrid/DataState.h"

#include "CeeGeometry/GeometryModel.h"
#include "CeeGeometry/DataGenerator.h"
#include "CeeGeometry/Part.h"
#include "CeeGeometry/PartSettings.h"
#include "CeeGeometry/EffectColor.h"

#include <QFrame>
#include <QHBoxLayout>
#include <QMouseEvent>
#include <QMenuBar>
#include <QMenu>




//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TutorialRunnerMainWindow::TutorialRunnerMainWindow(cee::vis::OpenGLContextGroup* contextGroup)
:   m_animationRunning(false)
{
    QFrame* mainFrame = new QFrame;
    QHBoxLayout* frameLayout = new QHBoxLayout(mainFrame);
    setCentralWidget(mainFrame);

    m_viewer = new TutorialRunnerViewer(contextGroup, mainFrame, this);
    m_viewer->setFocus();
    frameLayout->addWidget(m_viewer);

    cee::vis::View* gcView = new cee::vis::View;
    m_viewer->setView(gcView);

    // Show a dummy model on start up
    cee::geo::GeometryModel* geometryModel = new cee::geo::GeometryModel;
    gcView->addModel(geometryModel);

    // Add a box
    cee::PtrRef<cee::geo::Part> boxPart = new cee::geo::Part(cee::geo::DataGenerator::createBox(cee::Vec3d(1.0, 1.0, 1.0), cee::Vec3d(2.0, 2.0, 2.0)).get());
    boxPart->settings().addEffect(new cee::geo::EffectColor(cee::Color3f(1,1,0)));
    geometryModel->addPart(boxPart.get());

    // Add a sphere
    cee::PtrRef<cee::geo::Part> spherePart = new cee::geo::Part(cee::geo::DataGenerator::createSphere(cee::Vec3d(4.0, 2.0, 1.0), 1.0, 16).get());
    geometryModel->addPart(spherePart.get());

    cee::BoundingBox bb = gcView->boundingBox();
    gcView->camera().fitView(bb, cee::Vec3d(0, 0, -1), cee::Vec3d(0, 1, 0));
    gcView->camera().inputHandler()->setRotationPoint(bb.center());

    QMenu* menu = menuBar()->addMenu("Tutorials");
    menu->addAction("UnstructGridModel: Create two triangles",         this, SLOT(createTriangleGeometry()));
    menu->addAction("UnstructGridModel: Create triangle with results", this, SLOT(createTriangleGeometryWithAllResults()));
    menu->addAction("Load VTFx file",               this, SLOT(loadVtfAndSetResults()));
    menu->addAction("Scalar settings",              this, SLOT(setScalarSettings()));
    menu->addAction("Vector settings",              this, SLOT(setVectorSettings()));
    menu->addAction("Part settings",                this, SLOT(setPartSettings()));
    menu->addAction("Cutting plane",                this, SLOT(addCuttingPlane()));
    menu->addAction("Isosurface",                   this, SLOT(addIsosurface()));
    menu->addAction("Isovolume",                    this, SLOT(addIsovolume()));
    menu->addAction("Highlighting",                 this, SLOT(highlighting()));
    menu->addAction("Animation",                    this, SLOT(setupAnimation()));
    menu->addAction("MarkupModel",                  this, SLOT(createMarkup()));
    menu->addAction("GeometryModel",                this, SLOT(createGeometryModel()));
    menu->addAction("GeometryModel: Texture",       this, SLOT(createGeometryModelWithTexture()));
    menu->addAction("GeometryModel: Primitives",    this, SLOT(createGeometryModelWithPrimitives()));
    menu->addAction("Custom navigation",            this, SLOT(customNavigation()));
    menu->addAction("Custom DataReader",            this, SLOT(createCustomDataReader()));
    menu->addAction("UnstructGridModel: DataElementSets", this, SLOT(addDataElementSets()));
    menu->addAction("Report",                       this, SLOT(createReport()));
    menu->addAction("Plot2d",                       this, SLOT(createPlot()));
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TutorialRunnerMainWindow::~TutorialRunnerMainWindow()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::closeEvent(QCloseEvent* /*event*/)
{
    if (!m_viewer) return;

    m_animationRunning = false;

    m_viewer->setView(NULL);
    delete m_viewer;
    m_viewer = NULL;
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::View* TutorialRunnerMainWindow::getTutorialView()
{
    return m_viewer->view();
}


