//##################################################################################################
//
//  Ceetron Desktop Components
//  QtTutorialRunner
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include <QMainWindow>

#include "CeeCore/PtrRef.h"
#include "CeeCore/Ray.h"
#include "CeeVisualization/View.h"
#include "CeeVisualization/OpenGLContextGroup.h"

class TutorialRunnerViewer;



//==================================================================================================
//
// 
//
//==================================================================================================
class TutorialRunnerMainWindow : public QMainWindow
{
    Q_OBJECT

public:
    TutorialRunnerMainWindow(cee::vis::OpenGLContextGroup* contextGroup);
    virtual ~TutorialRunnerMainWindow();

    void    pick(const cee::Ray& ray);

protected slots:
    void    createTriangleGeometry();
    void    createTriangleGeometryWithAllResults();
    void    loadVtfAndSetResults();
    void    setVectorSettings();
    void    setScalarSettings();
    void    setPartSettings();
    void    addCuttingPlane();
    void    addIsosurface();
    void    addIsovolume();
    void    highlighting();
    void    setupAnimation();
    void    createMarkup();
    void    createGeometryModel();
    void    createGeometryModelWithTexture();
    void    createGeometryModelWithPrimitives();
    void    customNavigation();
    void    createCustomDataReader();
    void    addDataElementSets();
    void    createReport();
    void    createPlot();

    void    update3dView();

private:
    virtual void    closeEvent(QCloseEvent* event);
    cee::vis::View* getTutorialView();
    void            snippets();

private:
    TutorialRunnerViewer*                       m_viewer;
    bool                                        m_animationRunning;
};

