//##################################################################################################
//
//  Ceetron Desktop Components
//  QtTutorialRunner
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "TutorialRunnerViewer.h"
#include "TutorialRunnerMainWindow.h"

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/MouseEvent.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "ceeqtUtilsGui.h"



//==================================================================================================
//
// 
//
//==================================================================================================

//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TutorialRunnerViewer::TutorialRunnerViewer(cee::vis::OpenGLContextGroup* contextGroup, QWidget* parent, TutorialRunnerMainWindow* mainWindow)
:   cee::qt::ViewerOpenGLWidget(contextGroup, parent),
    m_mainWindow(mainWindow)
{
}



//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewer::mouseMoveEvent(QMouseEvent* event)
{
    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = cee::qt::UtilsGui::toMouseEvent(*this, *event);
        if (inputHandler->mouseMoveEvent(me))
        {
            update();
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewer::mousePressEvent(QMouseEvent* event)
{
    cee::vis::View* theView = view();
    if (!theView) return;

    // Picking!
    cee::Ray ray = theView->camera().rayFromWindowCoordinates(event->pos().x(), height() - event->pos().y());
    m_mainWindow->pick(ray);

    cee::vis::CameraInputHandler* inputHandler = theView->camera().inputHandler();
    if (inputHandler)
    {
        cee::vis::MouseEvent me = cee::qt::UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonPressed = cee::qt::UtilsGui::toMouseButton(event->button());

        inputHandler->mousePressEvent(buttonPressed, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewer::mouseReleaseEvent(QMouseEvent* event)
{
    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = cee::qt::UtilsGui::toMouseEvent(*this, *event);
        cee::vis::MouseButton buttonReleased = cee::qt::UtilsGui::toMouseButton(event->button());

        inputHandler->mouseReleaseEvent(buttonReleased, me);
        update();
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewer::wheelEvent(QWheelEvent* event)
{
    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::WheelEvent wEvent = cee::qt::UtilsGui::toWheelEvent(*this, *event);

        inputHandler->wheelEvent(wEvent);
        update();
    }
}


