//##################################################################################################
//
//  Ceetron Desktop Components
//  Tutorials
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#if MFC_TUTORIAL
#include "MfcTutorialRunner/TutorialRunnerMainWindow.h"
#elif WX_TUTORIAL
#include "WxTutorialRunner/TutorialRunnerMainWindow.h"
#else
#include "QtTutorialRunner/TutorialRunnerMainWindow.h"
#endif

#include "TutorialUtils.h"

#include "CeeVisualization/View.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"

#include "CeeUnstructGrid/ModelSpec.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/DataSourceVTF.h"
#include "CeeUnstructGrid/PartSettingsIterator.h"
#include "CeeUnstructGrid/PartSettings.h"
#include "CeeUnstructGrid/ScalarSettings.h"
#include "CeeUnstructGrid/ColorMapper.h"



//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerMainWindow::setScalarSettings()
{
    //--------------------------------------------------------------------------
    // Load a UnstructGridModel from file and set a scalar result
    // Apply some scalar settings
    //--------------------------------------------------------------------------

    // Create the model and data source
    cee::PtrRef<cee::ug::UnstructGridModel> ugModel = new cee::ug::UnstructGridModel();
    cee::PtrRef<cee::ug::DataSourceVTF> source = new cee::ug::DataSourceVTF(42);

    // Open the vtf file
    cee::Str vtfFile = TutorialUtils::testDataDir() + "100x100x100.vtf";
    if (!source->open(vtfFile))
    {
        // VTF file not found
        return;
    }

    // Add the data source to the model
    ugModel->setDataSource(source.get());

    // Set the first state as current
    std::vector<cee::ug::StateInfo> stateInfos = source->directory()->stateInfos();
    int stateId = stateInfos[0].id();
    ugModel->modelSpec().setStateId(stateId);

    // Show the scalar result as fringes on the model
    std::vector<cee::ug::ResultInfo> scalarResultInfos = source->directory()->scalarResultInfos();
    int scalarId = scalarResultInfos[0].id();
    ugModel->modelSpec().setFringesResultId(scalarId);

    // Set fringes visible for all parts
    cee::ug::PartSettingsIterator it(ugModel.get());
    while (it.hasNext())
    {
        cee::ug::PartSettings* partSettings = it.next();
        partSettings->setFringesVisible(true);
    } 

    // Get scalar settings for the current fringes result
    cee::ug::ScalarSettings* settings = ugModel->scalarSettings(scalarId);

    // Set number of legend levels to 0 to get a continuous legend
    settings->colorMapper().setTypeFilledContoursUniform(5);

    // Set the color scheme to the reverted normal
    settings->colorMapper().setColorScheme(cee::ug::ColorMapper::NORMAL_INVERTED);

    // Use filter to only show values above 20. Toggle on filtering.
    settings->setFringesElementFilteringVisibleRange(20, 99);

    // Add model to view. Ensure that old models are removed first
    cee::vis::View* gcView = getTutorialView();
    gcView->removeAllModels();
    gcView->addModel(ugModel.get());

    ugModel->updateVisualization();

    cee::BoundingBox bb = gcView->boundingBox();
    gcView->camera().fitView(bb, cee::Vec3d(0, 0, -1), cee::Vec3d(0, 1, 0));
    gcView->camera().inputHandler()->setRotationPoint(bb.center());

    gcView->requestRedraw();
}
