//##################################################################################################
//
//  Ceetron Desktop Components
//  WxTutorialRunner
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "wx/frame.h"

#include "CeeCore/PtrRef.h"
#include "CeeCore/Ray.h"
#include "CeeVisualization/View.h"
#include "CeeVisualization/OpenGLContextGroup.h"

class TutorialRunnerViewerCanvas;



//==================================================================================================
//
// 
//
//==================================================================================================
class TutorialRunnerMainWindow : public wxFrame
{

public:
    TutorialRunnerMainWindow();
    virtual ~TutorialRunnerMainWindow();

    void    pick(const cee::Ray& ray);

private:
    void    onShow(wxShowEvent&);
    void    onClose(wxCloseEvent&);

    void    onTutorialMenuItemSelected(wxCommandEvent&);
    void    onExit(wxCommandEvent&);

private:
    cee::vis::View* getTutorialView();

    void            createTriangleGeometry();
    void            createTriangleGeometryWithAllResults();
    void            loadVtfAndSetResults();
    void            setVectorSettings();
    void            setScalarSettings();
    void            setPartSettings();
    void            addCuttingPlane();
    void            addIsosurface();
    void            addIsovolume();
    void            highlighting();
    void            setupAnimation();
    void            createMarkup();
    void            createGeometryModel();
    void            createGeometryModelWithTexture();
    void            createGeometryModelWithPrimitives();
    void            createCustomDataReader();
    void            customNavigation();
    void 			addDataElementSets();
    void            createReport();
    void            createPlot();

private:
    cee::PtrRef<cee::vis::OpenGLContextGroup>   m_contextGroup;
    TutorialRunnerViewerCanvas*                 m_viewer;

    DECLARE_EVENT_TABLE()
};

