//##################################################################################################
//
//  Ceetron Desktop Components
//  WxTutorialRunner
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#include "TutorialRunnerViewerCanvas.h"
#include "TutorialRunnerMainWindow.h"

#include "wx/wx.h"

#include "CeeVisualization/VisualizationComponent.h"
#include "CeeVisualization/Camera.h"
#include "CeeVisualization/CameraInputHandler.h"



//==================================================================================================
//
// 
//
//==================================================================================================

BEGIN_EVENT_TABLE(TutorialRunnerViewerCanvas, cee::wx::ViewerCanvas)
    EVT_MOTION(TutorialRunnerViewerCanvas::onMouseMove)
    EVT_LEFT_DOWN(TutorialRunnerViewerCanvas::onMouseDown)
    EVT_MIDDLE_DOWN(TutorialRunnerViewerCanvas::onMouseDown)
    EVT_RIGHT_DOWN(TutorialRunnerViewerCanvas::onMouseDown)
    EVT_LEFT_UP(TutorialRunnerViewerCanvas::onMouseReleased)
    EVT_MIDDLE_UP(TutorialRunnerViewerCanvas::onMouseReleased)
    EVT_RIGHT_UP(TutorialRunnerViewerCanvas::onMouseReleased)
    EVT_MOUSEWHEEL(TutorialRunnerViewerCanvas::onMouseWheelMove)
END_EVENT_TABLE()


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TutorialRunnerViewerCanvas::TutorialRunnerViewerCanvas(cee::vis::OpenGLContextGroup* contextGroup, TutorialRunnerMainWindow* parentMainWindow, int* attribList)
:   cee::wx::ViewerCanvas(contextGroup, parentMainWindow, attribList),
    m_mainWindow(parentMainWindow),
    m_cleanLeftButtonPressOccurred(false),
    m_cleanLeftButtonPressPos(0, 0)
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
TutorialRunnerViewerCanvas::~TutorialRunnerViewerCanvas()
{
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewerCanvas::onMouseDown(wxMouseEvent& event)
{
    // Detect a clean button press (button pressed without any other mouse buttons down)
    m_cleanLeftButtonPressOccurred = false;
    m_cleanLeftButtonPressPos = event.GetPosition();
    if (event.Button(wxMOUSE_BTN_LEFT) && !event.MiddleIsDown() && !event.RightIsDown())
    {
        m_cleanLeftButtonPressOccurred = true;
    }

    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = toMouseEvent(event);
        cee::vis::MouseButton buttonPressed = toMouseButton(event);
        inputHandler->mousePressEvent(buttonPressed, me);
    }

    Refresh();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewerCanvas::onMouseMove(wxMouseEvent& event)
{
    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = toMouseEvent(event);
        if (inputHandler->mouseMoveEvent(me))
        {
            Refresh();
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewerCanvas::onMouseReleased(wxMouseEvent& event)
{
    // Check if the left mouse button 'cleanly clicked' within tolerance
    const int cleanButtonClickMovementThreshold = 4;
    bool leftButtonCleanlyClicked = false;
    if (m_cleanLeftButtonPressOccurred && event.Button(wxMOUSE_BTN_LEFT))
    {
        if ((abs(event.GetX() - m_cleanLeftButtonPressPos.x) + abs(event.GetY() - m_cleanLeftButtonPressPos.y)) < cleanButtonClickMovementThreshold)
        {
            leftButtonCleanlyClicked = true;
        }
    }
    m_cleanLeftButtonPressOccurred = false;


    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::MouseEvent me = toMouseEvent(event);
        cee::vis::MouseButton buttonPressed = toMouseButton(event);
        inputHandler->mouseReleaseEvent(buttonPressed, me);
    }


    if (leftButtonCleanlyClicked)
    {
        const int xPos = event.GetX();
        const int yPos = GetClientSize().GetHeight() - event.GetY();

        // Picking!
        cee::Ray ray = view()->camera().rayFromWindowCoordinates(xPos, yPos);
        m_mainWindow->pick(ray);
    }

    Refresh();
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
void TutorialRunnerViewerCanvas::onMouseWheelMove(wxMouseEvent& event)
{
    cee::vis::CameraInputHandler* inputHandler = view() ? view()->camera().inputHandler() : NULL;
    if (inputHandler)
    {
        cee::vis::WheelEvent wEvent = toWheelEvent(event);
        inputHandler->wheelEvent(wEvent);

        Refresh();
    }
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::MouseEvent TutorialRunnerViewerCanvas::toMouseEvent(const wxMouseEvent& event)
{
    const int xPos = event.GetX();
    const int yPos = GetClientSize().GetHeight() - event.GetY();

    cee::vis::MouseButtons buttonsDown = cee::vis::NoButton;
    if (event.LeftIsDown())     buttonsDown |= cee::vis::LeftButton;
    if (event.MiddleIsDown())   buttonsDown |= cee::vis::MiddleButton;
    if (event.RightIsDown())    buttonsDown |= cee::vis::RightButton;

    cee::vis::KeyboardModifiers modifiers = toKeyboardModifiers(event);

    return cee::vis::MouseEvent(xPos, yPos, buttonsDown, modifiers);
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::WheelEvent TutorialRunnerViewerCanvas::toWheelEvent(const wxMouseEvent& event)
{
    const int xPos = event.GetX();
    const int yPos = GetClientSize().GetHeight() - event.GetY();
    
    cee::vis::MouseButtons buttonsDown = cee::vis::NoButton;
    if (event.LeftIsDown())     buttonsDown |= cee::vis::LeftButton;
    if (event.MiddleIsDown())   buttonsDown |= cee::vis::MiddleButton;
    if (event.RightIsDown())    buttonsDown |= cee::vis::RightButton;

    cee::vis::KeyboardModifiers modifiers = toKeyboardModifiers(event);

    return cee::vis::WheelEvent(event.GetWheelRotation(), xPos, yPos, buttonsDown, modifiers);
}



//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::MouseButton TutorialRunnerViewerCanvas::toMouseButton(const wxMouseEvent& event)
{
    if (event.Button(wxMOUSE_BTN_LEFT))    return cee::vis::LeftButton;
    if (event.Button(wxMOUSE_BTN_MIDDLE))  return cee::vis::MiddleButton;
    if (event.Button(wxMOUSE_BTN_RIGHT))   return cee::vis::RightButton;

    return cee::vis::NoButton;
}


//--------------------------------------------------------------------------------------------------
/// 
//--------------------------------------------------------------------------------------------------
cee::vis::KeyboardModifiers TutorialRunnerViewerCanvas::toKeyboardModifiers(const wxMouseEvent& event)
{
    cee::vis::KeyboardModifiers modifiers = 0;

    if (event.ControlDown()) modifiers |= cee::vis::ControlModifier;
    if (event.ShiftDown())   modifiers |= cee::vis::ShiftModifier;

    return modifiers;
}


