//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeApp/Base.h"
#include "CeeCore/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/Model.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/SimulationInfo.h"

#include <map>


namespace cee {

namespace ug
{
    class UnstructGridModel;
    class Error;
}

namespace app {

//==================================================================================================
//
// Class for UnstructGridModel management
//
//==================================================================================================
class CEE_APP_EXPORT Models : public RefCountedObject
{

public:
    static Models*                  instance();

    size_t                          count() const;

    cee::ug::UnstructGridModel*     current() const;
    cee::ug::SimulationInfo         currentInfo() const;
    void                            setCurrent(cee::ug::UnstructGridModel* model);

    size_t                          currentIndex() const;
    void                            setCurrentIndex(size_t index);

    size_t                          indexOf(const cee::ug::UnstructGridModel* model) const;
    cee::ug::UnstructGridModel*     get(size_t index) const;

    cee::ug::SimulationInfo         info(size_t index) const;
    cee::ug::SimulationInfo         info(const cee::ug::UnstructGridModel* model) const;

    size_t                          add(cee::ug::UnstructGridModel* model);

    cee::ug::UnstructGridModel*     open(const cee::Str& filename, cee::ug::Error* error);
    cee::ug::UnstructGridModel*     open(const cee::Str& filename, const cee::Str& password, cee::ug::Error* error);
    cee::ug::UnstructGridModel*     open(const std::vector<cee::Str>& filenameArr, cee::ug::Error* error);
    cee::ug::UnstructGridModel*     open(const std::vector<cee::Str>& filenameArr, const cee::Str& password, cee::ug::Error* error);
    bool                            reopen(cee::ug::UnstructGridModel* model, const cee::Str& filename, cee::ug::Error* error);
    bool                            reopen(size_t index, const cee::Str& filename, cee::ug::Error* error);
    bool                            openSecondary(const cee::Str& filename, cee::ug::Error* error);

    void                            close(size_t index);
    void                            close(cee::ug::UnstructGridModel* model);
    bool                            closeCurrent();
    void                            closeAll();

private:
    Models();
    virtual ~Models();
    CEE_PRIVATE_IMPL(Models);
    CEE_DISALLOW_COPY_AND_ASSIGN(Models);

};

} // namespace cee
} // namespace app
