//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeApp/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"

#include "CeeUnstructGrid/PartInfo.h"

namespace cee {

namespace ug
{
    class DataPart;
}

namespace app {


//==================================================================================================
//
// Class for simplified access to the parts in the currents state of the current unstructured grid model
//
//==================================================================================================
class CEE_APP_EXPORT Parts : public RefCountedObject
{

public:
    static Parts*                           instance();
    
    size_t                                  count() const;

    cee::ug::DataPart*                      current() const;
    cee::ug::PartInfo                       currentInfo() const;
    void                                    setCurrent(cee::ug::DataPart* part);

    size_t                                  currentIndex() const;
    void                                    setCurrentIndex(size_t index);

    size_t                                  indexOf(const cee::ug::DataPart* part) const;
    cee::ug::DataPart*                      get(size_t index) const;
    cee::ug::DataPart*                      get(const cee::Str& wildcard) const;

    cee::ug::PartInfo                       info(size_t index) const;
    cee::ug::PartInfo                       info(cee::ug::DataPart* part) const;

    size_t                                  geometryIndex(size_t index) const;
    size_t                                  geometryIndex(const cee::ug::DataPart* part) const;

private:
    Parts();
    virtual ~Parts();
    CEE_PRIVATE_IMPL(Parts);
    CEE_DISALLOW_COPY_AND_ASSIGN(Parts);

};


} // namespace cee
} // namespace app
