//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Str.h"
#include "CeeUnstructGrid/ResultInfo.h"
#include "CeeUnstructGrid/CustomResultGenerator.h"

namespace cee {
namespace app {

//==================================================================================================
///
/// Math function callback class
///
//==================================================================================================
class CEE_APP_EXPORT MathFunctionCallBack
{
public:
    MathFunctionCallBack(const cee::Str& functionName) : m_functionName(functionName){};

    virtual double operator()(double x)
    {
        CEE_UNUSED(x);
        return cee::UNDEFINED_DOUBLE;
    }

    const cee::Str& functionName() const { return m_functionName;  }

    virtual ~MathFunctionCallBack() {};

private:
    cee::Str m_functionName;
};


} // namespace cee
} // namespace app

const cee::ug::ResultInfo CEE_APP_EXPORT operator+(const cee::ug::ResultInfo& first, const cee::ug::ResultInfo& second); // R = U + V
const cee::ug::ResultInfo CEE_APP_EXPORT operator-(const cee::ug::ResultInfo& first, const cee::ug::ResultInfo& second); // R = U - V
const cee::ug::ResultInfo CEE_APP_EXPORT operator-(const cee::ug::ResultInfo& first);                                    // R = -U
const cee::ug::ResultInfo CEE_APP_EXPORT operator*(const cee::ug::ResultInfo& first, const cee::ug::ResultInfo& second); // R = U.V
const cee::ug::ResultInfo CEE_APP_EXPORT operator^(const cee::ug::ResultInfo& first, const cee::ug::ResultInfo& second); // R = UxV
const cee::ug::ResultInfo CEE_APP_EXPORT operator/(const cee::ug::ResultInfo& first, const cee::ug::ResultInfo& second); // R = U/V

const cee::ug::ResultInfo CEE_APP_EXPORT operator+(const cee::ug::ResultInfo& first, double constant); // R=U+k
const cee::ug::ResultInfo CEE_APP_EXPORT operator-(const cee::ug::ResultInfo& first, double constant); // R=U-k
const cee::ug::ResultInfo CEE_APP_EXPORT operator*(const cee::ug::ResultInfo& first, double constant); // R=k.U
const cee::ug::ResultInfo CEE_APP_EXPORT operator/(const cee::ug::ResultInfo& first, double constant); // R=U/k

const cee::ug::ResultInfo CEE_APP_EXPORT extractComponent(const cee::ug::ResultInfo& first, size_t componentIndex);  // R = U[i]
const cee::ug::ResultInfo CEE_APP_EXPORT l1Norm(const cee::ug::ResultInfo& first); // R = |Ux| + |Uy| + |Uz|
const cee::ug::ResultInfo CEE_APP_EXPORT l2Norm(const cee::ug::ResultInfo& first); // R = sqrt(Ux*Ux + Uy*Uy + Uz*Uz)

const cee::ug::ResultInfo CEE_APP_EXPORT average(const std::vector<cee::ug::ResultInfo>& arguments); // Arithmetic average
const cee::ug::ResultInfo CEE_APP_EXPORT min(const std::vector<cee::ug::ResultInfo>& arguments); // Min
const cee::ug::ResultInfo CEE_APP_EXPORT max(const std::vector<cee::ug::ResultInfo>& arguments); // Max

const cee::ug::ResultInfo CEE_APP_EXPORT minValue(const cee::ug::ResultInfo& first, bool perPart = false); // Min value of argument, returned as a constant scalar
const cee::ug::ResultInfo CEE_APP_EXPORT maxValue(const cee::ug::ResultInfo& first, bool perPart = false); // Max value of argument, returned as a constant scalar
const cee::ug::ResultInfo CEE_APP_EXPORT averageValue(const cee::ug::ResultInfo& first, bool perPart = false); // Avg value of argument, returned as a constant scalar

const cee::ug::ResultInfo CEE_APP_EXPORT toElementsArithmetic(const cee::ug::ResultInfo& first); // Node to element, simple arithmetic averaging
const cee::ug::ResultInfo CEE_APP_EXPORT toNodesArithmetic(const cee::ug::ResultInfo& first); // Element to node, simple arithmetic averaging
const cee::ug::ResultInfo CEE_APP_EXPORT createMathFunctionResult(const cee::ug::ResultInfo& first, cee::app::MathFunctionCallBack* function);

const cee::ug::ResultInfo CEE_APP_EXPORT                vonMises(const cee::ug::ResultInfo& first);
const std::vector<cee::ug::ResultInfo> CEE_APP_EXPORT   principals(const cee::ug::ResultInfo& first);

/*const cee::ug::ResultInfo CEE_APP_EXPORT firstPrincipalVector(const cee::ug::ResultInfo& first);
const cee::ug::ResultInfo CEE_APP_EXPORT secondPrincipalVector(const cee::ug::ResultInfo& first);
const cee::ug::ResultInfo CEE_APP_EXPORT thirdPrincipalVector(const cee::ug::ResultInfo& first);
const cee::ug::ResultInfo CEE_APP_EXPORT firstPrincipalValue(const cee::ug::ResultInfo& first);
const cee::ug::ResultInfo CEE_APP_EXPORT secondPrincipalValue(const cee::ug::ResultInfo& first);
const cee::ug::ResultInfo CEE_APP_EXPORT thirdPrincipalValue(const cee::ug::ResultInfo& first);

*/
