//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeApp/Base.h"
#include "CeeCore/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeUnstructGrid/DataState.h"
#include "CeeUnstructGrid/ResultInfo.h"

namespace cee {
namespace app {


//==================================================================================================
//
// Class for result management
//
//==================================================================================================
class CEE_APP_EXPORT Results : public RefCountedObject
{

public:

    static Results*                     instance();

    size_t                              count() const;
    size_t                              scalarCount() const;
    size_t                              vectorCount() const;
    size_t                              displacementCount() const;
    size_t                              tensorCount() const;

    size_t                              indexOf(cee::ug::ResultInfo info) const;
    size_t                              scalarIndexOf(cee::ug::ResultInfo info) const;
    size_t                              vectorIndexOf(cee::ug::ResultInfo info) const;
    size_t                              displacementIndexOf(cee::ug::ResultInfo info) const;
    size_t                              tensorIndexOf(cee::ug::ResultInfo info) const;

    cee::ug::ResultInfo                 resultInfo(size_t index) const;
    cee::ug::ResultInfo                 resultInfo(const Str& wildcard) const;

    cee::ug::ResultInfo                 scalarInfo(size_t index) const;
    cee::ug::ResultInfo                 scalarInfo(const Str& wildcard) const;

    cee::ug::ResultInfo                 vectorInfo(size_t index) const;
    cee::ug::ResultInfo                 vectorInfo(const Str& wildcard) const;

    cee::ug::ResultInfo                 displacementInfo(size_t index) const;
    cee::ug::ResultInfo                 displacementInfo(const Str& wildcard) const;

    cee::ug::ResultInfo                 tensorInfo(size_t index) const;
    cee::ug::ResultInfo                 tensorInfo(const Str& wildcard) const;

    cee::ug::DataPartScalar*            currentDataPartScalar(cee::ug::ResultInfo scalar);
    cee::ug::DataPartVector*            currentDataPartVector(cee::ug::ResultInfo vector);
    cee::ug::DataPartSymmetricTensor*   currentDataPartTensor(cee::ug::ResultInfo tensor);
    cee::ug::DataPartDisplacement*      currentDataPartDisplacement(cee::ug::ResultInfo displacement);

    cee::ug::DataPartScalar*            currentDataPartScalar(const Str& wildcard);
    cee::ug::DataPartVector*            currentDataPartVector(const Str& wildcard);
    cee::ug::DataPartSymmetricTensor*   currentDataPartTensor(const Str& wildcard);
    cee::ug::DataPartDisplacement*      currentDataPartDisplacement(const Str& wildcard);

    cee::ug::DataResultScalar*          currentDataResultScalar(cee::ug::ResultInfo scalar);
    cee::ug::DataResultVector*          currentDataResultVector(cee::ug::ResultInfo vector);
    cee::ug::DataResultSymmetricTensor* currentDataResultTensor(cee::ug::ResultInfo tensor);
    cee::ug::DataResultDisplacement*    currentDataResultDisplacement(cee::ug::ResultInfo displacement);

    cee::ug::DataResultScalar*          currentDataResultScalar(const Str& wildcard);
    cee::ug::DataResultVector*          currentDataResultVector(const Str& wildcard);
    cee::ug::DataResultSymmetricTensor* currentDataResultTensor(const Str& wildcard);
    cee::ug::DataResultDisplacement*    currentDataResultDisplacement(const Str& wildcard);

    void                                add(cee::ug::ResultInfo result);
    void                                add(cee::ug::ResultInfo result, const Str& name);
    void                                remove(cee::ug::ResultInfo result);
    void                                removeAll();

private:
    Results();
    virtual ~Results();

    CEE_PRIVATE_IMPL(Results);
    CEE_DISALLOW_COPY_AND_ASSIGN(Results);

};

}
}
