//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeApp/Base.h"
#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/RefCountedObject.h"

#include <map>

namespace cee {
namespace ug  {
    class HitItem;
}
}

namespace cee {
namespace app {



//==================================================================================================
//
// Class for selection management
//
//==================================================================================================
class CEE_APP_EXPORT Selection : public RefCountedObject
{

public:

    enum ItemType
    {
        NONE,
        ELEMENT,
        NODE,
        ELEMENT_NODE,
        ELEMENT_SURFACE,
        COORDINATE
    };

    Selection();

    size_t              count();

    void                set(size_t index, const cee::ug::HitItem& item);
    cee::ug::HitItem    get(size_t index);

    size_t              add(const cee::ug::HitItem& hitItem);

    size_t              addNode(size_t index);
    size_t              addElement(size_t index);
    size_t              addElementNode(size_t elementIndex, size_t localNodeIndex);
    size_t              addElementSurface(size_t elementIndex, size_t surfaceIndex);
    size_t              addCoordinate(const cee::Vec3d& coordinate);

    void                remove(size_t index);
    void                removeAll();

    void                setItemType(ItemType selectionType);
    ItemType            itemType() const;

    bool                isMultiSelectionEnabled() const;
    void                enableMultiSelection(bool enable);

    const int           initialGeometryId(size_t index) const;

private:
    virtual ~Selection();

    CEE_PRIVATE_IMPL(Selection);
    CEE_DISALLOW_COPY_AND_ASSIGN(Selection);

};

} // namespace cee
} // namespace app
