//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeApp/Base.h"
#include "CeeUnstructGrid/Situation.h"

#include <map>

namespace cee {

namespace ug {
class UnstructGridModel;
class Situation;
}

namespace vis { class View; }
}

namespace cee {
namespace app {

//==================================================================================================
//
// Class for Situations management
//
//==================================================================================================
class CEE_APP_EXPORT Situations : public RefCountedObject
{
public:
    static Situations*      instance();

    void                    add(ug::Situation* situation);
    void                    remove(size_t index);
    void                    removeAll();

    PtrRef<ug::Situation>   captureCurrent() const;

    size_t                  situationsCount() const;
    ug::Situation*          situation(size_t index);
    const ug::Situation*    situation(size_t index) const;
    size_t                  situationIndex(const ug::Situation* situation) const;
    bool                    updateSituationInfo(size_t index, const cee::Str& situationFolder, const cee::Str& newName, const cee::Str& newDesc);

    void                    setRecoveryFolder(const Str& folder);
    bool                    recoverySituationExists() const;
    void                    saveRecovery();
    PtrRef<ug::Situation>   loadRecovery() const;
    void                    removeRecovery();
    ug::Situation*          captureRecovery();

    void                    exportSituations(std::vector<const ug::Situation*> situations, const Str& archiveFile);
    std::vector<PtrRef<ug::Situation> > importSituations(const Str& archiveFile);


private:
    Situations();
    virtual ~Situations();

    CEE_PRIVATE_IMPL(Situations);
    CEE_DISALLOW_COPY_AND_ASSIGN(Situations);
};

} // namespace cee
} // namespace app
