//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: App
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeApp/Base.h"
#include "CeeCore/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/View.h"


namespace cee {
namespace app {

class Selection;

//==================================================================================================
///
/// Small view info struct
///
//==================================================================================================
struct CEE_APP_EXPORT ViewInfo
{
    ViewInfo() {};
};


//==================================================================================================
//
// Class for View management
//
//==================================================================================================
class CEE_APP_EXPORT Views : public RefCountedObject
{

public:

    static Views*               instance();

    size_t                      count() const;

    cee::vis::View*             current();
    ViewInfo                    currentViewInfo() const;
    Selection*                  currentViewSelection();
    void                        setCurrent(cee::vis::View* view);

    size_t                      currentIndex() const;
    void                        setCurrentIndex(size_t index);

    size_t                      indexOf(const cee::vis::View* view) const;
    cee::vis::View*             get(size_t index);

    ViewInfo                    viewInfo(size_t viewIndex) const;
    ViewInfo                    viewInfo(cee::vis::View* view) const;

    Selection*                  viewSelection(size_t viewIndex);
    Selection*                  viewSelection(cee::vis::View* view);

    cee::PtrRef<cee::vis::View> add();

    bool                        remove(size_t index);
    bool                        remove(cee::vis::View* view);
    bool                        removeCurrent();


private:
    Views();
    virtual ~Views();

    CEE_PRIVATE_IMPL(Views);
    CEE_DISALLOW_COPY_AND_ASSIGN(Views);

};

} // namespace cee
} // namespace app
