//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {


//==================================================================================================
//
// Static helper class for handling assert failures in CDC
//
//==================================================================================================
class CEE_CORE_EXPORT AssertHelper
{
public:
    enum PostFailAction
    {
        CONTINUE,   // Continue execution
        DEBUGBREAK  // Execution should be stopped and debugger should be triggered (currently windows only)
    };

public:
    static PostFailAction handleFailedAssert(const char* fileName, int lineNumber, const char* expr, const char* msg);
};

} // namespace cee



// Default behavior for assert macros
// In debug builds, all asserts are in action, both normal asserts and tight asserts
// In release builds, tight asserts are disabled by default while normal asserts are still in action 
#ifndef CEE_ENABLE_APP_ASSERTS
    #define CEE_ENABLE_APP_ASSERTS 1
#endif

#ifndef CEE_ENABLE_TIGHT_APP_ASSERTS
    #ifdef _DEBUG
        #define CEE_ENABLE_TIGHT_APP_ASSERTS 1
    #else
        #define CEE_ENABLE_TIGHT_APP_ASSERTS 0
    #endif
#endif


// Define to trigger debug trap for use with assert macros on Windows
#ifdef WIN32
    #define CEE_APP_ASSERT_DEBUGTRAP()  __debugbreak()
#else
    #define CEE_APP_ASSERT_DEBUGTRAP()  ((void)0)
#endif



// Define our assert macros
#if CEE_ENABLE_APP_ASSERTS == 1
    #define CEE_APP_ASSERT(expr)            (void)( (!!(expr)) || (cee::AssertHelper::CONTINUE == cee::AssertHelper::handleFailedAssert(__FILE__, __LINE__, #expr, NULL))  || (CEE_APP_ASSERT_DEBUGTRAP(), 0) )
    #define CEE_APP_ASSERT_MSG(expr, msg)   (void)( (!!(expr)) || (cee::AssertHelper::CONTINUE == cee::AssertHelper::handleFailedAssert(__FILE__, __LINE__, #expr, (msg))) || (CEE_APP_ASSERT_DEBUGTRAP(), 0) )
    #define CEE_APP_FAIL_MSG(msg)           (void)( (cee::AssertHelper::CONTINUE == cee::AssertHelper::handleFailedAssert(__FILE__, __LINE__, NULL, (msg))) || (CEE_APP_ASSERT_DEBUGTRAP(), 0) )
#else 
    #define CEE_APP_ASSERT(expr)            ((void)0)
    #define CEE_APP_ASSERT_MSG(expr, msg)   ((void)0)
    #define CEE_APP_FAIL_MSG(msg)           ((void)0)
#endif

#if CEE_ENABLE_TIGHT_APP_ASSERTS == 1 && CEE_ENABLE_APP_ASSERTS == 1
    #define CEE_APP_TIGHT_ASSERT(expr)      CEE_APP_ASSERT(expr)
#else 
    #define CEE_APP_TIGHT_ASSERT(expr)      ((void)0)
#endif



