//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/LogManager.h"
#include "CeeCore/CodeLocation.h"
#include "CeeCore/PtrRef.h"

namespace cee {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_CORE_EXPORT AppLogging
{
public:
    static void     initialize(const Str& appLoggerName);

    static void     logError(const Str& message, const CodeLocation& location);
    static void     logWarning(const Str& message, const CodeLocation& location);
    static void     logInfo(const Str& message, const CodeLocation& location);
    static void     logDebug(const Str& message, const CodeLocation& location);

    static void     setStandardLogLevels();
    static void     setNormalDebugLogLevels();
    static void     setFullInfoLogLevels();
    static void     setFullDebugLogLevels();

private:
    static Str      sm_appLoggerName;   // Name of the application log
};

} // namespace cee


#define CEE_LOG_APP_ERROR(theMessage)        (cee::AppLogging::logError(theMessage, CEE_CODE_LOCATION))  
#define CEE_LOG_APP_WARNING(theMessage)      (cee::AppLogging::logWarning(theMessage, CEE_CODE_LOCATION))  
#define CEE_LOG_APP_INFO(theMessage)         (cee::AppLogging::logInfo(theMessage, CEE_CODE_LOCATION))  
#define CEE_LOG_APP_DEBUG(theMessage)        (cee::AppLogging::logDebug(theMessage, CEE_CODE_LOCATION))  
