//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "Base.h"

#ifdef WIN32
  #define CEE_ATOMIC_COUNTER_CLASS_EXISTS
#elif defined(CEE_OSX)
  #include <libkern/OSAtomic.h>
  #define CEE_ATOMIC_COUNTER_CLASS_EXISTS
#elif defined __GNUC__
    #define CEE_HAVE_GCC_ATOMICS
    #define CEE_ATOMIC_COUNTER_CLASS_EXISTS
#endif
#ifdef CEE_IOS // Disable for all compilers and architectures for Apple iOS
  #undef CEE_ATOMIC_COUNTER_CLASS_EXISTS
  #ifndef CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS
    #define CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS
  #endif
#endif

#if defined(CEE_ATOMIC_COUNTER_CLASS_EXISTS)

namespace cee {

class CEE_CORE_EXPORT AtomicCounter
{
public:
    explicit AtomicCounter(int initialValue);
    ~AtomicCounter();

    operator int () const;
    void setCount(int count);
    
    void add(int count);

    int operator ++ ();     // prefix
    int operator ++ (int);  // postfix

    int operator -- ();     // prefix
    int operator -- (int);  // postfix

private:
    
    CEE_DISALLOW_COPY_AND_ASSIGN(AtomicCounter);

#ifdef WIN32
    typedef volatile long ImplType;
#elif defined(CEE_IOS) || defined(CEE_OSX)
    typedef int32_t ImplType;
#else
    typedef int ImplType;
#endif

    ImplType m_counter;
};


}

#endif
