//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once


// Disable some annoying warnings so we can compile with warning level Wall
#ifdef WIN32
// 4512  'class' : assignment operator could not be generated : Due to problems with classes with reference member variables (e.g. VertexCompactor)
// 4514  unreferenced inline/local function has been removed
// 4625  copy constructor could not be generated because a base class copy constructor is inaccessible
// 4626  assignment operator could not be generated because a base class assignment operator is inaccessible
// 4640  'staticInstance' : construction of local static object is not thread-safe -> used by singletons. To be revisited.
// 4710  function 'func_name' not inlined
// 4711  function 'func_name' selected for automatic inline expansion
// 4738  storing 32-bit float result in memory, possible loss of performance
// 4820  'bytes' bytes padding added after construct 'member_name'
// 4986  'operator new[]': exception specification does not match previous declaration
#pragma warning (disable: 4512 4514 4625 4626 4640 4710 4711 4738 4820 4986)
#endif


// Brings in size_t and definition of NULL
#include <cstddef>


// The default configuration is for consuming CDC as a dynamic library (DLL).
// To build DLL version of CDC, CEE_XXXX_BUILD_DLL_EXPORT must be defined and CEE_STATIC_LIBS must NOT be defined.
// To build or use CDC as static libraries, make sure CEE_STATIC_LIBS is defined.
#ifdef CEE_STATIC_LIBS
    #define CEE_CORE_EXPORT
#else
    #if defined(WIN32)
        #ifdef CEE_CORE_BUILD_DLL_EXPORT
            #define CEE_CORE_EXPORT __declspec(dllexport)
        #else
            #define CEE_CORE_EXPORT __declspec(dllimport)
        #endif
    #elif defined(__GNUC__)
        #ifdef CEE_CORE_BUILD_DLL_EXPORT
            #define CEE_CORE_EXPORT __attribute__ ((visibility ("default")))
        #else
            #define CEE_CORE_EXPORT
        #endif
    #else
        #define CEE_CORE_EXPORT
    #endif
#endif


// Get rid of the the macros MIN and MAX which are specified in the windows headers
#if defined WIN32 && !defined NOMINMAX
#define NOMINMAX
#endif


// Macro to disallow copy constructor and assignment operator functions
// Should be placed in the class' private: section
#define CEE_DISALLOW_COPY_AND_ASSIGN(Class) \
  Class(const Class&); \
  void operator=(const Class&)

// Macro for avoiding "unused parameter" warnings
// The bottom one is the best alternative, but unfortunately doesn't work on VS2010
#ifdef WIN32
#define CEE_UNUSED(EXPR) (void)(EXPR);
#else
#define CEE_UNUSED(EXPR) (void)sizeof(EXPR); 
#endif

// Makes it easier to check on the current GCC version
#ifdef __GNUC__
// 40302 means version 4.3.2.
    # define CEE_GCC_VER  (__GNUC__*10000 + __GNUC_MINOR__*100 + __GNUC_PATCHLEVEL__)
#endif

// Introduce types and constants in the Ceetron Desktop Components cee namespace
namespace cee {

/// Undefined value for int
const int       UNDEFINED_INT               = 2147483647;
/// Undefined value for size_t
const size_t    UNDEFINED_SIZE_T            = static_cast<size_t>(-1);  // 18446744073709551615u
/// Undefined value for double
const double    UNDEFINED_DOUBLE            = 1.7976931348623158e+308;
/// Undefined threshold for double
const double    UNDEFINED_DOUBLE_THRESHOLD  = 1.00e+308;
/// Undefined value for float
const float     UNDEFINED_FLOAT             = 3.402823466e+38f;
/// Undefined threshold for float
const double    UNDEFINED_FLOAT_THRESHOLD   = 2.99e+38f;

} // namespace cee

// Check if the compiler supports the 'override' keyword
#if (defined(_MSC_VER) && (_MSC_VER > 1600)) || (!defined(_MSC_VER) && __cplusplus >= 201103L)
#define CEE_SUPPORTS_OVERRIDE_KEYWORD
#endif

#include "CeeCore/Pimpl.h"

