//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Mat4d.h"

namespace cee {


//==================================================================================================
//
// A bounding box
//
//==================================================================================================
class CEE_CORE_EXPORT BoundingBox
{
public:
    BoundingBox();
    BoundingBox(const Vec3d& min, const Vec3d& max);
    BoundingBox(const BoundingBox& other);
    ~BoundingBox();

    BoundingBox& operator=(const BoundingBox& rhs);
    BoundingBox  operator+(const BoundingBox& rhs);

    void    reset();
    bool    isValid() const;

    void    add(const Vec3d& vertex);
    void    add(const BoundingBox& bb);

    Vec3d   minimum() const;
    Vec3d   maximum() const;

    Vec3d   center() const;
    Vec3d   extent() const;
    double  radius() const;

    void    transform(const Mat4d& matrix);

private:
    CEE_PRIVATE_IMPL(BoundingBox);
};

} // namespace cee
