//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2020, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Str.h"

#include <vector>

namespace cee {


//==================================================================================================
//
// Static class containing file system utility functions
//
//==================================================================================================
class CEE_CORE_EXPORT FileSystem
{
public:
    static Str                  fromNativeSeparators(const Str& path);
    static Str                  toNativeSeparators(const Str& path);

    static Str                  addTrailingSeparator(const Str& path);
    static Str                  removeTrailingSeparator(const Str& path);

    static Str                  fileName(const Str& path);
    static Str                  extension(const Str& path);
    static Str                  parentPath(const Str& path);

    static Str                  makeAbsolute(const Str& path);

    static Str                  currentPath();

    static bool                 pathExists(const Str& pathName);
    static bool                 fileExists(const Str& fileName);
    static bool                 directoryExists(const Str& dirName);

    static bool                 createDirectory(const Str& dirName);
    static bool                 createDirectoryTree(const Str& dirPath);

    static bool                 copyFile(const Str& fromFileName, const Str& toFileName);

    static bool                 deleteFile(const Str& fileName);
    static bool                 deleteDirectory(const Str& dirName);
    static bool                 deleteAllFilesInDirectory(const Str& dirName);
    static bool                 deleteDirectoryRecursive(const Str& dirName);

    static bool                 getDirectoryContents(const Str& dirName, std::vector<Str>* fileNameArr, std::vector<Str>* dirNameArr);
    static std::vector<Str>     getAllFilesInDirectory(const Str& dirName);
    static std::vector<Str>     findFilesInDirectory(const Str& dirName, const Str& filePattern);

    static uint64_t             fileSize(const Str& fileName);
    static unsigned int         fileContentsCrc(const Str& fileName, uint64_t maxBytesToRead);
};

}  

