//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"

namespace cee {

class LogEvent;

//==================================================================================================
///
/// \class cee::LogDestination
/// \ingroup Core
/// 
/// Base class for log destinations.
/// 
/// Available log destinations are [console](@ref LogDestinationConsole) and [file](@ref LogDestinationFile).
/// You can also create your own log destination by deriving from LogDestination and overriding the 
/// log method. This is useful if you already have a logging system in your application and would like 
/// to incorporate the logging message from Ceetron toolkits into your application log.
///
//==================================================================================================
class CEE_CORE_EXPORT LogDestination : public RefCountedObject
{
public:
	virtual Str targetName() const				= 0; ///< Returns a string to identify the logger's target
    virtual void log(const LogEvent& logEvent)	= 0; ///< Sends event to logger
};

} // namespace cee
