//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"

namespace cvf {
class LogManager;
}

namespace cee {

class LogDestination;
class CodeLocation;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_CORE_EXPORT LogManager : public RefCountedObject
{
public:
    virtual ~LogManager();

    void setLevel(const Str& loggerName, int logLevel);
    void setDestination(LogDestination* logDestination);
    
    void logError(const Str& loggerName, const Str& errorMessage);
    void logError(const Str& loggerName, const Str& errorMessage, const CodeLocation& location);
    void logWarning(const Str& loggerName, const Str& warningMessage);
    void logWarning(const Str& loggerName, const Str& warningMessage, const CodeLocation& location);
    void logInfo(const Str& loggerName, const Str& infoMessage);
    void logInfo(const Str& loggerName, const Str& infoMessage, const CodeLocation& location);
    void logDebug(const Str& loggerName, const Str& debugMessage);
    void logDebug(const Str& loggerName, const Str& debugMessage, const CodeLocation& location);

    void logInfoForced(const Str& loggerName, const Str& infoMessage);

public:
    cvf::LogManager* internal_cvfLogManager();

private:
    LogManager(cvf::LogManager* cvfLogManager);

    CEE_PRIVATE_IMPL(LogManager);
    CEE_DISALLOW_COPY_AND_ASSIGN(LogManager);

    friend class CoreComponent;
};

} // namespace cee
