//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"

namespace cee {


//==================================================================================================
//
//
//
//==================================================================================================
class CEE_CORE_EXPORT Mat4d 
{
public:
    Mat4d();
    Mat4d(const Mat4d& other);
    Mat4d(double m00, double m01, double m02, double m03, 
          double m10, double m11, double m12, double m13, 
          double m20, double m21, double m22, double m23, 
          double m30, double m31, double m32, double m33);
    ~Mat4d();

    Mat4d&          operator=(const Mat4d& other);

    bool            operator==(const Mat4d& rhs) const;
    bool            operator!=(const Mat4d& rhs) const;

    const Mat4d     operator*(const Mat4d& rhs) const;

    bool            isIdentity() const;
    void            setIdentity();

    bool            isZero() const;
    void            setZero();
    
    bool            invert();

    double          rowCol(int row, int col) const;
    void            setRowCol(int row, int col, double value);
    double&         operator()(int row, int col);
    double          operator()(int row, int col) const;

    const double*   rawPointer() const;

    static Mat4d    fromTranslation(const Vec3d& trans);
    static Mat4d    fromScaling(const Vec3d& scale);
    static Mat4d    fromRotation(const Vec3d& axis, double angle);

private:
    CEE_PRIVATE_IMPL(Mat4d);
};


} // namespace cee
