//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once


namespace cee {
namespace internal {


// Helper class for holding our pimpl pointers
template <typename T>
class PimplPointerHolder
{
public:
    explicit PimplPointerHolder(T* ptr = NULL): m_ptr(ptr) {}
    ~PimplPointerHolder()       { delete m_ptr; m_ptr = NULL; }
    T* operator->()             { return m_ptr; }
    T* operator->() const       { return m_ptr; }
    T& operator*()              { return *m_ptr; }
    T& operator*() const        { return *m_ptr; }
    T* get()                    { return m_ptr; }
    T* get() const              { return m_ptr; }

    void reset(T* ptr = NULL)
    {
        if (ptr != m_ptr)
        {
            if (m_ptr != NULL)
            {
                delete m_ptr;
            }

            m_ptr = ptr;
        }
    }

private:
    CEE_DISALLOW_COPY_AND_ASSIGN(PimplPointerHolder);
    T* m_ptr;
};

}
}



#ifdef WIN32

// Disable 4251, only used in private: so it is not accessible for users of the DLL
#define CEE_PRIVATE_IMPL(Class) \
__pragma(warning (push)) \
__pragma(warning (disable: 4251)) \
cee::internal::PimplPointerHolder<class Class##Private> m_pimpl; \
__pragma(warning (pop))

#else

#define CEE_PRIVATE_IMPL(Class) \
cee::internal::PimplPointerHolder<class Class##Private> m_pimpl

#endif


#define CEE_BASE_F(Class) \
friend class Class

#define CEE_PRIVATE_F(Class) \
friend class Class##Private

#define CEE_PRIVATE_INIT(Class) \
m_pimpl.reset(new Class##Private)

#define CEE_PRIVATE_INIT_PARAM1(Class, ConstrutorParam1) \
m_pimpl.reset(new Class##Private(ConstrutorParam1))

#define CEE_PRIVATE_INIT_PARAM2(Class, ConstrutorParam1, ConstrutorParam2) \
m_pimpl.reset(new Class##Private(ConstrutorParam1, ConstrutorParam2))

#define CEE_PRIVATE_INIT_PARAM3(Class, ConstrutorParam1, ConstrutorParam2, ConstrutorParam3) \
m_pimpl.reset(new Class##Private(ConstrutorParam1, ConstrutorParam2, ConstrutorParam3))

#define CEE_PRIVATE_INIT_PARAM4(Class, ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4) \
m_pimpl.reset(new Class##Private(ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4))

#define CEE_PRIVATE_INIT_PARAM5(Class, ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4, ConstrutorParam5) \
m_pimpl.reset(new Class##Private(ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4, ConstrutorParam5))

#define CEE_PRIVATE_INIT_PARAM6(Class, ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4, ConstrutorParam5, ConstrutorParam6) \
m_pimpl.reset(new Class##Private(ConstrutorParam1, ConstrutorParam2, ConstrutorParam3, ConstrutorParam4, ConstrutorParam5, ConstrutorParam6))
