//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"

namespace cee {

class Ray;

//==================================================================================================
//
// Class defining a plane in space
//
//==================================================================================================
class CEE_CORE_EXPORT Plane
{
public:
    Plane();
    Plane(const Plane& other);
    Plane(double A, double B, double C, double D);
    ~Plane();

    Plane&  operator=(const Plane& other);

    bool    operator==(const Plane& rhs) const;
    bool    operator!=(const Plane& rhs) const;

    double  A() const;
    double  B() const;
    double  C() const;
    double  D() const;

    bool    isValid() const;
    double  distance(const Vec3d& point) const;
    double  distanceSquared(const Vec3d& point) const;
    Vec3d   normal() const;
    Vec3d   pointInPlane() const;

    void    set(double A, double B, double C, double D);
    bool    setFromPointAndNormal(const Vec3d& point, const Vec3d& normal);
    bool    setFromPoints(const Vec3d& p1, const Vec3d& p2, const Vec3d& p3);

    void    transform(const Mat4d& matrix);
    
    bool    rayIntersect(const Ray& ray, Vec3d* intersectionPoint) const;
    bool    rayIntersect(const Ray& ray, double* distance) const;
    
    bool    projectVector(const Vec3d& vector, Vec3d* projectedVector) const;
    Vec3d   projectPoint(const Vec3d& point) const;
    
    static Plane    createFromPointAndNormal(const Vec3d& point, const Vec3d& normal);
    static Plane    createFromPoints(const Vec3d& p1, const Vec3d& p2, const Vec3d& p3);

private:
    CEE_PRIVATE_IMPL(Plane);
};

} // namespace cee
