//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {


//==================================================================================================
//
// Smart pointer class
//
//==================================================================================================
template <typename T>
class PtrRef
{
public:
    PtrRef(T* object = NULL);
    PtrRef(const PtrRef& other);
    template<typename T2> PtrRef(const PtrRef<T2>& other);
    ~PtrRef();

    PtrRef&                         operator=(T* rhs);
    PtrRef&                         operator=(PtrRef rhs);
    template<typename T2> PtrRef&   operator=(const PtrRef<T2>& rhs);
    inline T*                       operator->();
    inline const T*                 operator->() const;
    inline T&                       operator*();
    inline const T&                 operator*() const;
    bool                            operator<(const PtrRef& rhs) const; 

    inline const T*                 get() const;
    inline T*                       get();

    void                            swap(PtrRef& other);

    T*                              detach();

    typedef T* PtrRef::*unspecified_bool_type;  ///< Helper type to implement the safe-bool idiom
    operator unspecified_bool_type() const;

private:
    T* m_object;
};

template<typename T1, typename T2> inline bool operator==(const PtrRef<T1>& a, const PtrRef<T2>& b) { return a.get() == b.get(); }  ///< Returns true if the internal pointers of refs \a a and \a b are equal.
template<typename T1, typename T2> inline bool operator!=(const PtrRef<T1>& a, const PtrRef<T2>& b) { return a.get() != b.get(); }  ///< Returns true if the internal pointers of refs \a a and \a b are different.
template<typename T1, typename T2> inline bool operator==(const PtrRef<T1>& a, T2* b)               { return a.get() == b; }        ///< Returns true if the internal pointer of ref \a a is equal to the naked pointer \a b.
template<typename T1, typename T2> inline bool operator!=(const PtrRef<T1>& a, T2* b)               { return a.get() != b; }        ///< Returns true if the internal pointer of ref \a a is different from the naked pointer \a b.
template<typename T1, typename T2> inline bool operator==(T1* a, const PtrRef<T2>& b)               { return a == b.get(); }        ///< Returns true if the naked pointer \a a is equal to the internal pointer of ref \a b.
template<typename T1, typename T2> inline bool operator!=(T1* a, const PtrRef<T2>& b)               { return a != b.get(); }        ///< Returns true if the naked pointer \a a is different from the internal pointer of ref \a b.

/// Swap contents of \a a and \a b. Matches signature of std::swap().
/// \todo Need to investigate which STL algorithms actually utilize the swap() function.
template<typename T> inline void swap(PtrRef<T>& a, PtrRef<T>& b)   { a.swap(b); }              
}

#include "CeeCore/PtrRef.inl"
