//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/AtomicCounter.h"

#if defined(CEE_ATOMIC_COUNTER_CLASS_EXISTS) && defined(CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS)
#error Two mutually exclusive defines detected : CEE_ATOMIC_COUNTER_CLASS_EXISTS && CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS
#endif

#if !defined(CEE_ATOMIC_COUNTER_CLASS_EXISTS) && !defined(CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS)
#error No support for atomics. Define CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS to be able to compile
#endif

namespace cee {


//==================================================================================================
//
// Base class for all reference counted objects
//
//==================================================================================================
class CEE_CORE_EXPORT RefCountedObject
{
public:
    RefCountedObject();
    virtual ~RefCountedObject();

    void    addRef() const;
    void    release() const;
    int     refCount() const;

    void    setRefCountZero() const;

private:
#if defined(CEE_ATOMIC_COUNTER_CLASS_EXISTS)
    mutable AtomicCounter m_refCount;
#elif defined(CEE_WORKAROUND_TO_COMPILE_ON_SYSTEMS_WITHOUT_ATOMICS)
    mutable int m_refCount;
#endif

    CEE_DISALLOW_COPY_AND_ASSIGN(RefCountedObject);
};

}
