//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

#include <string>
#include <vector>

namespace cee {


//==================================================================================================
//
// String class
//
//==================================================================================================
class CEE_CORE_EXPORT Str
{
public:
    Str();
    Str(const Str& other);
    Str(const char* theString);
    explicit Str(const wchar_t* theString);
    explicit Str(const std::string& theString);
    explicit Str(const std::wstring& theString);
    explicit Str(char c);
    ~Str();

    Str&                operator=(const Str& other);

    bool                operator==(const Str& rhs) const;
    bool                operator!=(const Str& rhs) const;
    bool                operator<(const Str& rhs) const;
    const Str           operator+(const Str& rhs) const;
    Str&                operator+=(const Str& rhs);

    Str                 toLower() const;
    Str                 toUpper() const;
    Str                 trimmedRight() const;
    Str                 trimmedLeft() const;
    Str                 trimmed() const;
    Str                 simplified() const;

    std::vector<Str>    split(const Str& delimiters = " ") const;
    size_t              find(const Str& str, size_t start = 0) const;
    bool                startsWith(const Str& str) const;
    Str                 subStr(size_t start, size_t length = npos) const;
    void                replace(const Str& before, const Str& after);

    const wchar_t*      c_str() const;
    std::string         toStdString() const;
    std::wstring        toStdWString() const;
    std::string         toUtf8() const;

    bool                isEmpty() const;
    size_t              size() const;

    static Str          number(int n);
    static Str          number(unsigned int n);
    static Str          number(float n, char format = 'g', int precision = -1);
    static Str          number(double n, char format = 'g', int precision = -1);
    static Str          numberFromAddress(const void* ptr);

    double              toDouble(bool* ok = NULL) const;
    double              toDouble(double defaultValue) const;
    float               toFloat(bool* ok = NULL) const;
    float               toFloat(float defaultValue) const;
    int                 toInt(bool* ok = NULL) const;
    int                 toInt(int defaultValue) const;
    unsigned int        toUInt(bool* ok = NULL) const;
    unsigned int        toUInt(unsigned int defaultValue) const;

    Str                 arg(const Str& a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(char a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(int a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(int64_t a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(unsigned int a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(uint64_t a, int fieldWidth = 0, char fillChar = ' ') const;
    Str                 arg(float a, int fieldWidth = 0, char format = 'g', int precision = -1, char fillChar = ' ') const;
    Str                 arg(double a, int fieldWidth = 0, char format = 'g', int precision = -1, char fillChar = ' ') const;
#ifdef CEE_OSX
    Str                 arg(size_t a, int fieldWidth = 0, char fillChar = ' ') const;
#endif

    void                swap(Str& other);

    bool                matches(const Str& wildcard) const;

    const void*         unspecifiedWCharPointer() const;
    static Str          fromUnspecifiedWCharPointer(const void* voidPtr);

    static const size_t npos;       ///< Same as std::string::npos. This value, when used as the value for a count parameter n in string's member functions, roughly indicates "as many as possible".
                                    ///  As a return value it is usually used to indicate failure.
private:
    CEE_PRIVATE_IMPL(Str);
};

Str CEE_CORE_EXPORT  operator+(const char* str1, const Str& str2);

} // namespace cee
