//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"


namespace cee {

//==================================================================================================
//
// A symmetric 3x3 tensor represented by 6 values: xx, yy, zz, xy, yz, zx
//
//==================================================================================================
class CEE_CORE_EXPORT SymmetricTensor 
{
public:
    SymmetricTensor();
    SymmetricTensor(const SymmetricTensor& other);
    SymmetricTensor(double xx, double yy, double zz, double xy, double yz, double zx);
	~SymmetricTensor();

    SymmetricTensor&        operator=(const SymmetricTensor& SymmetricTensor);
    bool                    operator==(const SymmetricTensor& rhs) const;
    bool                    operator!=(const SymmetricTensor& rhs) const;

    const SymmetricTensor   operator+(const SymmetricTensor& rhs) const;
    const SymmetricTensor   operator-(const SymmetricTensor& rhs) const;

    SymmetricTensor&        operator+=(const SymmetricTensor& rhs);
    SymmetricTensor&        operator-=(const SymmetricTensor& rhs);

    const Vec3d             operator*(const Vec3d& v) const;

    const SymmetricTensor   operator*(double scalar) const;
    const SymmetricTensor   operator/(double scalar) const;
    SymmetricTensor&        operator*=(double scalar);
    SymmetricTensor&        operator/=(double scalar);

    friend SymmetricTensor  operator*(double scalar, const SymmetricTensor& t) { return t*scalar; }

    const double&           xx() const;
    const double&           yy() const;
    const double&           zz() const;
    const double&           xy() const;
    const double&           yz() const;
    const double&           zx() const;
    
    double&                 xx();
    double&                 yy();
    double&                 zz();
    double&                 xy();
    double&                 yz();
    double&                 zx();

    void                    set(double xx, double yy, double zz, double xy, double yz, double zx);

    const double*           rawPointer() const;

    double                  trace() const;
    SymmetricTensor         deviator() const;
    static SymmetricTensor  identity();

    double                  vonMises() const;
    double                  firstPrincipalValue() const;
    double                  secondPrincipalValue() const;
    double                  thirdPrincipalValue() const;

    Vec3d                   firstPrincipalVector() const;
    Vec3d                   secondPrincipalVector() const;
    Vec3d                   thirdPrincipalVector() const;

protected:
    SymmetricTensor(double* values);

private:
    CEE_PRIVATE_IMPL(SymmetricTensor);
    CEE_BASE_F(SharedMemorySymmetricTensor);
};

//==================================================================================================
//
// A symmetric 3x3 tensor represented by 6 values: xx, yy, zz, xy, yz, zx
//
//==================================================================================================
class CEE_CORE_EXPORT SharedMemorySymmetricTensor : public SymmetricTensor
{
public:
    SharedMemorySymmetricTensor(double* data);
    void  setSharedData(double* data);
};

} // namespace cee
