//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Str.h"

#include <vector>

namespace cee {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_CORE_EXPORT Variant
{
public:
    enum DataType
    {
        INVALID,
        INT,
        UINT,
        DOUBLE,
        FLOAT,
        BOOL,
        VEC3D,
        COLOR3F,
        STRING,
        ARRAY
    };

public:
    Variant();
    Variant(const Variant& other);
    ~Variant();

    Variant(int val);
    Variant(unsigned int val);
    Variant(double val);
    Variant(float val);
    Variant(bool val);
    Variant(const Vec3d& val);
    Variant(const Color3f& val);
    Variant(const Str& val);
    Variant(const char* val);
    Variant(const std::vector<Variant>& arr);

    Variant&                operator=(const Variant& rhs);
    bool                    operator==(const Variant& rhs) const;
    bool                    operator!=(const Variant& rhs) const;

    DataType                type() const;
    bool                    isValid() const;

    int                     getInt() const;
    unsigned int            getUInt() const;
    double                  getDouble() const;
    float                   getFloat() const;
    bool                    getBool() const;
    Vec3d                   getVec3d() const;
    Color3f                 getColor3f() const;
    Str                     getString() const;
    std::vector<Variant>    getArray() const;

private:
    CEE_PRIVATE_IMPL(Variant);
};

}  // namespace cee
