//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {

//==================================================================================================
//
// 2D double vector
//
//==================================================================================================
class CEE_CORE_EXPORT Vec2d 
{
public:
    Vec2d();
    Vec2d(const Vec2d& other);
    Vec2d(double x, double y);

    Vec2d&          operator=(const Vec2d& other);
    bool            operator==(const Vec2d& rhs) const;
    bool            operator!=(const Vec2d& rhs) const;
    const Vec2d     operator+(const Vec2d& rhs) const;
    const Vec2d     operator-(const Vec2d& rhs) const;
    const Vec2d     operator*(double scalar) const;
    const Vec2d     operator/(double scalar) const;
    Vec2d&          operator+=(const Vec2d& rhs);
    Vec2d&          operator-=(const Vec2d& rhs);
    Vec2d&          operator*=(double scalar);
    Vec2d&          operator/=(double scalar);

    static double    dot(const Vec2d& v1, const Vec2d& v2);
    double           operator*(const Vec2d& rhs) const;    // Dot product

    const double&   x() const;
    const double&   y() const;
    double&         x();
    double&         y();
    void            set(double x, double y);

    bool            normalize();
    double          length() const;

private:
    double m_vec[2];
};

} // namespace cee
