//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {

//==================================================================================================
//
// 2D float vector
//
//==================================================================================================
class CEE_CORE_EXPORT Vec2f 
{
public:
    Vec2f();
    Vec2f(const Vec2f& other);
    Vec2f(float x, float y);

    Vec2f&          operator=(const Vec2f& other);
    bool            operator==(const Vec2f& rhs) const;
    bool            operator!=(const Vec2f& rhs) const;
    const Vec2f     operator+(const Vec2f& rhs) const;
    const Vec2f     operator-(const Vec2f& rhs) const;
    const Vec2f     operator*(float scalar) const;
    const Vec2f     operator/(float scalar) const;
    Vec2f&          operator+=(const Vec2f& rhs);
    Vec2f&          operator-=(const Vec2f& rhs);
    Vec2f&          operator*=(float scalar);
    Vec2f&          operator/=(float scalar);

    static float    dot(const Vec2f& v1, const Vec2f& v2);
    float           operator*(const Vec2f& rhs) const;    // Dot product
    
    const float&    x() const;
    const float&    y() const;
    float&          x();
    float&          y();
    void            set(float x, float y);

    bool            normalize();
    float           length() const;

private:
    float m_vec[2];
};

} // namespace cee
