//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {

class Mat4d;

//==================================================================================================
//
// 3D double vector
//
//==================================================================================================
class CEE_CORE_EXPORT Vec3d 
{
public:
    Vec3d();
    Vec3d(const Vec3d& other);
    Vec3d(double x, double y, double z);

    Vec3d&          operator=(const Vec3d& other);
    bool            operator==(const Vec3d& rhs) const;
    bool            operator!=(const Vec3d& rhs) const;
    const Vec3d     operator+(const Vec3d& rhs) const;
    const Vec3d     operator-(const Vec3d& rhs) const;
    const Vec3d     operator*(double scalar) const;
    const Vec3d     operator/(double scalar) const;
    Vec3d&          operator+=(const Vec3d& rhs);
    Vec3d&          operator-=(const Vec3d& rhs);
    Vec3d&          operator*=(double scalar);
    Vec3d&          operator/=(double scalar);

	/// Returns a vector that is the vector \a v multiplied by a given \a scalar
    friend Vec3d    operator*(double scalar, const Vec3d& v) { return v*scalar; }

    static double   dot(const Vec3d& v1, const Vec3d& v2);
    double          operator*(const Vec3d& rhs) const;    // Dot product

    static Vec3d    cross(const Vec3d& v1, const Vec3d& v2);
    const Vec3d     operator^(const Vec3d& rhs) const;    // Cross product

    const double&   x() const;
    const double&   y() const;
    const double&   z() const;
    double&         x();
    double&         y();
    double&         z();
    void            set(double x, double y, double z);

    void            transformPoint(const Mat4d& m);    
    void            transformVector(const Mat4d& m);
    bool            normalize();

    double          length() const;
    double          lengthSquared() const;
    double          angle(const Vec3d& other);

private:
    double m_vec[3];
};

/// Undefined value for Vec3d
const Vec3d UNDEFINED_VECTOR = Vec3d(cee::UNDEFINED_DOUBLE, cee::UNDEFINED_DOUBLE, cee::UNDEFINED_DOUBLE);

} // namespace cee
