//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Core
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"

namespace cee {

class Mat4d;

//==================================================================================================
//
// 3D float vector
//
//==================================================================================================
class CEE_CORE_EXPORT Vec3f 
{
public:
    Vec3f();
    Vec3f(const Vec3f& other);
    Vec3f(float x, float y, float z);

    Vec3f&          operator=(const Vec3f& other);
    bool            operator==(const Vec3f& rhs) const;
    bool            operator!=(const Vec3f& rhs) const;
    const Vec3f     operator+(const Vec3f& rhs) const;
    const Vec3f     operator-(const Vec3f& rhs) const;
    const Vec3f     operator*(float scalar) const;
    const Vec3f     operator/(float scalar) const;
    Vec3f&          operator+=(const Vec3f& rhs);
    Vec3f&          operator-=(const Vec3f& rhs);
    Vec3f&          operator*=(float scalar);
    Vec3f&          operator/=(float scalar);

    static float    dot(const Vec3f& v1, const Vec3f& v2);
    float           operator*(const Vec3f& rhs) const;    // Dot product

    static Vec3f    cross(const Vec3f& v1, const Vec3f& v2);
    const Vec3f     operator^(const Vec3f& rhs) const;    // Cross product

    const float&    x() const;
    const float&    y() const;
    const float&    z() const;
    float&          x();
    float&          y();
    float&          z();
    void            set(float x, float y, float z);

    void            transformPoint(const Mat4d& m);    
    void            transformVector(const Mat4d& m);
    bool            normalize();

    float           length() const;
    float           angle(const Vec3f& other);

private:
    float m_vec[3];
};

} // namespace cee
