//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeExport/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"

#include <vector>

namespace cee {

class PropertySetCollection;
class Image;
class ImageResources;

namespace ug {
class UnstructGridModel;
class VTFxMemoryFile;
}

namespace exp {

//==================================================================================================
//
// Export selected view to VTFx
//
//==================================================================================================
class CEE_EXP_EXPORT ExportVTFx
{
public:
    enum ExportType
    {
        DISPLAY_MODEL_ONLY,
        FULL_DATA_MODEL
    };
    
    enum OverlayPosition
    {
        TOP_LEFT,
        TOP_RIGHT,
        BOTTOM_LEFT,
        BOTTOM_RIGHT
    };

public:
    ExportVTFx(ExportType type, ug::UnstructGridModel* model);
    ExportVTFx(ExportType type, ug::UnstructGridModel* model, bool forceIdenticalDatabase);
    ~ExportVTFx();

    void addProperties(const PropertySetCollection& properties);
    void addResources(const ImageResources& resources);
    void clearPropertiesAndResources();
    size_t addPlotData(const Str& plotName, const std::vector<Str>& variableNames, const std::vector<std::vector<double> >& variableValues);

    void setVendorNameAndApplication(const Str& vendorName, const Str& vendorApplication);
    void setSnapshotImage(Image* snapshotImage);
    void setCaseName(const Str& name);
    void setCaseDescription(const Str& description);
    void setCustomLogo(Image* logoImage, OverlayPosition position);

    void setVisiblePartsOnly(bool on);
    void setStartAnimation(bool on);
    void setCompressionLevel(unsigned int compressionLevel);
    void setPassword(const Str& password);
    void setCreateSignature(bool on);
    void setExportRelativeValues(bool on);
    void setWriteAsciiDataFiles(bool on);
    void setRelativeDisplacements(bool on);
    void setExportOnlyVisibleElements(bool on);

    bool addUserTextFile(const Str& filename, const Str& content);

    bool saveCase(const Str& fileName);
    bool appendCase(const Str& fileName);

    void close();

    bool saveCase(ug::VTFxMemoryFile* vtfxMemoryFile);
    bool appendCase(ug::VTFxMemoryFile* vtfxMemoryFile);

private:
    CEE_PRIVATE_IMPL(ExportVTFx);
    CEE_DISALLOW_COPY_AND_ASSIGN(ExportVTFx);
};

}  // namespace ug
}  // namespace cee
