//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeCore/BoundingBox.h"
#include "CeeCore/RefCountedObject.h"

namespace cee {
namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT Data : public RefCountedObject
{
public:

    /// Enumerator for each of the available primitives
    enum Type
    {
        INDEXED_TRIANGLES,                              ///< Indexed triangles
        TRIANGLE_FAN,                                   ///< Triangles organized as a fan
        TRIANGLE_STRIP,                                 ///< Triangles organized as a strip
        INDEXED_POLYLINES,                              ///< Indexed polylines
        POINTS                                          ///< Point cloud
    };

public:
    virtual ~Data();

    virtual Type            type() const = 0;           ///< Returns the primitive type of the associated data
    virtual cee::BoundingBox boundingBox() const = 0;    ///< Returns the bounding box of the associated data

protected:
    Data();

private:
    CEE_PRIVATE_F(Part);
    CEE_PRIVATE_F(GeometryModel);
    CEE_PRIVATE_F(PartClassification);
    CEE_DISALLOW_COPY_AND_ASSIGN(Data);
};

} // namespace geo
} // namespace cee
