//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeGeometry/DataIndexedTriangles.h"
#include "CeeGeometry/DataIndexedPolylines.h"

namespace cee {
namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT DataGenerator
{
public:
    static PtrRef<DataIndexedTriangles> createSphere(const Vec3d& centerPos, double radius, size_t numSubDivisions);
    static PtrRef<DataIndexedTriangles> createCylinder(const Vec3d& bottomCenterPos, double outerRadius, double innerRadius, const Vec3d& heightDirection, double height, size_t numSubDivisions);
    static PtrRef<DataIndexedPolylines> createCircle(const Vec3d& centerPos, double radius, const Vec3d& normal, size_t numSubDivisions);
    static PtrRef<DataIndexedPolylines> createArc(const Vec3d& centerPos, double radius, const Vec3d& normal, const Vec3d& startDirection, double angle, size_t numSubDivisions);
    static PtrRef<DataIndexedTriangles> createBox(const Vec3d& centerPos, const Vec3d& totalExtent);
    static PtrRef<DataIndexedPolylines> createLineSegments(const std::vector<Vec3d>& vertices);
    static PtrRef<DataIndexedTriangles> createTriangles(const std::vector<Vec3d>& vertices, double creaseAngleDegrees = 0.0);
    static PtrRef<DataIndexedTriangles> createTriangles(const std::vector<Vec3d>& vertices, const std::vector<unsigned int>& indices, double creaseAngleDegrees = 0.0);

private:
    DataGenerator();
    CEE_DISALLOW_COPY_AND_ASSIGN(DataGenerator);
};

} // namespace geo
} // namespace cee
