//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeGeometry/Data.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/BoundingBox.h"

#include <vector>

namespace cee {
namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT DataIndexedTriangles : public Data
{
public:
    DataIndexedTriangles();
    DataIndexedTriangles(const std::vector<Vec3d>& vertices, const std::vector<unsigned int>& indices);
    virtual ~DataIndexedTriangles();

    virtual Type                type() const;
    virtual BoundingBox         boundingBox() const;

    std::vector<Vec3d>          vertices() const;
    void                        setVertices(const std::vector<Vec3d>& vertices);

    std::vector<unsigned int>   indices() const;
    void                        setIndices(const std::vector<unsigned int>& indices);

    size_t                      triangleCount() const;
    void                        removeAll();

private:
    CEE_PRIVATE_IMPL(DataIndexedTriangles);
    CEE_PRIVATE_F(Part);
    CEE_PRIVATE_F(GeometryModel);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataIndexedTriangles);
};

} // namespace geo
} // namespace cee
