//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/PtrRef.h"

namespace cee {
namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT Effect : public RefCountedObject
{
public:
    /// Enumerator for each of the available effect types
    enum Type
    {
        COLOR,                                  ///< %Effect describing the color to be used when drawing
        OPACITY,                                ///< %Effect describing the opacity to be used when drawing
        FRONT_AND_BACK_COLOR,                   ///< %Effect describing the colors to be used when drawing each side of a polygon
        FRONT_AND_BACK_OPACITY,                 ///< %Effect describing the opacity to be used when drawing each side of a polygon
        TEXTURE,                                ///< %Effect describing the texture to be used when drawing
        EYE_LIFT,                               ///< %Effect that moves the vertices towards the eye point when drawing
        POLYGON_OFFSET,                         ///< %Effect that moves the vertices into the screen when drawing
        HALO,                                   ///< %Effect describing how to draw a part with silhouetted edges
        LIGHTING,                               ///< %Effect describing if the part should be lighted or not
        LINE_WIDTH,                             ///< %Effect describing the width of the line, in pixels
        POINT_SIZE                              ///< %Effect describing the size of the points, in pixels
    };
    
public:
    virtual ~Effect();

    virtual Type            type() const = 0;   ///< Returns the primitive type of the associated data
    virtual PtrRef<Effect>  clone() = 0;        ///< Clones the effect

protected:
    Effect();

private:
    CEE_PRIVATE_F(GeometryModel);
    CEE_PRIVATE_F(PartSettings);
    CEE_PRIVATE_F(PartClassification);
    CEE_PRIVATE_F(CvfPartPoints);
    CEE_PRIVATE_F(CvfPartIndexedPolylines);
    CEE_PRIVATE_F(CvfPartIndexedTriangles);
};

} // namespace geo
} // namespace cee

