//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Color4f.h"
#include "CeeGeometry/Base.h"
#include "CeeGeometry/Effect.h"

namespace cee {
class Image;

namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT EffectTexture : public Effect
{
public:
    /// Specifies what should happen to texture coordinates that are outside the 0 to 1 range
    enum SamplerWrapMode
    {
        REPEAT,                 ///< The integer part of the coordinate will be ignored and a repeating pattern is formed.
        CLAMP_TO_EDGE,          ///< The texture coordinates will be clamped into the range 0 to 1
        CLAMP_TO_BORDER,        ///< The coordinates that fall outside the 0 to 1 range will be given the borderColor().
        MIRRORED_REPEAT         ///< The texture will also be repeated, but it will be mirrored when the integer part of the coordinate is odd.
    };

    /// Specifies the minifying and magnification filtering used when doing the texture mapping.
    enum SamplerFilter
    {
        NEAREST,                ///< Nearest neighbor filtering on the base MipMap level. Texture will look blocky when zoom into.
        LINEAR,                 ///< Linear filtering on the base MipMap level. (Default magnification filter in OpenGL). Texture will look blurry when zoomed into.
        NEAREST_MIPMAP_NEAREST, ///< Selects nearest MipMap level and performs nearest neighbor filtering.
        LINEAR_MIPMAP_NEAREST,  ///< Selects nearest MipMap level and performs linear filtering.
        NEAREST_MIPMAP_LINEAR,  ///< Perform linear interpolation between MipMap levels and perform nearest neighbor filtering. (Default minifying filter in OpenGL).
        LINEAR_MIPMAP_LINEAR    ///< Perform linear interpolation between MipMap levels and perform linear filtering (trilinear mipmapping).
    };

public:
    EffectTexture();
    EffectTexture(const Image* textureImage);
    virtual ~EffectTexture();

    virtual Type            type() const;
    virtual PtrRef<Effect>  clone();
    
    static PtrRef<EffectTexture>    createResultMapping(const Image* textureImage);

    const Image*            image() const;
    void                    setImage(const Image* textureImage);

    SamplerWrapMode         samplerWrapMode() const;
    void                    setSamplerWrapMode(SamplerWrapMode wrapMode);

    SamplerFilter           samplerMinifyingFilter() const;
    void                    setSamplerMinifyingFilter(SamplerFilter filterType);

    SamplerFilter           samplerMagnificationFilter() const;
    void                    setSamplerMagnificationFilter(SamplerFilter filterType);

    Color4f                 borderColor() const;
    void                    setBorderColor(const Color4f& color);

    float                   ambientIntensity() const;
    void                    setAmbientIntensity(float intensity);

    float                   specularIntensity() const;
    void                    setSpecularIntensity(float intensity);

private:
    CEE_PRIVATE_IMPL(EffectTexture);
    CEE_PRIVATE_F(GeometryModel);
    CEE_PRIVATE_F(PartSettings);
    CEE_PRIVATE_F(PartClassification);
    CEE_PRIVATE_F(CvfPartIndexedPolylines);
    CEE_PRIVATE_F(CvfPartPoints);
    CEE_BASE_F(CvfPartTrianglesBase);
    CEE_DISALLOW_COPY_AND_ASSIGN(EffectTexture);
};

} // namespace geo
} // namespace cee
