//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeGeometry/PartHitItems.h"
#include "CeeVisualization/Model.h"
#include "CeeCore/BoundingBox.h"
#include "CeeCore/Ray.h"
#include "CeeVisualization/View.h"
#include "CeeCore/Str.h"
#include "CeeCore/Vec2f.h"

namespace cee {
namespace geo {

class Part;
class HitItem;


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT GeometryModel : public vis::Model
{
public:
    GeometryModel();
    virtual ~GeometryModel();

    virtual BoundingBox     boundingBox();
    virtual Str             modelInfo() const;
    virtual void            update(bool forceRecreate);

    const Mat4d&            transformation() const;
    void                    setTransformation(const Mat4d& transformMatrix);

    size_t                  partCount() const;
    size_t                  partIndex(Part* part) const;
    Part*                   part(size_t index);
    const Part*             part(size_t index) const;
    Part*                   partById(int id);
    const Part*             partById(int id) const;
    void                    addPart(Part* part);
    void                    removePart(Part* part);
    void                    removeAllParts();
    bool                    hasPart(Part* part) const;

    bool                    rayIntersect(int x, int y, const vis::View& view, HitItem* hitItem);
    bool                    regionIntersect(int x, int y, unsigned int width, unsigned int height, const vis::View& view, bool acceptPartiallyContainedParts, std::vector<Part*>* partList);
    bool                    regionIntersectPartItems(int x, int y, unsigned int width, unsigned int height, const vis::View& view, bool acceptPartiallyContainedItems, std::vector<PartHitItems>* hitItems);
    bool                    polygonIntersect(const std::vector<Vec2f>& polygonInWindowCoordinates, const vis::View& view, bool acceptPartiallyContainedParts, std::vector<Part*>* partList);
    bool                    rayIntersect(const Ray& ray, const vis::View& view, HitItem* hitItem);
    
    size_t                  renderedPartCount() const;

public:
    bool                    internal_exportGeometry(const Str& fileName);

private:
    virtual cvf::Model*     internal_cvfModel();
    virtual void            internal_setViewCapabilities(const cvf::OpenGLCapabilities& capabilities, bool viewDoesHighlighting);
    virtual bool            internal_canAddToViewWithCapabilities(const cvf::OpenGLCapabilities& viewCapabilities) const;
    virtual NotifyAction    internal_detectAndConfigureViewClippingInModel(const vis::View& triggeringView, cvf::OpenGLContext* cvfContext);
    virtual void            internal_applyViewClippingPlanesBeforeRendering(const cvf::ClipPlaneSet* viewClipPlaneSet, const cvf::Plane* dropShadowClipPlane);
    virtual void            internal_applyCameraTransformationToClippingPlanes(const cvf::Camera& camera);
    virtual NotifyAction    internal_preRenderUpdateModel(const vis::View& triggeringView, cvf::OpenGLContext* cvfContext);

private:
    CEE_PRIVATE_IMPL(GeometryModel);
    CEE_DISALLOW_COPY_AND_ASSIGN(GeometryModel);
};

} // namespace geo
} // namespace cee
