//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Mat4d.h"
#include "CeeCore/BoundingBox.h"

namespace cee {
namespace geo {

class PartSettings;
class Data;
class TextureCoordinates;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT Part : public RefCountedObject
{
public:
    Part();
    Part(Data* data);
    virtual ~Part();

    Data*                   data();
    const Data*             data() const;
    void                    setData(Data* data);

    int                     id() const;
    void                    setId(int id);

    template<typename T>
    const T*                dataOfType() const;
    template<typename T>
    T*                      dataOfType();

    TextureCoordinates*     textureCoordinates();
    void                    setTextureCoordinates(TextureCoordinates* coordinates);

    PartSettings&           settings();
    const PartSettings&     settings() const;

    const Mat4d&            transformation() const;
    void                    setTransformation(const Mat4d& matrix);

    BoundingBox             boundingBox();

private:
    CEE_PRIVATE_IMPL(Part);
    CEE_PRIVATE_F(GeometryModel);
    CEE_DISALLOW_COPY_AND_ASSIGN(Part);
};


//--------------------------------------------------------------------------------------------------
/// Returns the active (last inserted/top of stack) effect of the given type T, or NULL if 
/// no effect of that type was found
//--------------------------------------------------------------------------------------------------
template<typename T>
const T* Part::dataOfType() const
{
    if (dynamic_cast<const T*>(data()))
    {
        return static_cast<const T*>(data());
    }

    return NULL;
}


//--------------------------------------------------------------------------------------------------
/// Returns the active (last inserted/top of stack) effect of the given type T, or NULL if 
/// no effect of that type was found
//--------------------------------------------------------------------------------------------------
template<typename T>
T* Part::dataOfType()
{
    // Use the const version to avoid code duplication
    return const_cast<T*>(static_cast<const Part&>(*this).dataOfType<T>());
}

} // namespace geo
} // namespace cee

