//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Geometry
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeGeometry/Base.h"
#include "CeeVisualization/Model.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeGeometry/Effect.h"

namespace cee {
namespace geo {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_GEO_EXPORT PartSettings : public RefCountedObject
{
public:
    PartSettings();
    virtual ~PartSettings();

    void                setVisible(bool visible);
    bool                visible() const;
    
    void                setIntersectable(bool intersectable);
    bool                intersectable() const;

    size_t              effectCount() const;
    size_t              effectIndex(const Effect* effect) const;
    Effect*             effect(size_t index);
    const Effect*       effect(size_t index) const;
    void                addEffect(Effect* effect);
    void                removeEffect(Effect* effect);
    void                removeAllEffects();

    template<typename T>
    const T*            activeEffectOfType() const;
    template<typename T>
    T*                  activeEffectOfType();

    template<typename T>
    void                removeAllEffectsOfType();

private:
    CEE_PRIVATE_IMPL(PartSettings);
    CEE_PRIVATE_F(GeometryModel);
    CEE_PRIVATE_F(Part);
    CEE_DISALLOW_COPY_AND_ASSIGN(PartSettings);
};


//--------------------------------------------------------------------------------------------------
/// Removes all effects of the given type
//--------------------------------------------------------------------------------------------------
template<typename T>
void PartSettings::removeAllEffectsOfType()
{
    size_t i = effectCount();
    while (i > 0)
    {
        i--;
        if (dynamic_cast<const T*>(effect(i)))
        {
            removeEffect(effect(i));
        }
    }
}


//--------------------------------------------------------------------------------------------------
/// Returns the active (last inserted/top of stack) effect of the given type T, or NULL if 
/// no effect of that type was found
//--------------------------------------------------------------------------------------------------
template<typename T>
const T* PartSettings::activeEffectOfType() const
{
    size_t i = effectCount();
    while (i > 0)
    {
        i--;
        if (dynamic_cast<const T*>(effect(i)))
        {
            return static_cast<const T*>(effect(i));
        }
    }

    return NULL;
}


//--------------------------------------------------------------------------------------------------
/// Returns the active (last inserted/top of stack) effect of the given type T, or NULL if 
/// no effect of that type was found
//--------------------------------------------------------------------------------------------------
template<typename T>
T* PartSettings::activeEffectOfType()
{
    // Use the const version to avoid code duplication
    return const_cast<T*>(static_cast<const PartSettings&>(*this).activeEffectOfType<T>());
}

} // namespace geo
} // namespace cee

