//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: ImportCae
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeImportCae/Base.h"
#include "CeeUnstructGrid/DataSourceInterface.h"

namespace cee {

namespace ug {
    class Error;
}

namespace imp {
namespace cae {

struct ReaderInfo
{
    Str interfaceName;  ///< Name of file interface
    Str fileMasks;      ///< ; separated list of file masks
};

class ReaderSettings;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_IMP_CAE_EXPORT DataSourceCae : public cee::ug::DataSourceInterface
{
public:
    /// Poll change results description
    enum PollChangeResult
    {
        NONE                     = 0x00000000,  ///< None
        RESULT_MODIFIED          = 0x00000001,  ///< Result has been modified
        GEOMETRY_MODIFIED        = 0x00000002,  ///< Geometry has been modified
        META_DATA_CHANGED        = 0x00000004,  ///< Metadata has been modified
        TRANSFORMATIONS_MODIFIED = 0x00000008   ///< Transformations has been modified
    };
    
    typedef int PollChangeResults; ///< Poll change results
    
public:
    DataSourceCae(int dataSourceId);
    ~DataSourceCae();

    virtual bool                open(const Str& fileName, cee::ug::Error* error = NULL);
    virtual bool                open(const std::vector<Str>& fileNameArr, cee::ug::Error* error = NULL);
    virtual bool                openSecondary(const Str& fileName, cee::ug::Error* error = NULL);
    virtual void                close();

    bool                        isSupported(const Str& fileName) const;

    PollChangeResults           pollForChanges();

    bool                        executeRequest(const Str& message, const Str& data, Str* response, PollChangeResults* changeResults);

    void                        enableResultCaching(bool enable);
    void                        clearCachedResults();

    Str                         vdmVersion() const;

    static ReaderSettings&      readerSettings();
    
    static std::vector<ReaderInfo>  supportedFormats();
    static bool                     supportsDataProviderPlugins();
    static bool                     loadDataProviderPlugin(const Str& baseFileName);
    static std::vector<cee::Str>    loadDataProviderPluginsFromWildcard(const Str& wildcardString);

private:
    CEE_PRIVATE_IMPL(DataSourceCae);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataSourceCae);
};
    
}
}
}
