//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: ImportCae
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeUnstructGrid/Element.h"

#include "CeeImportCae/Base.h"
#include "CeeImportCae/ReaderSettingsMarc.h"
#include "CeeImportCae/ReaderSettingsPtc.h"
#include "CeeImportCae/ReaderSettingsOpenFoam.h"
#include "CeeImportCae/ReaderSettingsDataProvider.h"

#include <vector>
#include <map>
#include <string>

namespace cee {
namespace imp {
namespace cae {

class DataSourceCaePrivate;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_IMP_CAE_EXPORT ReaderSettings
{

public:
    ReaderSettings();
	virtual ~ReaderSettings();

	const ReaderSettingsMarc&           marc() const;
    ReaderSettingsMarc&                 marc();

    const ReaderSettingsPtc&            ptc() const;
    ReaderSettingsPtc&                  ptc();

    const ReaderSettingsOpenFoam&       openFoam() const;
    ReaderSettingsOpenFoam&             openFoam();

    size_t                              dataProviderSettingsCount() const;
    const ReaderSettingsDataProvider&   dataProviderSettings(size_t index) const;
    ReaderSettingsDataProvider&         dataProviderSettings(size_t index);
    void                                addDataProviderSettings(const cee::imp::cae::ReaderSettingsDataProvider& cdpSettings);

    void                                setEnsuresUniqueResultNames(bool ensure);
    bool                                ensuresUniqueResultNames() const;

    void                                setUsesUndefinedResults(bool use);
    bool                                usesUndefinedResults() const;

    bool                                isGlobalCoordinateTransformationEnabled() const;
    void                                enableGlobalCoordinateTransformation(bool enable);

    void                                            excludeElementCategory(cee::ug::Element::Category category);
    void                                            clearExcludedElementCategories();
    const std::vector<cee::ug::Element::Category>&  excludedElementCategories() const;

    int                                 getCategoryBottomSectionId(int categoryId) const;
    int                                 getCategoryTopSectionId(int categoryId) const;
    std::vector<int>                    getSectionCategoryIds() const;

    void                                setTopAndBottomSectionIdForCategory(int categoryId, int sectionIdTop, int sectionIdBottom);

private:
    CEE_PRIVATE_IMPL(ReaderSettings);
    CEE_BASE_F(DataSourceCaePrivate);
};

}
}
}
