//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: ImportCae
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2020, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeImportCae/Base.h"
#include "CeeCore/Str.h"
#include "CeeCore/RefCountedObject.h"

namespace cee {
namespace imp {
namespace cae {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_IMP_CAE_EXPORT ReaderSettingsDataProvider
{
public:
    /// Data value types
    enum DataType 
    {
        DOUBLE,    ///< Enum to indicates that value is a double
        BOOL,      ///< Enum to indicates that value is a boolean
        STRING     ///< Enum to indicates that value is a string
    };

public:
    ReaderSettingsDataProvider(const cee::Str& providerIdString);
    ~ReaderSettingsDataProvider();
    
    ReaderSettingsDataProvider(const ReaderSettingsDataProvider& other);
    ReaderSettingsDataProvider& operator=(const ReaderSettingsDataProvider& other);
    
    const cee::Str& providerIdString() const;

    size_t          count() const;
    const cee::Str& key(size_t index) const;
    DataType        dataType(size_t index) const;
    const cee::Str& description(size_t index) const;
    double          doubleValue(size_t index) const;
    bool            boolValue(size_t index) const;
    const cee::Str& stringValue(size_t index) const;

    size_t          keyIndex(const cee::Str& key) const;

    void            setDoubleValue(const cee::Str& key, double value);
    void            setBoolValue(const cee::Str& key, bool value);
    void            setStringValue(const cee::Str& key, const cee::Str& value);

    void            setDescription(const cee::Str& key, const cee::Str& description);

private:
    CEE_PRIVATE_IMPL(ReaderSettingsDataProvider);
};

} // namespace cae
} // namespace imp
} // namespace cee
