//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Plot2d
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeePlot2d/Base.h"

#include "CeeVisualization/OverlayItem.h"

#include "CeeCore/Str.h"

namespace cee {
class Color3f;

namespace vis {
class Font;
}

namespace plt {
class AxisSettings;
class MarkerLine;
class Curve;

//==================================================================================================
//
// A plot that can be used as an overlay item in a view
//
//==================================================================================================
class CEE_PLT_EXPORT OverlayPlot : public vis::OverlayItem
{
public:
    /// Position of the plot legend
    enum LegendPosition
    {
        RIGHT,          ///< Show the legend to the right of the plot
        BELOW           ///< Show the legend below the plot
    };

public:
    OverlayPlot(vis::Font* font);
    virtual ~OverlayPlot();

    virtual unsigned int    width() const;
    virtual unsigned int    height() const;
    void                    setSize(unsigned int width, unsigned int height);

    virtual int             priority() const;
    virtual void            setPriority(int priority);

    Str                     title() const;
    void                    setTitle(const Str& title);

    bool                    drawBackground() const;
    void                    setDrawBackground(bool drawBackground);

    Color3f                 backgroundColor() const;
    void                    setBackgroundColor(const Color3f& color);

    float                   backgroundOpacity() const;
    void                    setBackgroundOpacity(float opacity);

    Color3f                 borderColor() const;
    void                    setBorderColor(const Color3f& color);

    float                   borderWidth() const;
    void                    setBorderWidth(float width);

    bool                    drawBorder() const;
    void                    setDrawBorder(bool drawBorder);

    const AxisSettings&     axisSettings() const;
    AxisSettings&           axisSettings();

    const MarkerLine&       markerLine() const;
    MarkerLine&             markerLine();

    bool                    drawCurveLegend() const;
    void                    setDrawCurveLegend(bool drawCurveLegend);
    LegendPosition          curveLegendPosition() const;
    void                    setCurveLegendPosition(LegendPosition position);

    Color3f                 textColor() const;
    void                    setTextColor(const Color3f& color);

    void                    setFont(vis::Font* font);

    size_t                  curveCount() const;
    const Curve*            curve(size_t curveIndex) const;
    Curve*                  curve(size_t curveIndex);
    Curve*                  addCurve(const Str& curveName, const std::vector<double>& xValues, const std::vector<double>& yValues);
    void                    removeAllCurves();
    void                    removeCurve(size_t curveIndex);

    double                  xValuesMinimum() const;
    double                  xValuesMaximum() const;
    double                  yValuesMinimum() const;
    double                  yValuesMaximum() const;

private:
    virtual cvf::OverlayItem* internal_cvfOverlayItem();

private:
    CEE_PRIVATE_IMPL(OverlayPlot);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayPlot);
};

} // namespace plt
} // namespace cee
