//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Report
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/RefCountedObject.h"

#include "CeeCore/Image.h"
#include "CeeReport/Table.h"
#include "CeeUnstructGrid/VTFxMemoryFile.h"

#include <string>
#include <map>

namespace cee {
namespace ug {
    class UnstructGridModel;
}
namespace rep {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_REP_EXPORT Snapshot : public RefCountedObject
{
public:
    enum SnapshotType 
    {
        OBJECT_IMAGE  = 1, // Image file
        OBJECT_VTFX   = 2, // VTFx file
        OBJECT_TABLE  = 3  // Tabular data
    };

public:
    Snapshot(SnapshotType type);
    virtual ~Snapshot();

    // General attributes
    Str                     name() const;
    void                    setName(const Str& name);
    SnapshotType            type() const;
    Str                     title() const;
    void                    setTitle(const Str& title);
    Str                     description() const;
    void                    setDescription(const Str& description);
    Str                     expandedDescription() const;
    std::map<Str, Str>      fieldValues() const;
    void                    setFieldValues(const std::map<Str, Str>& fieldValues);
    Str                     cloudViewerUrl() const;
    void                    setCloudViewerUrl(const Str& url);
    Str                     originalModelPath() const;
    void                    setOriginalModelPath(const Str& path);

    // Media type attributes
    const Image*                image() const;
    void                        setImage(Image* image);
    const ug::VTFxMemoryFile*   modelVTFx() const;
    void                        setModelVTFx(ug::VTFxMemoryFile* memoryFile);
    bool                        setModelVTFxFromFile(const Str& vtfxFileName);
    const Table*                table() const;
    void                        setTable(Table* table);


private:
    CEE_PRIVATE_IMPL(Snapshot);
    CEE_DISALLOW_COPY_AND_ASSIGN(Snapshot);
};

} // namespace rep
} // namespace cee

