//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"


// The default configuration is for consuming CDC as a dynamic library (DLL).
// To build DLL version of CDC, CEE_XXXX_BUILD_DLL_EXPORT must be defined and CEE_STATIC_LIBS must NOT be defined.
// To build or use CDC as static libraries, make sure CEE_STATIC_LIBS is defined.
#ifdef CEE_STATIC_LIBS
    #define CEE_UG_EXPORT
#else
    #if defined(WIN32)
        #ifdef CEE_UG_BUILD_DLL_EXPORT
            #define CEE_UG_EXPORT __declspec(dllexport)
        #else
            #define CEE_UG_EXPORT __declspec(dllimport)
        #endif
    #elif defined(__GNUC__)
        #ifdef CEE_UG_BUILD_DLL_EXPORT
            #define CEE_UG_EXPORT __attribute__ ((visibility ("default")))
        #else
            #define CEE_UG_EXPORT
        #endif
    #else
        #define CEE_UG_EXPORT
    #endif
#endif



namespace cee {
namespace ug {

/// Available scalar and vector result mappings
enum ResultMapping
{
    PER_NODE = 1,           ///< Per node
    PER_ELEMENT,            ///< Per element
    PER_ELEMENT_NODE,       ///< Per element node
    PER_ELEMENT_SURFACE,    ///< Per element surface
    UNKNOWN_MAPPING = 99
};

/// Available operations to derive a result from a base result
enum DeriveOperation
{
    NOT_DERIVED = 0,

    X_COMPONENT,                            ///< Derive X-component    Scalar result from Vector or Displacement results
    Y_COMPONENT,                            ///< Derive Y-component    Scalar result from Vector or Displacement results
    Z_COMPONENT,                            ///< Derive Z-component    Scalar result from Vector or Displacement results

    MAGNITUDE,                              ///< Derive Magnitude      Scalar result from Vector or Displacement results

    XX_COMPONENT,                           ///< Derive XX-component   Scalar result from SymmetricTensor result
    YY_COMPONENT,                           ///< Derive YY-component   Scalar result from SymmetricTensor result
    ZZ_COMPONENT,                           ///< Derive ZZ-component   Scalar result from SymmetricTensor result
    XY_COMPONENT,                           ///< Derive XY-component   Scalar result from SymmetricTensor result
    YZ_COMPONENT,                           ///< Derive YZ-component   Scalar result from SymmetricTensor result
    ZX_COMPONENT,                           ///< Derive ZX-component   Scalar result from SymmetricTensor result

    VON_MISES,                              ///< Derive VonMises       Scalar result from SymmetricTensor result
    FIRST_PRINCIPAL,                        ///< Derive 1st principal  Scalar result from SymmetricTensor result
    SECOND_PRINCIPAL,                       ///< Derive 2nd principal  Scalar result from SymmetricTensor result
    THIRD_PRINCIPAL,                        ///< Derive 3rd principal  Scalar result from SymmetricTensor result

    FIRST_PRINCIPAL_VECTOR,                 ///< Derive 1st principal  Vector result from SymmetricTensor result
    SECOND_PRINCIPAL_VECTOR,                ///< Derive 2nd principal  Vector result from SymmetricTensor result
    THIRD_PRINCIPAL_VECTOR,                 ///< Derive 3rd principal  Vector result from SymmetricTensor result

    DISPLACEMENT_VECTOR,                    ///< Derive displacement   Vector result from Displacement result

    UNKNOWN_DERIVE_OPERATION = 99
};


/// Different types of results
enum ResultType
{
	SCALAR = 1,             ///< Scalar result
	VECTOR,                 ///< Vector result
	DISPLACEMENT,           ///< Scalar result
	SYMMETRIC_TENSOR,       ///< Symmetric tensor result
	VISIBILITY,             ///< Result used to indicate element visibility
	ELEMENT,                ///< General element result
	NO_RESULT_TYPE,         ///< No type

    UNKNOWN_RESULT_TYPE = 99
};


/// Type of the strain result in the model, if any
enum StrainType
{
    ENGINEERING_STRAIN = 1, ///< Strain tensor components are engineering values (off-diagonal components are doubled)
    STANDARD_STRAIN,        ///< Standard strain tensor components
    NO_STRAIN_TYPE = 99     ///< No strain type, result is not a strain tensor
};



/// Type of the simulation run to produce the result database
enum SimulationType
{
    SIMULATION_TYPE_STRUCTURAL,
    SIMULATION_TYPE_THERMAL,
    SIMULATION_TYPE_ELECTRIC,
    SIMULATION_TYPE_MAGNETIC,
    SIMULATION_TYPE_FLUID,
    SIMULATION_TYPE_ACOUSTIC,
    SIMULATION_TYPE_DIFFUSION,
    SIMULATION_TYPE_OTHER
};


/// Type of the solution run to produce the result database
enum SolutionType
{
    SOLUTION_TYPE_STATIC,
    SOLUTION_TYPE_STEADY_STATE,
    SOLUTION_TYPE_VIBRATION,
    SOLUTION_TYPE_BUCKLING,
    SOLUTION_TYPE_TRANSIENT,
    SOLUTION_TYPE_SUPERELEMENT,
    SOLUTION_TYPE_FREQRESPONSE,
    SOLUTION_TYPE_COMPLEXEIGEN,
    SOLUTION_TYPE_QUASISTATIC,
    SOLUTION_TYPE_OUTPUT,
    SOLUTION_TYPE_SINGULAR,
    SOLUTION_TYPE_OTHER
};

}  // namespace ug
}  // namespace cee
