//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2017, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/UnstructGridModel.h"

namespace cee {
namespace ug {

//==================================================================================================
//
//
//
//==================================================================================================
class CEE_UG_EXPORT BulkValues
{
public:
    size_t globalGeometryIndex; ///< Geometry index of the part associated with the bulk values
    int partId;                 ///< Part id of the part associated with the bulk values
    
    double minimumValue;        ///< Minimum element scalar value in the region
    double maximumValue;        ///< Maxinum element scalar value in the region
    double averageValue;        ///< Average element scalar value in the region
    double resultant;           ///< Sum of SURFACE_AREA*SURFACE_RESULT_VALUE for all element surfaces in the region

    double volumeMinimumValue;  ///< The minimum result for volume elements in the part (only applicable for parts with volume elements)
    double volumeMaximumValue;  ///< The Maximum result for volume elements in the part (only applicable for parts with volume elements)
    double volumeAverageValue;  ///< The Average result for volume elements in the part (only applicable for parts with volume elements)
    double volumeResultant;     ///< Sum of VOLUME*ELEMENT_RESULT_VALUE for all volume elements (only applicable for parts with volume elements)
    
    double nodeMinimumValue;    ///< Minimum per surface node nodal scalar value in the region (only applicable for PER_NODE or PER_ELEMENT_NODE results)
    double nodeMaximumValue;    ///< Maxinum per surface node nodal scalar value in the region (only applicable for PER_NODE or PER_ELEMENT_NODE results)
    double nodeAverageValue;    ///< Average per surface node nodal scalar value in the region (only applicable for PER_NODE or PER_ELEMENT_NODE results)
    
    double volumeNodeMinimumValue; ///< Minimum per volume element node value (applicable for PER_NODE and PER_ELEMENT_NODE results)
    double volumeNodeMaximumValue; ///< Maximum per volume element node value (applicable for PER_NODE and PER_ELEMENT_NODE results)
    double volumeNodeAverageValue; ///< Average per volume element node value (applicable for PER_NODE and PER_ELEMENT_NODE results)

    double area;                ///< The surface area of the region
    double volume;              ///< The volume of the part (only applicable for parts with volume elements). For cutting planes: the volue of the intersected elements
    size_t surfaceCount;        ///< Number of surfaces in the region
    size_t nodeCount;           ///< Number of nodes in the region
    size_t elementCount;        ///< Number of elements. For cutting planes: The number of elements intersected

    Vec3d centroid;             ///< The centroid of the region
    Vec3d boundingBoxMinimum;   ///< The centroid of the region
    Vec3d boundingBoxMaximum;   ///< The centroid of the region

    BulkValues();
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT BulkCalculation
{
public:
    BulkCalculation(UnstructGridModel* model, size_t frameIndex);
    ~BulkCalculation();
    
    bool getCutplaneValues(size_t cuttingPlaneIndex, ResultType resultType, int resultId, std::vector<BulkValues>* cutplaneRegionArr);
    bool getPartValues(size_t globalGeometryIndex, int partId, ResultType resultType, int resultId, BulkValues* bulkValues);

private:
    CEE_PRIVATE_IMPL(BulkCalculation);
    CEE_DISALLOW_COPY_AND_ASSIGN(BulkCalculation);
};

}  // namespace ug
}  // namespace cee
