//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Str.h"


namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ColorMapper : public RefCountedObject
{
public:
    /// Color scheme used for the color mapper
    enum ColorScheme
    {
        NORMAL,                 ///< Normal
        NORMAL_INVERTED,        ///< Normal inverted
        BLACK_TO_WHITE,         ///< Black to white
        WHITE_TO_BLACK,         ///< White to black
        GREEN_TO_BROWN,         ///< Green to brown
        WHITE_TO_BROWN,         ///< White to brown
        METAL_CASTING,          ///< Metal casting
        BLUE_TO_WHITE_TO_RED,   ///< Blue to white to red
        THERMAL_1,              ///< Thermal 1
        THERMAL_2,              ///< Thermal 2
        THERMAL_3,              ///< Thermal 3
        CUSTOM                  ///< Custom
    };

    /// MappingType of color mapper
    enum MappingType
    {
        FILLED_CONTOURS_UNIFORM,    ///< Filled contours color mapping, discrete mapping with uniform size for the legend levels
        FILLED_CONTOURS,            ///< Filled contours color mapping, customized level values for each level
        CONTINUOUS                  ///< Color mapping using a continuous legend
    };

public:
    virtual ~ColorMapper();
    MappingType             type() const;
    void                    setTypeFilledContoursUniform(unsigned int numColors);
    void                    setTypeFilledContours(const std::vector<double>& values);
    void                    setTypeContinuous();

    ColorScheme             colorScheme() const;
    void                    setColorScheme(ColorScheme scheme);
    
    Color3f                 aboveRangeColor() const;
    void                    setAboveRangeColor(const Color3f& aboveColor);
    bool                    isAboveRangeMaskingEnabled() const;
    void                    enableAboveRangeMasking(bool enableMasking);

    Color3f                 belowRangeColor() const;
    void                    setBelowRangeColor(const Color3f& belowColor);
    bool                    isBelowRangeMaskingEnabled() const;
    void                    enableBelowRangeMasking(bool enableMasking);

    Color3f                 undefinedColor() const;
    void                    setUndefinedColor(const Color3f& undefinedColor);
    bool                    isUndefinedMaskingEnabled() const;
    void                    enableUndefinedMasking(bool enableMasking);
    
    void                    setTextureSize(unsigned int textureSize);

    // FILLED_CONTOURS_UNIFORM and FILLED_CONTOURS
    unsigned int            numberOfFilledContoursColors() const;
    std::vector<Color3f>    filledContoursColors() const;
    std::vector<double>     filledContoursLevelValues() const;
    void                    setCustomFilledContoursColors(const std::vector<Color3f>& levelColors);

    // CONTINUOUS
    std::vector<double>     customContinuousValues() const;
    std::vector<Color3f>    customContinuousColors() const;
    void                    setCustomContinuousColorsAtValues(const std::vector<double>& values, const std::vector<Color3f>& colorAtValues);
    std::vector<double>     customContinuousTickMarks() const;
    void                    setCustomContinuousTickMarks(const std::vector<double>& tickMarkValues);

private:
    ColorMapper();
    CEE_PRIVATE_F(ScalarSettings);
    CEE_PRIVATE_IMPL(ColorMapper);
    CEE_DISALLOW_COPY_AND_ASSIGN(ColorMapper);
};

}  // namespace ug
}  // namespace cee

