//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"

#include <vector>

namespace cee {
namespace ug {

class CuttingPlaneData;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT CuttingPlane : public RefCountedObject
{
public:
    /// Cutting plane surface draw styles
    enum DrawStyle
    {
        SURFACE,                    ///< Draw as surface
        SURFACE_MESH,               ///< Draw as surface mesh
        SURFACE_OUTLINE_MESH,       ///< Draw as outline mesh         
        LINE,                       ///< Draw as lines
        OUTLINE                     ///< Draw as outline
    };

    /// Vector grid spacing mode
    enum SpacingMode
    {
        ABSOLUTE_SPACING,           ///< Use absolute spacing
        RELATIVE_SPACING            ///< Use relative spacing
    };

public:
    CuttingPlane();
    virtual ~CuttingPlane();

    bool        visible() const;
    void        setVisible(bool visible);
    bool        highlight() const;
    void        setHighlight(bool showHighlighted);

    bool        planeSurfaceVisible() const;
    void        setPlaneSurfaceVisible(bool show);
    DrawStyle   drawStyle() const;
    void        setDrawStyle(DrawStyle drawStyle);
    Color3f     surfaceColor() const;
    void        setSurfaceColor(const Color3f& color);
    float       opacity() const;
    void        setOpacity(float opacity);
    float       specularIntensity() const;
    void        setSpecularIntensity(float intensity);
    bool        useSourcePartColor() const;
    void        setUseSourcePartColor(bool useSourcePartColor);
    bool        crinkleCutSurface() const;
    void        setCrinkleCutSurface(bool useCrinkleCutSurface);
    
    bool        clipping() const;
    void        setClipping(bool enableClipping);

    Vec3d       point() const;
    void        setPoint(Vec3d pointInPlane);
    Vec3d       normal() const;
    void        setNormal(Vec3d normal);
    bool        computeFromVisibleParts() const;
    void        setComputeFromVisibleParts(bool visiblePartsOnly);
    bool        alsoComputeFromUndeformedModel() const;
    void        setAlsoComputeFromUndeformedModel(bool alsoShowForUndeformedModel);

    int         mapScalarResultId() const;
    void        setMapScalarResultId(int resultId);
    int         mapVectorResultId() const;
    void        setMapVectorResultId(int resultId);
    int         mapContourLinesResultId() const;
    void        setMapContourLinesResultId(int resultId);

    double      vectorGridSpacing() const;
    SpacingMode vectorGridSpacingMode() const;
    void        setVectorGridSpacing(SpacingMode mode, double spacing);
    bool        gridLinesVisible() const;
    void        setGridLinesVisible(bool visible);
    bool        projectVectorsOnPlane() const;
    void        setProjectVectorsOnPlane(bool project);

    bool        isPrecomputed() const;

    double      sampleScalarValue(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;
    Vec3d       sampleVector(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;

    void        data(int stateId, CuttingPlaneData* cuttingPlaneData);
    void        partData(int stateId, size_t globalGeometryIndex, int partId, CuttingPlaneData* cuttingPlaneData);
    
    void        setPerFrameTransformationMatrices(const std::vector<Mat4d>& matrices);
    size_t      transformationMatrixFrameCount() const;
    Mat4d       frameTransformationMatrix(size_t frameIndex) const;
    

private:
    CEE_BASE_F(CuttingPlanePropertyHandler);
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(CuttingPlane);
    CEE_DISALLOW_COPY_AND_ASSIGN(CuttingPlane);
};

}  // namespace ug
}  // namespace cee
