//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataElementSetItem.h"
#include "CeeCore/RefCountedObject.h"

#include <vector>

namespace cee {
namespace ug {

class DataSourcePrivate;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataElementSet : public RefCountedObject
{
public:
    DataElementSet(int setId);
    DataElementSet(const DataElementSet& other, int setId);
    virtual ~DataElementSet();

    
    bool                isEqual(const DataElementSet& rhs) const;
    
    int                 id() const;
    
    size_t              itemCount() const;
    size_t              itemIndex(const DataElementSetItem& item) const;
    DataElementSetItem  item(size_t index) const;
    void                addItem(const DataElementSetItem& item);
    void                addItems(int geometryId, int partId, const std::vector<size_t>& elementIndices);
    
    void                removeItem(const DataElementSetItem& item);
    void                removeAllItems();

    std::vector<size_t> elementIndicesForPart(int geometryId, int partId) const;

    void                unionWith(const DataElementSet& other);
    void                intersect(const DataElementSet& other);
    void                subtract(const DataElementSet& other);
        
private:
    CEE_PRIVATE_F(DataSource);
    CEE_PRIVATE_F(DataElementSetGenerator);
    CEE_PRIVATE_F(DataElementSetBuilder);
    CEE_PRIVATE_IMPL(DataElementSet);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataElementSet);
};

}  // namespace ug
}  // namespace cee
