//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/DataElementSet.h"

#include "CeeVisualization/View.h"
#include "CeeCore/Plane.h"
#include "CeeCore/Vec2f.h"

namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataElementSetGenerator
{
public:
	/// Bit-mask used to specify which elements to include with regards to the plane. These values are then combined into the ElementPositionCriteria
	/// used in createFromPlane()
    enum ElementPositionCriterion
    {
        FRONT   = 0x01,                 ///< Constant indicating in front of plane
        ON      = 0x02,                 ///< Constant indicating on the plane
        BACK    = 0x04                  ///< Constant indicating behind plane
    };
    
    typedef int ElementPositionCriteria;		///< Bit-mask made up of ElementPositionCriterion which specifies which elements will be accepted based on the corresponding plane.

public:
    DataElementSetGenerator(const UnstructGridModel& model, size_t frameIndex);
    ~DataElementSetGenerator();
    
    PtrRef<DataElementSet>  createFromRegion(int setId, int x, int y, unsigned int width, unsigned int height, const vis::View& view, bool includePartiallyContainedElements);
    PtrRef<DataElementSet>  createFromPolygon(int setId, const std::vector<Vec2f>& polygonInWindowCoordinates, const vis::View& view, bool includePartiallyContainedElements);
    PtrRef<DataElementSet>  createFromPlane(int setId, const Plane& plane, ElementPositionCriteria criteria);
	PtrRef<DataElementSet>  createFromPlanes(int setId, const std::vector<Plane>& planes, ElementPositionCriteria criteria);
	PtrRef<DataElementSet>  createAllVisibleElements(int setId);
    PtrRef<DataElementSet>  createAllElementsInPart(int setId, size_t geometryIndex, int partId);

    
private:
    CEE_PRIVATE_IMPL(DataElementSetGenerator);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataElementSetGenerator);
};

}  // namespace ug
}  // namespace cee
