//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeUnstructGrid/Element.h"

#include <vector>

namespace cee {
namespace ug {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataElements : public RefCountedObject
{
public:
    DataElements(bool withElementIds, size_t userPropertyCount);
    virtual ~DataElements();

    size_t                      elementCount() const;
    Element::Type               elementType(size_t elementIndex) const;
    Element::Type               singleElementType() const;
    std::vector<unsigned int>   elementNodes(size_t elementIndex) const;
    void                        reserve(size_t elementCount, size_t elementNodeCount);
    size_t                      addElement(Element::Type elementType, const std::vector<unsigned int>& elementNodes);
    size_t                      addElement(Element::Type elementType, const unsigned int elementNodes[], size_t elementNodeCount);
    size_t                      addElementInt(Element::Type elementType, const int elementNodes[], size_t elementNodeCount);
    void                        addElements(Element::Type elementType, size_t elementCount, const std::vector<unsigned int>& elementNodes);
    void                        addElements(Element::Type elementType, size_t elementCount, const unsigned int elementNodes[], size_t totalElementNodeCount);
    void                        addElementsInt(Element::Type elementType, size_t elementCount, const int elementNodes[], size_t totalElementNodeCount);

    void                        removeAllElements();

    bool                        hasElementIds() const;
    int                         elementId(size_t elementIndex) const;
    void                        setElementId(size_t elementIndex, int elementId);

    size_t                      elementIndex(int elementId) const;

    size_t                      userPropertyCount() const;
    void                        setUserProperty(size_t userPropertyIndex, size_t elementIndex, int userProperty);
    int                         userProperty(size_t userPropertyIndex, size_t elementIndex) const;

    size_t                      elementNodeCount() const;
    size_t                      elementSurfaceCount() const;

    size_t                      nodesPerElement(size_t elementIndex) const;
    size_t                      surfacesPerElement(size_t elementIndex) const;
    size_t                      edgesPerElement(size_t elementIndex) const;

private:
    CEE_PRIVATE_F(DataPart);
    CEE_PRIVATE_F(ElementsQuery);
    CEE_PRIVATE_IMPL(DataElements);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataElements);
};


}  // namespace ug
}  // namespace cee
