//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataNodeSetItem.h"
#include "CeeCore/RefCountedObject.h"

#include <vector>

namespace cee {
namespace ug {

class DataSourcePrivate;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataNodeSet : public RefCountedObject
{
public:
    DataNodeSet(int setId);
    virtual ~DataNodeSet();
    
    bool                isEqual(const DataNodeSet& rhs) const;
    
    int                 id() const;
    
    size_t              itemCount() const;
    size_t              itemIndex(const DataNodeSetItem& item) const;
    DataNodeSetItem     item(size_t index) const;
    void                addItem(const DataNodeSetItem& item);
    void                addItems(int geometryId, int partId, const std::vector<size_t>& nodeIndices);
    
    void                removeItem(const DataNodeSetItem& item);
    void                removeAllItems();
    
    void                unionWith(const DataNodeSet& other);
    void                intersect(const DataNodeSet& other);
    void                subtract(const DataNodeSet& other);
        
private:
    CEE_PRIVATE_F(DataSource);
    CEE_PRIVATE_F(DataNodeSetGenerator);
    CEE_PRIVATE_IMPL(DataNodeSet);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataNodeSet);
};

}  // namespace ug
}  // namespace cee
