//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/DataNodeSet.h"

#include "CeeVisualization/View.h"
#include "CeeCore/Plane.h"
#include "CeeCore/Vec2f.h"

namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataNodeSetGenerator
{
public:
	/// Bit-mask used to specify which nodes to include with regards to the plane. These values are then combined into the NodePositionCriteria
	/// used in createFromPlane()
    enum NodePositionCriterion
    {
        FRONT   = 0x01,                 ///< Constant indicating in front of plane
        ON      = 0x02,                 ///< Constant indicating on the plane
        BACK    = 0x04                  ///< Constant indicating behind plane
    };
    
    typedef int NodePositionCriteria;		///< Bit-mask made up of NodePositionCriterion which specifies which nodes will be accepted based on the corresponding plane.

public:
    DataNodeSetGenerator(const UnstructGridModel& model, size_t frameIndex);
    ~DataNodeSetGenerator();
    
    PtrRef<DataNodeSet>  createFromRegion(int setId, int x, int y, unsigned int width, unsigned int height, const vis::View& view);
    PtrRef<DataNodeSet>  createFromPolygon(int setId, const std::vector<Vec2f>& polygonInWindowCoordinates, const vis::View& view);
    PtrRef<DataNodeSet>  createFromPlane(int setId, const Plane& plane, NodePositionCriteria criteria);
    PtrRef<DataNodeSet>  createAllNodes(int setId);
    
private:
    CEE_PRIVATE_IMPL(DataNodeSetGenerator);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataNodeSetGenerator);
};

}  // namespace ug
}  // namespace cee
