//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Vec3d.h"

#include <vector>


namespace cee {
namespace ug {


//==================================================================================================
//
// DataNodes stores the nodes (optionally with ID) 
//
//==================================================================================================
class CEE_UG_EXPORT DataNodes : public RefCountedObject
{
public:
    DataNodes(bool hasNodeIds);
    virtual ~DataNodes();

    size_t  nodeCount() const;
    Vec3d   node(size_t index) const;

    void    resize(size_t nodeCount);
    void    setNode(size_t index, const Vec3d& node);

    void    setNodes(const std::vector<Vec3d>& nodes);
    void    setNodes(const double nodes[], size_t nodeCount);
    void    setNodesFloat(const float nodes[], size_t nodeCount);

    bool    hasNodeIds() const;
   
    int     nodeId(size_t index) const;

    size_t  nodeIndex(int nodeId) const;

    void    setNodeId(size_t index, int nodeId);
    void    setNodeIds(const std::vector<int>& nodeIds);
    void    setNodeIds(const int nodeIds[], size_t nodeCount);

    const double*   rawNodesPointer() const;
    const int*      rawIdsPointer() const;

private:
    CEE_PRIVATE_F(DataPart);
    CEE_PRIVATE_F(ElementsQuery);
    CEE_PRIVATE_IMPL(DataNodes);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataNodes);
};


}  // namespace ug
}  // namespace cee
