//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeUnstructGrid/Error.h"
#include "CeeUnstructGrid/DataResultGroup.h"
#include "CeeUnstructGrid/DataSourceDirectory.h"

namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataReader : public RefCountedObject
{
public:
    DataReader();
    virtual ~DataReader();
    
    virtual bool    isSupportedFileFormat(const Str& filename) = 0;
    
    virtual bool    open(const Str& filename, Error* error) = 0;
    virtual void    close() = 0;
    
    virtual size_t  geometryCount() const = 0;
    virtual bool    initializeDirectory(DataSourceDirectory* directory, Error* error) = 0;
    virtual bool    hasNewGeometryForEachState(size_t globalGeometryIndex) const;
    
    virtual bool    readGeometry(int stateId, size_t geometryIndex, DataGeometry* geometry, Error* error) = 0;
    virtual bool    readScalarResult(int stateId, size_t geometryIndex, int resultId, DataResultScalar* scalarResult, Error* error);       
    virtual bool    readVectorResult(int stateId, size_t geometryIndex, int resultId, DataResultVector* vectorResult, Error* error);       
    virtual bool    readDisplacementResult(int stateId, size_t geometryIndex, int resultId, DataResultDisplacement* displacementResult, Error* error); 
    virtual bool    readSymmetricTensorResult(int stateId, size_t geometryIndex, int resultId, DataResultSymmetricTensor* symmetricTensorResult, Error* error); 
    virtual bool    readTransformationResult(int stateId, size_t geometryIndex, DataResultTransformation* transformationResult, Error* error);

protected:

private:
    CEE_DISALLOW_COPY_AND_ASSIGN(DataReader);
};

}  // namespace ug
}  // namespace cee
