//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataGeometry.h"
#include "CeeUnstructGrid/DataResultScalar.h"
#include "CeeUnstructGrid/DataResultVector.h"
#include "CeeUnstructGrid/DataResultDisplacement.h"
#include "CeeUnstructGrid/DataResultSymmetricTensor.h"
#include "CeeUnstructGrid/DataResultTransformation.h"
#include "CeeUnstructGrid/DataResultVisibility.h"

namespace cee {
namespace ug {


//==================================================================================================
//
// Stores the mapping between a geometry and its results for a given DataState
//
//==================================================================================================
class CEE_UG_EXPORT DataResultGroup : public RefCountedObject
{
public:
    virtual ~DataResultGroup();

    const DataGeometry*                 geometry() const;

    size_t                              scalarCount() const;
    const DataResultScalar*             scalar(size_t resultIndex) const;
    DataResultScalar*                   scalar(size_t resultIndex);
    size_t                              scalarIndex(int resultId) const;
    void                                addScalar(DataResultScalar* scalarResult);
    void                                removeAllScalars();

    size_t                              vectorCount() const;
    const DataResultVector*             vector(size_t resultIndex) const;
    DataResultVector*                   vector(size_t resultIndex);
    size_t                              vectorIndex(int resultId) const;
    void                                addVector(DataResultVector* vectorResult);
    void                                removeAllVectors();

    size_t                              displacementCount() const;
    const DataResultDisplacement*       displacement(size_t resultIndex) const;
    DataResultDisplacement*             displacement(size_t resultIndex);
    size_t                              displacementIndex(int resultId) const;
    void                                addDisplacement(DataResultDisplacement* displacementResult);
    void                                removeAllDisplacements();

    size_t                              symmetricTensorCount() const;
    const DataResultSymmetricTensor*    symmetricTensor(size_t resultIndex) const;
    DataResultSymmetricTensor*          symmetricTensor(size_t resultIndex);
    size_t                              symmetricTensorIndex(int resultId) const;
    void                                addSymmetricTensor(DataResultSymmetricTensor* symmetricTensorResult);
    void                                removeAllSymmetricTensors();

    const DataResultTransformation*     transformation() const;
    DataResultTransformation*           transformation();
    void                                setTransformation(DataResultTransformation* transformationResult);
    void                                removeTransformation();

    const DataResultVisibility*         visibility() const;
    DataResultVisibility*               visibility();
    void                                setVisibility(DataResultVisibility* visibilityResult);
    void                                removeVisibility();

private:
    DataResultGroup(DataGeometry* geometry);

private:
    CEE_PRIVATE_F(DataState);
    CEE_PRIVATE_F(DataSource);
    CEE_PRIVATE_IMPL(DataResultGroup);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataResultGroup);
};


}  // namespace ug
}  // namespace cee
