//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataState.h"
#include "CeeUnstructGrid/DataStateSpec.h"
#include "CeeUnstructGrid/DataSourceDirectory.h"

#include "CeeCore/Str.h"

class VTDatabase;

namespace cee {
namespace ug {

class UnstructGridModel;
class DataElementSet;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataSource : public RefCountedObject
{
public:
    virtual ~DataSource();

    int                         id() const;
    size_t                      stateCount() const;
    size_t                      stateIndex(int stateId) const;
    size_t                      geometryCountPerState() const;

    const DataState*            state(size_t stateIndex) const;
    DataState*                  state(size_t stateIndex);
    const DataState*            currentState(const UnstructGridModel* model) const;
    DataState*                  currentState(const UnstructGridModel* model);
    
    const DataSourceDirectory*  directory() const;
    DataSourceDirectory*        directory();
    
    const DataElementSet*       elementSet(size_t setIndex) const;
    DataElementSet*             elementSet(size_t setIndex);
    size_t                      elementSetCount() const;
    size_t                      elementSetIndex(int setId) const;
    void                        addElementSet(DataElementSet* elementSet);
    void                        removeElementSet(const DataElementSet* elementSet);
    void                        removeAllElementSets();
    
    bool                        isDataSourceValid(Str* failReason) const;
    
    int                         defaultDispacementResultId() const;

    bool                        computeCustomResults(int stateId, const DataStateSpec& stateSpec);

public:
    VTDatabase*                 internal_database();
    const VTDatabase*           internal_database() const;
    void                        internal_setDatabase(VTDatabase* vtDB);
    void                        internal_setGeometryCount(size_t geometryCount);

private:
    DataSource(int dataSourceId);

    virtual bool                        loadCustomResultBaseDependencies(int stateId, const DataStateSpec& stateSpec) = 0;
    bool                                computeCustomResult(int stateId, const ResultInfo& info);
    std::vector<CustomResultArgument>   customResultArguments(int stateId, const ResultInfo& info) const;

    virtual void                internal_wasAssignedToModel(ug::UnstructGridModel* model);

private:
    CEE_BASE_F(DataSourceInterface);
    CEE_BASE_F(DataSourceReader);
    CEE_BASE_F(DataSourceMemory);
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_F(ResultsQuery);

    CEE_PRIVATE_IMPL(DataSource);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataSource);

};

}  // namespace ug
}  // namespace cee
