//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"

#include "CeeUnstructGrid/PartInfo.h"
#include "CeeUnstructGrid/ResultInfo.h"
#include "CeeUnstructGrid/SectionInfo.h"
#include "CeeUnstructGrid/SetInfo.h"
#include "CeeUnstructGrid/StateInfo.h"
#include "CeeUnstructGrid/StateGroupInfo.h"
#include "CeeUnstructGrid/SimulationInfo.h"

#include <string>
#include <vector>


namespace cee {
namespace ug {


//==================================================================================================
//
/// 
//
//==================================================================================================
class CEE_UG_EXPORT DataSourceDirectory : public RefCountedObject
{
public:
    virtual ~DataSourceDirectory();

    void                        clear();
    size_t                      geometryCountPerState() const;

    SimulationInfo              simulationInfo() const;

    std::vector<StateInfo>      stateInfos() const;
    std::vector<StateGroupInfo> stateGroupInfos() const;
    std::vector<StateInfo>      ungroupedStateInfos() const;

    std::vector<ResultInfo>     scalarResultInfos() const;
    std::vector<ResultInfo>     vectorResultInfos() const;
    std::vector<ResultInfo>     displacementResultInfos() const;
    std::vector<ResultInfo>     symmetricTensorResultInfos() const;
    std::vector<PartInfo>       partInfos(size_t globalGeometryIndex) const;
    bool                        transformationResult() const;
    bool                        visibilityResult() const;
	std::vector<SetInfo>		setInfos() const;

    std::vector<SectionInfo>            sectionInfos() const;
    std::vector<SectionCategoryInfo>    sectionCategoryInfos() const;
    std::vector<SectionGroupInfo>       sectionGroupInfos() const;


    std::vector<ResultInfo>     resultInfos(ResultType resultType) const;
    std::vector<Str>            resultNatures(ResultType resultType) const;

    StateInfo                   findStateInfo(int stateId) const;
    StateGroupInfo              findStateGroupInfo(int stateGroupId) const;

    ResultInfo                  findResultInfo(ResultType resultType, int resultId) const;
    ResultInfo                  findScalarResultInfo(int resultId) const;
    ResultInfo                  findVectorResultInfo(int resultId) const;
    ResultInfo                  findDisplacementResultInfo(int resultId) const;
    ResultInfo                  findSymmetricTensorResultInfo(int resultId) const;
    
    ResultInfo                  findResultInfo(ResultType resultType, const Str& resultIdString) const;
    ResultInfo                  findScalarResultInfo(const Str& resultIdString) const;
    ResultInfo                  findVectorResultInfo(const Str& resultIdString) const;
    ResultInfo                  findDisplacementResultInfo(const Str& resultIdString) const;
    ResultInfo                  findSymmetricTensorResultInfo(const Str& resultIdString) const;

    PartInfo                    findPartInfo(size_t globalGeometryIndex, int partId) const;
	SetInfo						findSetInfo(int setId) const;
    SectionInfo                 findSectionInfo(int sectiondId) const;
    SectionCategoryInfo         findSectionCategoryInfo(int categoryId) const;
    SectionGroupInfo            findSectionGroupInfo(int sectiondGroupId) const;

    std::vector<ResultInfo>     derivedScalarResultInfos(int resultId) const;
    std::vector<ResultInfo>     derivedVectorResultInfos(int resultId) const;

    bool                        hasResult(int stateId, ResultInfo info) const;
    bool                        hasScalarResult(int stateId, int resultId) const;
    bool                        hasVectorResult(int stateId, int resultId) const;
    bool                        hasSymmetricTensorResult(int stateId, int resultId) const;
    bool                        hasDisplacementResult(int stateId, int resultId) const;

    void                        setStateGroupInfo(StateGroupInfo info);
    void                        removeAllStateGroupInfos();

    void                        setStateInfo(StateInfo info);
    void                        addStateInfo(StateInfo info);
    void                        removeAllStateInfos();

	void                        setResultInfo(ResultInfo resultInfo);
	void                        removeAllResultInfos(ResultType resultType);

    void                        setScalarResultInfo(ResultInfo resultInfo); 
    void                        removeAllScalarResultInfos();

    void                        setVectorResultInfo(ResultInfo resultInfo);
    void                        removeAllVectorResultInfos();

    void                        setDisplacementResultInfo(ResultInfo resultInfo);
    void                        removeAllDisplacementResultInfos();

    void                        setSymmetricTensorResultInfo(ResultInfo resultInfo);
    void                        removeAllSymmetricTensorResultInfos();

    void                        setPartInfo(size_t globalGeometryIndex, PartInfo partInfo);
    void                        removeAllPartInfos(size_t globalGeometryIndex);

	void						setSetInfo(SetInfo setInfo);
	void						removeSetInfo(int setId);
	void						removeAllSetInfos();

    void                        setTransformationResult(bool hasTransformationResult);
    void                        setVisibilityResult(bool hasVisibilityResult);

    std::vector<ResultInfo>     customResultInfos() const;
    void                        setCustomResultInfo(ResultInfo resultInfo);
    void                        removeCustomResultInfo(ResultInfo resultInfo);

    std::vector<ResultInfo>     customResultBaseDependencies(ResultInfo resultInfo) const;

	// Set info!

private:
    DataSourceDirectory();

private:
    CEE_PRIVATE_F(DataSource);
    CEE_PRIVATE_IMPL(DataSourceDirectory);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataSourceDirectory);
};

}  // namespace ug
}  // namespace cee
