//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataSource.h"
#include "CeeUnstructGrid/DataStateSpec.h"
#include "CeeUnstructGrid/Error.h"
#include "CeeUnstructGrid/ResultCalculatorParams.h"

class VTDatabase;
class VTResultCalculatorRegistry;

namespace cee {
namespace ug {

class ResultCalculatorParams;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataSourceInterface : public DataSource
{
public:
    DataSourceInterface(int dataSourceId);
    virtual ~DataSourceInterface();
    
    virtual bool    open(const Str& filename, Error* error = NULL) = 0;
    virtual void    close() = 0;
    
    virtual bool    loadState(int stateId, const DataStateSpec& stateSpec, Error* error);
    virtual void    discardState(int stateId);
    virtual bool    loadCustomResultBaseDependencies(int stateId, const DataStateSpec& stateSpec);

    static bool     supportsResultCalculatorPlugins();
    static bool     loadResultCalculatorPlugin(const Str& baseFileName);
    
    bool            createResultCalculator(const Str& calculatorId, const Str& resultIdString, const Str& initString, Str* errorMessage);
    bool            recreateResultCalculator(const Str& calculatorId, const Str& resultIdString, const Str& initString, Str* errorMessage);
    bool            deleteResultCalculator(const Str& calculatorId, const Str& resultIdString);

    bool            setResultCalculatorParams(const Str& calculatorId, const Str& resultIdString, const ResultCalculatorParams& calcParams);
    void            getResultCalculatorParams(const Str& calculatorId, const Str& resultIdString, ResultCalculatorParams* calcParams) const;

    Str             primaryFilename() const;
    Str             description() const;
    
    virtual bool    scalarRange(int stateId, int resultId, double* min, double* max);
    virtual bool    vectorRange(int stateId, int resultId, double* min, double* max);
    virtual bool    displacementRange(int stateId, int resultId, double* min, double* max);

protected:
    void                                    updateDirectoryFromDatabase();

    void                                    internal_setDatabase(VTDatabase* database);
    static VTResultCalculatorRegistry*      internal_resultCalculatorRegistryInstance();

private:
    CEE_DISALLOW_COPY_AND_ASSIGN(DataSourceInterface);
};


}  // namespace ug
}  // namespace cee
