//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataSourceInterface.h"

#include "CeeCore/Str.h"

namespace cee {
class PropertySetCollection;
class ImageResources;
class Image;

namespace ug {

class Error;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataSourceVTF : public DataSourceInterface
{
public:
    DataSourceVTF(int dataSourceId);
    virtual ~DataSourceVTF();

    static bool     isVTFFile(const Str& filename);
    static bool     snapshotImage(const Str& filename, Image* image);
    static bool     isExpressFile(const Str& filename);

    bool            isOpen() const;
    virtual bool    open(const Str& fileName, Error* error = NULL);
    virtual void    close();

    bool            fileProperties(PropertySetCollection* propSetCollection, ImageResources* resources);
    int             vendorId() const;

    static size_t   plotDataCount(const Str& filename);
    static bool     read2dPlotData(const Str& filename, size_t plotDataIndex, std::vector<Str>* variableNames, std::vector<Str>* variableUnits, std::vector<std::vector<double> >* variableData);

    static Str      fileContentsInfo(const Str& filename);

private:
    virtual void    internal_wasAssignedToModel(ug::UnstructGridModel* model);

    CEE_DISALLOW_COPY_AND_ASSIGN(DataSourceVTF);
};


}  // namespace ug
}  // namespace cee
