//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/Str.h"

#include "CeeUnstructGrid/DataSourceInterface.h"

namespace cee {

class PropertySetCollection;
class ImageResources;

namespace ug {

class VTFxMemoryFile;
class UnstructGridModel;
class Error;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataSourceVTFx : public DataSourceInterface
{
public:
    DataSourceVTFx(int dataSourceId);
    virtual ~DataSourceVTFx();

    bool                    isOpen() const;

    bool                    openCase(const Str& fileName, size_t caseIndex, Error* error = NULL);
    bool                    openCase(const VTFxMemoryFile* vtfxMemFile, size_t caseIndex, Error* error = NULL);
    bool                    openCaseById(const Str& fileName, int caseId, Error* error = NULL);
    bool                    caseProperties(PropertySetCollection* propSetCollection, ImageResources* resources);
    bool                    isDisplayModelOnly();
    int                     caseId() const;
    Str                     caseName() const;
    Str                     caseDescription() const;

    void                    setReadShellsOnly(bool readSurfaceElementOnly);
    void                    setPassword(const Str& password);

    size_t                  plotDataCount() const;
    Str                     plotDataName(size_t plotIndex);
    size_t                  plotDataVariableCount(size_t plotIndex);
    Str                     plotDataVariableName(size_t plotIndex, size_t plotVariableIndex);
    std::vector<double>     plotDataVariableValues(size_t plotIndex, size_t plotVariableIndex);

    virtual bool            open(const Str& filename, Error* error = NULL);
    virtual void            close();

    static bool             isVTFxFile(const Str& filename);
                

private:
    virtual void            internal_wasAssignedToModel(ug::UnstructGridModel* model);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataSourceVTFx);

private:
    Str                     m_password;
    bool                    m_readShellsOnly;
};



}  // namespace ug
}  // namespace cee
