//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/DataGeometry.h"
#include "CeeUnstructGrid/DataResultGroup.h"

namespace cee {
namespace ug {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT DataState : public RefCountedObject
{
public:
    DataState(int id, size_t geometryCount);
    virtual ~DataState();

    int                     id() const;

    size_t                  geometryCount() const;
    const DataGeometry*     geometry(size_t globalGeometryIndex) const;
    DataGeometry*           geometry(size_t globalGeometryIndex);
    const DataGeometry*     firstGeometry() const;
    DataGeometry*           firstGeometry();
    void                    setGeometry(size_t globalGeometryIndex, DataGeometry* geometry);

    const DataResultGroup*  results(const DataGeometry* geometry) const;
    DataResultGroup*        results(const DataGeometry* geometry);
    const DataResultGroup*  resultsFirstGeometry() const;
    DataResultGroup*        resultsFirstGeometry();

    const DataPart*         findPart(size_t globalGeometryIndex, int partId) const;
    DataPart*               findPart(size_t globalGeometryIndex, int partId);

private:
    CEE_PRIVATE_F(DataSource);
    CEE_PRIVATE_IMPL(DataState);
    CEE_DISALLOW_COPY_AND_ASSIGN(DataState);
};


}  // namespace ug
}  // namespace cee
