//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"

#include <vector>

namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT Element
{
public:
    enum Type
    {
        POINTS = 0,
        BEAMS,
        BEAMS_3,
        TRIANGLES,
        TRIANGLES_6,
        QUADS,
        QUADS_8,
        QUADS_9,
        TETRAHEDRONS,
        TETRAHEDRONS_10,
        PYRAMIDS,
        PYRAMIDS_13,
        PENTAHEDRONS,
        PENTAHEDRONS_15,
        HEXAHEDRONS,
        HEXAHEDRONS_20,
        POLYHEDRONS,
        NULL_ELEMENT
    };

    enum Category
    {
        CATEGORY_SOLID,         
        CATEGORY_SHELL,         
        CATEGORY_MEMBRANE,      
        CATEGORY_BEAM,          
        CATEGORY_TRUSS,         
        CATEGORY_INFINITE,      
        CATEGORY_GAP,           
        CATEGORY_JOINT,         
        CATEGORY_SPRINGDASHPOT, 
        CATEGORY_RIGID,         
        CATEGORY_CONSTRAINT,    
        CATEGORY_MASS,          
        CATEGORY_INTERFACE,     
        CATEGORY_SUPER,         
        CATEGORY_UNDEFINED		
    };

public:
    static size_t                       nodesPerElement(Type elementType);
    static bool                         isVolumeElement(Type elementType);

    static size_t                       surfacesPerElement(Type elementType);
    static std::vector<unsigned int>    elementSurface(Type elementType, size_t surfaceIndex); 

    static size_t 	                    edgesPerElement(Type elementType);
    static std::vector<unsigned int>    elementEdge(Type elementType, size_t edgeIndex); 
};

}  // namespace ug
}  // namespace cee
