//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"

#include "CeeVisualization/MarkupPartLabels.h"

#include <vector>

namespace cee {

namespace vis { class MarkupModel; }

namespace ug {
class UnstructGridModel;
class HitItem;
class DataElements;
class DataNodes;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ElementHighlighter
{
public:
    ElementHighlighter(const UnstructGridModel* model, vis::MarkupModel* markupModel);
    ~ElementHighlighter();

    void                    addElementHighlight(const HitItem& item, const Color3f& surfaceColor);
    void                    addElementHighlight(const HitItem& item, const Color3f& surfaceColor, const Color3f& lineColor);
    void                    addElementLabel(const HitItem& item, vis::MarkupPartLabels::LabelDrawMode labelDrawMode = vis::MarkupPartLabels::DRAW_WITHOUT_ZBUFFER_TEST, cee::vis::Font* font = NULL);
    void                    addElementNodeLabels(const HitItem& item, vis::MarkupPartLabels::LabelDrawMode labelDrawMode = vis::MarkupPartLabels::DRAW_WITHOUT_ZBUFFER_TEST, cee::vis::Font* font = NULL);

    void                    addElementSurfaceHighlight(const HitItem& item, const Color3f& surfaceColor);
    void                    addElementSurfaceHighlight(const HitItem& item, const Color3f& surfaceColor, const Color3f& lineColor);
    void                    addElementSurfaceLabel(const HitItem& item, vis::MarkupPartLabels::LabelDrawMode labelDrawMode, cee::vis::Font* font);
    void                    addElementSurfaceNodeLabels(const HitItem& item, vis::MarkupPartLabels::LabelDrawMode labelDrawMode, cee::vis::Font* font);


    Vec3d                   elementLabelPosition(const HitItem& item) const;
    const ug::DataElements* elementsFromHitItem(const HitItem& item) const;
    const ug::DataNodes*    nodesFromHitItem(const HitItem& item) const;

private:
    CEE_PRIVATE_IMPL(ElementHighlighter);
    CEE_DISALLOW_COPY_AND_ASSIGN(ElementHighlighter);
};

}  // namespace ug
}  // namespace cee
