//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Vec3d.h"

#include <vector>

namespace cee {
namespace ug {
class DataElements;
class DataNodes;
class DataPartDisplacement;
class DataPartScalar;
class UnstructGridModel;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ElementsQuery
{
public:
    ElementsQuery(const DataElements* elements, const DataNodes* nodes);
    ElementsQuery(const DataElements* elements, const DataPartDisplacement* displacements);
    ElementsQuery(const UnstructGridModel* model, size_t frameIndex, size_t geometryIndex, int partId);
    ~ElementsQuery();

    void                        setDisplacements(const DataPartDisplacement* displacements);
    
    Vec3d                       nodeCoordinate(size_t nodeIndex);

    Vec3d                       elementCentroid(size_t elementIndex) const;
    std::vector<Vec3d>          elementCoordinates(size_t elementIndex) const;
    std::vector<unsigned int>   elementVertices(size_t elementIndex) const;
    bool                        elementContainingCoordinate(const Vec3d& coordinate, size_t* elementIndex);
    bool                        elementsContainingEdge(size_t nodeIndex1, size_t nodeIndex2, std::vector<size_t>* elementIndices);
    bool                        elementsContainingNode(size_t nodeIndex1, std::vector<size_t>* elementIndices);
    std::vector<unsigned int>   elementEdges(size_t elementIndex);
    double                      elementVolume(size_t elementIndex) const;
    std::vector<double>         elementVolumes() const;
    void                        allElementCentroids(std::vector<double>* values) const;

    Vec3d                       elementSurfaceCentroid(size_t elementIndex, size_t localSurfaceIndex) const;
    std::vector<Vec3d>          elementSurfaceCoordinates(size_t elementIndex, size_t localSurfaceIndex) const;
    std::vector<unsigned int>   elementSurfaceVertices(size_t elementIndex, size_t localSurfaceIndex) const;
    std::vector<Vec3d>          elementSurfaceTriangles(size_t elementIndex, size_t localSurfaceIndex) const;

    size_t                      nodeIndexFromElementNodeIndex(size_t elementNodeIdx) const;
    size_t                      elementNodeFromElementAndNodeIndex(size_t elementIndex, size_t nodeIndex) const;
    size_t                      elementLocalNodeIndexFromElementAndNodeIndex(size_t elementIndex, size_t nodeIndex) const; 

    size_t                      elementStartIndex(size_t elementIndex) const;
    void                        elementAndSurfaceIndexFromElementSurfaceIndex(size_t elementSurfaceIndex, size_t* elementIndex, size_t* localSurfaceIndex) const;
    size_t                      elementSurfaceIndex(size_t elementIndex, size_t localSurfaceIndex) const;
    bool                        elementNeighbor(size_t elementIndex, size_t localSurfaceIndex, size_t* neighborElementIndex = NULL, size_t* neighborLocalSurfaceIndex = NULL) const;

    double                      mapScalarResult(size_t elementIndex, const Vec3d& coordinate, const DataPartScalar* scalarPart, ResultMapping scalarResultMappingType) const;
    void                        mapScalarResults(const std::vector<Vec3d>& coords, const DataPartScalar* scalarPart, ResultMapping scalarResultMappingType, std::vector<double>* mappedResultValues);

private:
    CEE_PRIVATE_IMPL(ElementsQuery);
    CEE_DISALLOW_COPY_AND_ASSIGN(ElementsQuery);
};

}  // namespace ug
}  // namespace cee
