//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Vec3d.h"

namespace cee {
namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT HitItem 
{
public:
    /// Item types for hit items
    enum ItemType
    {
        PART,           ///< The hit item is a Part 
        CUTTING_PLANE,  ///< The hit item is a Cutting plane 
        ISOSURFACE,     ///< The hit item is an %Isosurface
        ISOVOLUME       ///< The hit item is an %Isovolume
    };

public:
    HitItem();
    HitItem(const HitItem& other);
    ~HitItem();

    HitItem& operator=(const HitItem& other);

    ItemType    itemType() const;
    void        setItemType(ItemType itemType);
    
    int         itemId() const;
    void        setItemId(int itemId);

    size_t      itemIndex() const;
    void        setItemIndex(size_t itemIndex);

    int         stateId() const;
    void        setStateId(int stateId);

    size_t      frameIndex() const;
    void        setFrameIndex(size_t frameIndex);
    
    size_t      geometryIndex() const;
    void        setGeometryIndex(size_t geometryIndex);

    int         partId() const;
    void        setPartId(int partId);
    
    size_t      elementIndex() const;
    void        setElementIndex(size_t elementIndex);

    size_t      nodeIndex() const;
    void        setNodeIndex(size_t nodeIndex);

    size_t      elementLocalNodeIndex() const;
    void        setElementLocalNodeIndex(size_t localNodeIndex);

    size_t      elementLocalSurfaceIndex() const;
    void        setElementLocalSurfaceIndex(size_t surfaceIndex);

    size_t      edgeNodeStartIndex() const;
    void        setEdgeNodeStartIndex(size_t nodeIndex);

    size_t      edgeNodeEndIndex() const;
    void        setEdgeNodeEndIndex(size_t nodeIndex);

    Vec3d       intersectionPoint() const;
    void        setIntersectionPoint(const Vec3d& intersectionPoint);

    Vec3d       intersectionPointNormal() const;
    void        setIntersectionPointNormal(const Vec3d& intersectionPointNormal);
    
    size_t      triangleIndex() const;
    void        setTriangleIndex(size_t triangleIndex);
    
    double      distanceAlongRay() const;
    void        setDistanceAlongRay(double distance);

private:
    ItemType    m_itemType;
    int         m_stateId;
    size_t      m_frameIndex;
    size_t      m_geometryIndex;
    int         m_partId;
    int         m_itemId;
    size_t      m_itemIndex;
    size_t      m_elementIndex;
    size_t      m_nodeIndex;                    // nearest node zero based index
    size_t      m_elementLocalNodeIndex;
    size_t      m_elementLocalSurfaceIndex;
    size_t      m_edgeNodeStartIndex;   
    size_t      m_edgeNodeEndIndex;
    Vec3d       m_intersectionPoint;
    Vec3d       m_intersectionPointNormal;
    size_t      m_triangleIndex;                // Index of the triangle hit in the display model
    double      m_distanceAlongRay;
};


}  // namespace ug
}  // namespace cee
