//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"

namespace cee {
namespace ug {

class IsosurfaceData;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT Isosurface : public RefCountedObject
{
public:
    /// Isosurface surface draw styles
    enum DrawStyle
    {
        SURFACE,                    ///< Draw as surface
        SURFACE_MESH,               ///< Draw as surface mesh
        SURFACE_OUTLINE_MESH,       ///< Draw as outline mesh         
        LINE,                       ///< Draw as lines
        OUTLINE                     ///< Draw as outline
    };

public:
    Isosurface();
    virtual ~Isosurface();

    bool        visible() const;
    void        setVisible(bool visible);
    bool        highlight() const;
    void        setHighlight(bool showHighlighted);

    bool        surfaceGeometryVisible() const;
    void        setSurfaceGeometryVisible(bool show);
    DrawStyle   drawStyle() const;
    void        setDrawStyle(DrawStyle drawStyle);
    Color3f     surfaceColor() const;
    void        setSurfaceColor(const Color3f& color);
    float       opacity() const;
    void        setOpacity(float opacity);
    float       specularIntensity() const;
    void        setSpecularIntensity(float intensity);

    double      isoValue() const;
    void        setIsoValue(double value);
    int         isoScalarResultId() const;
    void        setIsoScalarResultId(int resultId);
    bool        computeFromVisibleParts() const;
    void        setComputeFromVisibleParts(bool visiblePartsOnly);

    int         mapScalarResultId() const;
    void        setMapScalarResultId(int resultId);
    int         mapVectorResultId() const;
    void        setMapVectorResultId(int resultId);
    int         mapContourLinesResultId() const;
    void        setMapContourLinesResultId(int resultId);

    bool        isPrecomputed() const;

    double      sampleScalarValue(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;
    Vec3d       sampleVector(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;

    void        data(int stateId, IsosurfaceData* data) const;

private:
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(Isosurface);
    CEE_DISALLOW_COPY_AND_ASSIGN(Isosurface);
};

}  // namespace ug
}  // namespace cee

