//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"

namespace cee {
class Image;

namespace ug {

class IsovolumeData;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT Isovolume : public RefCountedObject
{
public:
    /// %Isovolume surface draw styles
    enum DrawStyle
    {
        SURFACE,                    ///< Draw as surface
        SURFACE_MESH,               ///< Draw as surface mesh
        SURFACE_OUTLINE_MESH,       ///< Draw as outline mesh         
        LINE,                       ///< Draw as lines
        OUTLINE                     ///< Draw as outline
    };

public:
    Isovolume();
    virtual ~Isovolume();

    bool        visible() const;
    void        setVisible(bool visible);
    bool        highlight() const;
    void        setHighlight(bool showHighlighted);

    bool        volumeGeometryVisible() const;
    void        setVolumeGeometryVisible(bool show);
    DrawStyle   drawStyle() const;
    void        setDrawStyle(DrawStyle drawStyle);
    Color3f     volumeColor() const;
    void        setVolumeColor(const Color3f& color);
    float       opacity() const;
    void        setOpacity(float opacity);
    float       specularIntensity() const;
    void        setSpecularIntensity(float intensity);

    double      minimumIsoValue() const;
    void        setMinimumIsoValue(double value);
    double      maximumIsoValue() const;
    void        setMaximumIsoValue(double value);
    int         isoScalarResultId() const;
    void        setIsoScalarResultId(int resultId);
    bool        computeFromVisibleParts() const;
    void        setComputeFromVisibleParts(bool visiblePartsOnly);

    int         mapScalarResultId() const;
    void        setMapScalarResultId(int resultId);
    int         mapVectorResultId() const;
    void        setMapVectorResultId(int resultId);

    Image*      textureImage() const;
    void        setTextureImage(Image* image);
    bool        textureEnvironmentMapping() const;
    void        setTextureEnvironmentMapping(bool useEnvironmentMapping);

    bool        isPrecomputed() const;

    double      sampleScalarValue(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;
	Vec3d       sampleVector(int stateId, const Vec3d& samplePosition, size_t triangleIndex = cee::UNDEFINED_SIZE_T) const;

    void        data(int stateId, IsovolumeData* data) const;

private:
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(Isovolume);
    CEE_DISALLOW_COPY_AND_ASSIGN(Isovolume);
};

}  // namespace ug
}  // namespace cee

