//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"

#include <vector>

namespace cee {
namespace ug {



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ModeShapeAnimation
{
public:
    /// Type of interpolation to use when creating the mode shape animation
    enum InterpolationType
    {
        LINEAR,     ///< Linear interpolation.
        SINE        ///< Sine(alpha) interpolation where the angular speed is constant, making the 
                    ///  interpolation factor follow a sine(x) curve
    };

    /// Type of animation to create, quarter, half or full
    enum AnimationType
    {
        QUARTER,    ///< Animate from original position to the displaced position: V0 ... VD
        HALF,       ///< Animate from original position to the displaced position and back again: V0 ... VD ... V0
        FULL        ///< Animate from original position to the displaced position and back again, 
                    ///  then to the negative displacement and back: V0 ... VD ... V0 ... -VD ... V0
    };
        
public:
    virtual ~ModeShapeAnimation();
    
    size_t              frameCount() const;
    void                setFrameCount(size_t stepCount);

    bool                interpolateScalars() const;
    void                setInterpolateScalars(bool interpolate);

    InterpolationType	interpolationMode() const;
    void                setInterpolationMode(InterpolationType interpolationType);

    AnimationType       animationMode() const;
    void                setAnimationMode(AnimationType animationType);

    void                clear();
    
private:
    ModeShapeAnimation();
    CEE_PRIVATE_F(ModelSpec);
    CEE_PRIVATE_IMPL(ModeShapeAnimation);
    CEE_DISALLOW_COPY_AND_ASSIGN(ModeShapeAnimation);
};

}  // namespace ug
}  // namespace cee
