//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Str.h"

#include "CeeVisualization/Font.h"

namespace cee {

namespace ug {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ModelSettings
{
public:
    /// The draw style to use for the undeformed model
    enum UndeformedDrawStyle
    {
        LINES,      ///< Show undeformed model as lines
        OUTLINE     ///< Show undeformed model as outline
    };
    
    /// Specify if transparent triangles seen from the back side should be rendered or not.
    /// Note: This applies only to simple TransparencyMode::SIMPLE or when using transparent fringes colors.
    enum SimpleTransparencyCullMode
    {
        AUTO,               ///< Try to detect if the part is a closed volume or not. If it is, back face culling is enabled.
        NO_CULLING,         ///< Draw all transparent triangles
        BACK_FACE_CULLING   ///< Draw only the transparent triangles that has a normal pointing towards the eye.
    };
    
public:
    virtual ~ModelSettings();

    Color3f             meshColor() const;
    void                setMeshColor(const Color3f& meshColor);

    Color3f             highlightColor() const;
    void                setHighlightColor(const Color3f& color);
    
    float               ambientIntensity() const;
    void                setAmbientIntensity(float intensity);
    
    bool                showUndeformedModel() const;
    void                setShowUndeformedModel(bool show);
    
    UndeformedDrawStyle undeformedModelDrawStyle() const;
    void                setUndeformedModelDrawStyle(UndeformedDrawStyle drawStyle);
    
    Color3f             undeformedModelColor() const;
    void                setUndeformedModelColor(Color3f color);
    
    bool                undeformedModelUsePartColor() const;
    void                setUndeformedModelUsePartColor(bool usePartColor);

    bool                useShaderPrograms() const;
    bool                setUseShaderPrograms(bool useShaderPrograms);

    bool                useVertexBufferObjects() const;
    void                setUseVertexBufferObjects(bool useVBOs);

    bool                useShaderComputedFlatNormals() const;
    void                setUseShaderComputedFlatNormals(bool useShaderComputedNormals);
    
    bool                useShaderBasedModeShapeAnimation() const;
    void                setUseShaderBasedModeShapeAnimation(bool useShaderBasedAnimation);

    vis::Font*          colorLegendTitleFont();
    void                setColorLegendTitleFont(vis::Font* font);

    vis::Font*          colorLegendDetailsFont();
    void                setColorLegendDetailsFont(vis::Font* font);

    Str                 colorLegendNoResultOverrideTickMarksText() const;
    void                setColorLegendNoResultOverrideTickMarksText(const Str& overrideText);

    bool                optimizePartRendering() const;
    void                setOptimizePartRendering(bool optimize);

    bool                removeInternalSurfaces() const;
    void                setRemoveInternalSurfaces(bool removeInternalSurfaces);

    double              outlineMeshCreaseAngleDegrees() const;
    void                setOutlineMeshCreaseAngleDegrees(double creaseAngleDegrees);

    double              maximumOutlineMeshCreaseAngleDegrees() const;
    void                setMaximumOutlineMeshCreaseAngleDegrees(double creaseAngleDegrees);
    
    double              smoothSurfaceNormalsCreaseAngleDegrees() const;
    void                setSmoothSurfaceNormalsCreaseAngleDegrees(double creaseAngleDegrees);

    bool                mustBeBehindAllCuttingPlanesToClip() const;
    void                setMustBeBehindAllCuttingPlanesToClip(bool mustBeBehindAllToClip);

    bool                isViewClippingIgnored() const;
    void                setIgnoreViewClipping(bool ignore);

    bool                useFirstOrderElementNodesOnly() const;
    void                setUseFirstOrderElementNodesOnly(bool useFirstOrderOnly);
    
    SimpleTransparencyCullMode  simpleTransparencyCullMode() const;
    void                        setSimpleTransparencyCullMode(SimpleTransparencyCullMode mode);

private:
    ModelSettings();
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(ModelSettings);
    CEE_DISALLOW_COPY_AND_ASSIGN(ModelSettings);
};

}  // namespace ug
}  // namespace cee
