//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/ModeShapeAnimation.h"

#include "CeeCore/RefCountedObject.h"

#include <vector>
#include <map>

namespace cee {
namespace ug {



class UnstructGridModelPrivate;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ModelSpec
{
public:
    /// Model color source. This will determine how to color each face of the model if no fringes results are used.
    enum ModelColorSource
    {
        PART,   ///< Use part colors (default)
        SET,    ///< Use set colors
        ELEMENT_TYPE ///< Color by element types
    };
    
public:
    virtual ~ModelSpec();

    std::vector<int>            stateIds() const;
    void                        setStateIds(const std::vector<int>& stateIds); 
    void                        setStateId(int stateId); 

    int                         fringesResultId() const;
    void                        setFringesResultId(int fringesId);
    int                         effectiveFringesResultId(size_t globalGeometryIndex, int partId) const;
    std::vector<int>            allFringesResultIds() const;

    int                         overridePartFringesResultId(size_t globalGeometryIndex, int partId) const;
    void                        setOverridePartFringesResultId(size_t globalGeometryIndex, int partId, int fringesId);
    void                        removeAllOverridePartFringesResultIds();

    int                         contourLinesResultId() const;
    void                        setContourLinesResultId(int contourLinesId);

    std::vector<int>            vectorResultIds() const;
    void                        setVectorResultIds(const std::vector<int>& vectorIds); 
    void                        setVectorResultId(int vectorId); 

    int                         displacementResultId() const;
    void                        setDisplacementResultId(int displacementId);

    bool                        transformationResult() const;
    void                        setTransformationResult(bool useTransformationResult);

    std::vector<int>            visibleSetIds() const;
    void                        setVisibleSetIds(const std::vector<int>& setIds);
    void                        setVisibleSetId(int setId);
    bool                        showComplementOfVisibleSets() const;
    void                        setShowComplementOfVisibleSets(bool showComplement);

    bool                        visibilityResult() const;
    void                        setVisibilityResult(bool useVisibilityResult);
    
    ModelColorSource            modelColorSource() const;
    void                        setModelColorSource(ModelColorSource colorSource);

    ModeShapeAnimation&         modeShapeAnimation();
    const ModeShapeAnimation&   modeShapeAnimation() const;
    
    void                        clear();
    
    void                        setDefaultResults();
    void                        removeInvalidSpecifications();
    
public:
    std::map<std::pair<size_t, int>, int> internal_partFringesResultIdMap() const;

private:
    ModelSpec();

    CEE_BASE_F(UnstructGridModel);
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(ModelSpec);
    CEE_DISALLOW_COPY_AND_ASSIGN(ModelSpec);
};

}  // namespace ug
}  // namespace cee
