//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Color3f.h"

namespace cee {
class Image;

namespace ug {
class PartSettingsTexture;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT PartSettings : public RefCountedObject
{
public:
    /// Part draw styles
    enum DrawStyle
    {
        SURFACE,                ///< Draw as surface
        SURFACE_MESH,           ///< Draw as surface mesh
        SURFACE_OUTLINE_MESH,   ///< Draw as outline mesh    
        LINES,                  ///< Draw as lines (wireframe)
        POINTS,                 ///< Draw as points
        OUTLINE,                ///< Draw as outline
        HIDDEN_LINES_REMOVED,   ///< Draw hidden lines but remove lines that are obscured by the part itself or other parts
        NONE                    ///< Don't draw the part itself (ie still draw vectors mapped on the part)
    };
    
    /// Depth test functions
    enum DepthTestFunc
    {
        LESS,                   ///< Passes if the incoming depth value is less than the stored depth value. This is the default.
        LEQUAL                  ///< Passes if the incoming depth value is less than or equal to the stored depth value.
    };

public:
    virtual                     ~PartSettings();
    
    int                         geometryIndex() const;
    int                         partId() const;

    bool                        visible() const;
    void                        setVisible(bool visible);
    bool                        highlight() const;
    void                        setHighlight(bool showHighlighted);

    float                       eyeLiftFactor() const;
    void                        setEyeLiftFactor(float factor);

    bool                        lighting() const;
    void                        setLighting(bool enable);

    DrawStyle                   drawStyle() const;
    void                        setDrawStyle(DrawStyle drawStyle);
    bool                        smoothShading() const;
    void                        setSmoothShading(bool enable); 
    Color3f                     color() const;
    void                        setColor(const Color3f& color);
    Color3f                     backColor() const;
    void                        setFrontAndBackColor(const Color3f& frontColor, const Color3f& backColor);
    float                       opacity() const;
    void                        setOpacity(float opacity);
    float                       specularIntensity() const;
    void                        setSpecularIntensity(float intensity);

    unsigned int                lineWidth() const;
    void                        setLineWidth(unsigned int lineWidth);
    unsigned int                pointSize() const;
    void                        setPointSize(unsigned int pointSize);
    unsigned int                meshLineWidth() const;
    void                        setMeshLineWidth(unsigned int lineWidth);

    bool                        fringesVisible() const;
    void                        setFringesVisible(bool visible);
    bool                        contourLinesVisible() const;
    void                        setContourLinesVisible(bool visible);
    bool                        vectorsVisible() const;
    void                        setVectorsVisible(bool visible);
    bool                        displacementVisible() const;
    void                        setDisplacementVisible(bool visible);

    bool                        elementSetFilteringEnabled() const;
    void                        setElementSetFilteringEnabled(bool enable);

    void                        setIntersectable(bool intersectable);
    bool                        intersectable() const;

    bool                        alwaysUsePolygonOffset() const;
    void                        setAlwaysUsePolygonOffset(bool alwaysUse);

    void                        setPriority(int drawOrderPriority);
    int                         priority() const; 
    void                        setDepthTestFunction(DepthTestFunc depthTestFunction);
    DepthTestFunc               depthTestFunction() const;

    const PartSettingsTexture&  textureSettings() const;
    PartSettingsTexture&        textureSettings();

private:
    PartSettings();
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(PartSettings);
    CEE_DISALLOW_COPY_AND_ASSIGN(PartSettings);
};

}  // namespace ug
}  // namespace cee

