//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"
#include "CeeUnstructGrid/ParticleTraceData.h"

#include <vector>

namespace cee {
namespace ug {
    
//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ParticleTraceGroup : public RefCountedObject
{
public:
    enum VisualizationStyle
    {
        CYLINDERS,
        SPHERES,
        RIBBONS,
        COMETS,
        LINES,
        ARROWS
    };

    enum ScaleMode
    {
        ABSOLUTE_SCALING,
        RELATIVE_SCALING
    };

public:
    ParticleTraceGroup();
    virtual ~ParticleTraceGroup();

    std::vector<Vec3d>          seedPoints() const;
    void                        setSeedPoints(const std::vector<Vec3d>& seedPoints);

    bool                        visible() const;
    void                        setVisible(bool visible);

    int                         vectorResultId() const;
    void                        setVectorResultId(int resultId);

    int                         mapScalarResultId() const;
    void                        setMapScalarResultId(int resultId);
    
    bool                        computeForward() const;
    void                        setComputeForward(bool compute);

    bool                        computeBackward() const;
    void                        setComputeBackward(bool compute);
    
    int                         maximumNumberOfPointsToCompute() const;
    void                        setMaximumNumberOfPointsToCompute(int maxNumPoints);

    double                      extrapolateRatioLength() const;
    void                        setExtrapolateRatioLength(double extrapolateRatioLength);

    VisualizationStyle          visualizationStyle() const;
    void                        setVisualizationStyle(VisualizationStyle style);

    Color3f                     color() const;
    void                        setColor(const Color3f& color);

    ScaleMode                   scaleMode() const;
    double                      scaleFactor() const;
    void                        set3DPrimitiveRadius(ScaleMode mode, double scaleFactor);

    double                      pointSamplingNumberOfPulses() const;
    void                        setPointSamplingNumberOfPulses(double numPulses);

    double                      cometLength() const;
    void                        setCometLength(double length);

    double                      arrowLength() const;
    void                        setArrowLength(double length);

    double                      ribbonRelativeWidth() const;
    void                        setRibbonRelativeWidth(double relativeSize);
    double                      ribbonRelativeThickness() const;
    void                        setRibbonRelativeThickness(double relativeSize);

    double                      minimumTraceTime(size_t frameIndex) const;
    double                      maximumTraceTime(size_t frameIndex) const;
    
    bool                        isPrecomputed() const;
    
    void                        setTraceData(size_t frameIndex, const std::vector<ParticleTraceData>& groupDataArr);
    void                        traceData(size_t frameIndex, std::vector<ParticleTraceData>* groupDataArr) const;

private:
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(ParticleTraceGroup);
    CEE_DISALLOW_COPY_AND_ASSIGN(ParticleTraceGroup);
};

}  // namespace ug
}  // namespace cee
