//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Unstruct
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2019, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "VTOBaseAPI.h"
#include "VTOResultCalculator.h"

#include "cvfBase.h"
#include "cvfLogger.h"
#include "cvfFlags.h"

#include "CRCResultType.h"

#include "crchlCalculatorMetaData.h"
#include "crchlLogger.h"

class CRCResultCalculator;
class VTDatabaseMetaData;
class VTDataFEMModel;
class VTDataScalarResult;

namespace crchl {
class PluginManager;
class CalculatorInstance;
class ResultCalculatorFactory;
}


//==================================================================================================
//
// Implements plugin backed result calculator
//
//==================================================================================================
class PluginResultCalculator : public VTResultCalculator
{
public:
    PluginResultCalculator(crchl::CalculatorInstance* crcCalculatorInstance);
    virtual ~PluginResultCalculator();
    
    virtual bool                    initializeCalculator(const VTDatabaseMetaData& metaData, const VTString& initString, const VTString& resultIdString, VTString* errorMessage) override;
    
    virtual VTString                calculatorIdString() const override;
    
    virtual VTString                outputResultName() const override;
    virtual VTFEMResultType         outputResultType() const override;
    virtual VTString                outputResultIdString() const override;

    virtual VTFEMResultMappingType  resultMapping() const override;
    virtual size_t                  inputResultCount() const override;
    virtual InputResultSpec         inputResultSpec(size_t index) const override;

    virtual void                    getCalculatorParams(VTResultCalculatorParams* calculatorParams) const override;
    virtual bool                    setCalculatorParams(const VTResultCalculatorParams& calculatorParams) override;

    virtual bool                    calculateResultForModel(const VTResultID& resultId, bool computeNodeAverage, VTDataFEMModel* model) const override;

    static void                     populateInputValueArr(const VTDataFEMModel& model, const std::vector<InputResultSpec>& inputResultSpecArr, const std::vector<VTint>& modelResultIndexPerInputRes, VTint resultValueIndex, VTint partIndex, std::vector<double>* inputValues);
    
private:
    static VTFEMResultType          toFEMResultType(CRCResultType crcResultType);
    static CRCResultType            toCRCResultType(VTFEMResultType femResultType);

    cvf::ref<VTDataScalarResult>    createCalculateScalarResult(const VTDataFEMModel& model, const VTResultID& resultId, bool computeNodeAverage) const;
    size_t                          determineRequiredSizeOfInputValueArr() const;
    static std::vector<VTint>       determineModelResultIndicesOfInputResults(const VTDataFEMModel& model, const std::vector<InputResultSpec>& inputResultSpecArr);
    VTint                           determineNumResultValuesForPart(const VTDataFEMModel& model, VTint partIndex) const;

private:
    crchl::CalculatorInstance*      m_crcCalculator;
    crchl::CalculatorMetaData       m_crcCalculatorMetaData;
    bool                            m_initOk;
    VTFEMResultMappingType          m_resultMapping;
    std::vector<InputResultSpec>    m_inputResultSpecArr;
    cvf::ref<cvf::Logger>           m_cvfLogger;

};


//==================================================================================================
//
// 
//
//==================================================================================================
class PluginResultCalculatorFactory : public VTResultCalculatorFactory
{
public:
    PluginResultCalculatorFactory(crchl::PluginManager* crcPluginManager, size_t calculatorIndex);

    virtual cvf::ref<VTResultCalculator> createNewCalculatorInstance() const override;

private:
    crchl::PluginManager*   m_crcPluginManager;
    size_t                  m_calculatorIndex;
};



//==================================================================================================
//
// Result calculator logger adapter
//
//==================================================================================================
class PluginResultCalculatorLoggerAdapter : public crchl::Logger
{
public:
    PluginResultCalculatorLoggerAdapter(cvf::Logger* cvfLogger);

    virtual void log(Logger::Level messageLevel, const std::string& message) override;

private:
    cvf::ref<cvf::Logger> m_cvfLogger;
};
