//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeVisualization/OverlayItem.h"
#include "CeeCore/Str.h"

namespace cee {
class PropertySetCollection;
class ImageResources;

namespace vis {
    class View;
    class OverlayImage;
}

namespace ug {
class UnstructGridModel;


//==================================================================================================
//
// Apply properties in a property set collection to an existing model/camera etc
//
//==================================================================================================
class CEE_UG_EXPORT PropertyApplierVTFx
{
public:
    PropertyApplierVTFx(PropertySetCollection* propSetCollection, ImageResources* resources);
    ~PropertyApplierVTFx();

    void    applyToModel(UnstructGridModel* model);
    void    applyToView(vis::View* view);

    bool    logoOverlayImage(vis::OverlayImage* image, vis::OverlayItem::LayoutCorner* layoutCorner, vis::OverlayItem::LayoutDirection* layoutDirection) const;
    double  defaultFramesPerSecond() const;
    int     firstFrameDelay() const;
    int     lastFrameDelay() const;
    int     numberOfParticleTraceAnimationSteps() const;
    bool    hasStartAnimation() const;
    int     animatedCuttingPlaneIndex() const;
    int     animatedCuttingPlaneFrameCount() const;
    double  animatedCuttingPlaneDistancePerFrame() const;

    Str     viewTitle() const;

private:
    CEE_PRIVATE_IMPL(PropertyApplierVTFx);
    CEE_DISALLOW_COPY_AND_ASSIGN(PropertyApplierVTFx);
};



}  // namespace ug
}  // namespace cee
