//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/Variant.h"
#include "CeeCore/Str.h"
#include "CeeVisualization/OverlayItem.h"
#include "CeeCore/BoundingBox.h"

#include <vector>

namespace cee {

class PropertySet;
class ImageResources;
class Plane;

namespace vis {
class View;
class Camera;
class Background;
class OverlayImage;
}

namespace ug {

class UnstructGridModel;
class ModelSpec;
class ScalarSettings;
class VectorSettings;
class PartSettings;
class CuttingPlane;
class Isosurface;
class Isovolume;
class ParticleTraceGroup;
class MirrorSettings;



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ResultSelectionPropertyHandler
{
public:
    enum PropertyKey
    {
        FRINGES_RESULT_ID,
        CONTOUR_LINES_RESULT_ID,
        VECTOR_RESULT_IDS,
        DISPLACEMENT_RESULT_ID,
        TRANSFORMATION_RESULT,
        VISIBILITY_RESULT,
        PER_PART_FRINGES_RESULT_IDS
    };

public:
    ResultSelectionPropertyHandler(PropertySet* propSet);
    ~ResultSelectionPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToModelSpec(ModelSpec* modelSpec) const;
    void                    buildProperties(const ModelSpec& modelSpec);

private:
    CEE_PRIVATE_IMPL(ResultSelectionPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ResultSelectionPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT SetPropertyHandler
{
public:
	enum PropertyKey
	{
		SET_MODE,
		SET_IDS
	};


public:
	SetPropertyHandler(PropertySet* propSet);
	~SetPropertyHandler();

	static Str              propertySetClassType();
	static bool             isValidPropertySet(const PropertySet& propSet);
	Variant                 propertyValue(PropertyKey key) const;
	void                    setPropertyValue(PropertyKey key, Variant data);

	void					    applyToModelSpec(ModelSpec* modelSpec);
	void                    buildProperties(const ModelSpec& modelSpec);

private:
	CEE_PRIVATE_IMPL(SetPropertyHandler);
	CEE_DISALLOW_COPY_AND_ASSIGN(SetPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT StateSelectionPropertyHandler
{
public:
    enum PropertyKey
    {
        STATE_IDS,
        MODE_SHAPE_NUM_FRAMES,
        MODE_SHAPE_ANIMATION_TYPE,
        MODE_SHAPE_INTERPOLATION_TYPE,
        MODE_SHAPE_INTERPOLATE_SCALARS
    };

public:
    StateSelectionPropertyHandler(PropertySet* propSet);
    ~StateSelectionPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToModelSpec(ModelSpec* modelSpec) const;
    void                    buildProperties(const ModelSpec& modelSpec);

private:
    CEE_PRIVATE_IMPL(StateSelectionPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(StateSelectionPropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ColorMapperFilledContoursUniformPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        RANGE_MIN,
        RANGE_MAX,
        COLOR_SCHEME,
        LOGARITHMIC_SCALE,
        NUM_COLORS,
        LEVEL_COLORS,
        ABOVE_RANGE_COLOR,
        BELOW_RANGE_COLOR,
        UNDEFINED_COLOR,
        UNDEFINED_OPACITY
    };

public:
    ColorMapperFilledContoursUniformPropertyHandler(PropertySet* propSet);
    ~ColorMapperFilledContoursUniformPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToScalarSettings(ScalarSettings* scalarSettings) const;
    void                    buildProperties(const ScalarSettings& scalarSettings);

private:
    CEE_PRIVATE_IMPL(ColorMapperFilledContoursUniformPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ColorMapperFilledContoursUniformPropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ColorMapperFilledContoursPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        COLOR_SCHEME,
        LOGARITHMIC_SCALE,
        LEVEL_VALUES,
        LEVEL_COLORS,
        ABOVE_RANGE_COLOR,
        BELOW_RANGE_COLOR,
        UNDEFINED_COLOR,
        UNDEFINED_OPACITY
    };

public:
    ColorMapperFilledContoursPropertyHandler(PropertySet* propSet);
    ~ColorMapperFilledContoursPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToScalarSettings(ScalarSettings* scalarSettings) const;
    void                    buildProperties(const ScalarSettings& scalarSettings);

private:
    CEE_PRIVATE_IMPL(ColorMapperFilledContoursPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ColorMapperFilledContoursPropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ColorMapperContinuousPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        RANGE_MIN,
        RANGE_MAX,
        COLOR_SCHEME,
        LOGARITHMIC_SCALE,
        COLORS_AT_VALUES,
        ABOVE_RANGE_COLOR,
        BELOW_RANGE_COLOR,
        UNDEFINED_COLOR,
        UNDEFINED_OPACITY
    };

public:
    ColorMapperContinuousPropertyHandler(PropertySet* propSet);
    ~ColorMapperContinuousPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToScalarSettings(ScalarSettings* scalarSettings) const;
    void                    buildProperties(const ScalarSettings& scalarSettings);

private:
    CEE_PRIVATE_IMPL(ColorMapperContinuousPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ColorMapperContinuousPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT OverlayColorLegendPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        TEXT_COLOR,
        LINE_COLOR,
        VISIBILITY_MODE,
        COLOR_MARKER_POSITIONS,
        COLOR_MARKER_COLORS,
        COLOR_MARKER_DESCRIPTIONS,
        CUSTOM_LEGEND_TITLE,
        NUMERIC_FORMAT,
        NUMERIC_PRECISION,
        USER_SCALING_FIRST_DEGREE,
        USER_SCALING_CONSTANT,
        FORCE_UNIFORM_LEVEL_SPACING,
        ABOVE_BELOW_RANGE_MARKER_MODE,
        COLOR_MARKER_BOX_HEIGHT,
        COLOR_MARKER_VERTICAL_MARGIN,
        FORCE_TICKMARKS_OVER_LEGEND,
        TICKMARKS_POSITION,
        DIRECTION,

        // Deprecated:
        DEPRECATED_VISIBLE
    };

public:
    OverlayColorLegendPropertyHandler(PropertySet* propSet);
    ~OverlayColorLegendPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToScalarSettings(ScalarSettings* scalarSettings) const;
    void                    buildProperties(const ScalarSettings& scalarSettings);

private:
    CEE_PRIVATE_IMPL(OverlayColorLegendPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayColorLegendPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ScalarSettingsPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        FILTER_MIN,
        FILTER_MAX,
        NODE_AVERAGE,
        AUTO_RANGE_MODE,
        CONTOUR_LINES_LEVEL_COUNT,
        CONTOUR_LINES_WIDTH,
        CONTOUR_LINES_COLOR_MODE,
        CONTOUR_LINES_SINGLE_COLOR
    };

public:
    ScalarSettingsPropertyHandler(PropertySet* propSet);
    ~ScalarSettingsPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToScalarSettings(ScalarSettings* scalarSettings) const;
    void                    buildProperties(const ScalarSettings& scalarSettings);

private:
    CEE_PRIVATE_IMPL(ScalarSettingsPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ScalarSettingsPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT VectorSettingsPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_RESULT_ID,
        SCALE_MODE,
        SCALE_FACTOR,
        COLOR_MODE,
        SINGLE_COLOR,
        CLAMP_MIN_LENGTH,
        CLAMP_MAX_LENGTH,
        FILTER_MIN_LENGTH,
        FILTER_MAX_LENGTH,
        ARROW_SHAFT_RELATIVE_RADIUS,
        ARROW_HEAD_RELATIVE_RADIUS,
        ARROW_HEAD_RELATIVE_LENGTH,
        LOCAL_COORDINATES,
        SHOW_ON_REFERENCED_NODES_ONLY,
        SIMPLE_VECTOR_LINE_WIDTH,
        DRAW_MODE,
        DRAW_SKIPBY,
        DRAW_MAX_COUNT
    };

public:
    VectorSettingsPropertyHandler(PropertySet* propSet);
    ~VectorSettingsPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(int* resultId) const;
    void                    encodeContext(int resultId);
    void                    applyToVectorSettings(VectorSettings* vectorSettings) const;
    void                    buildProperties(const VectorSettings& vectorSettings);

private:
    CEE_PRIVATE_IMPL(VectorSettingsPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(VectorSettingsPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT PartSettingsPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_GEOMETRY_INDEX,
        CONTEXT_PART_ID,
        VISIBLE,
        DRAW_STYLE,
        COLOR,
        OPACITY,
        SPECULAR_INTENSITY,
        LIGHTING,
        SMOOTH_SHADING,
        LINE_WIDTH,
        POINT_SIZE,
        FRINGES_VISIBLE,
        CONTOUR_LINES_VISIBLE,
        VECTORS_VISIBLE,
        DISPLACEMENT_VISIBLE,
        ELEMENTSET_FILTERING_ENABLED,
        EYE_LIFT_FACTOR,
        TEXTURE_IMAGE_NAME,
        TEXTURE_ENVIRONMENT_MAPPING,
        TEXTURE_REPEAT_FACTOR,
        TEXTURE_AUTO_IMAGE_SCALE,
        TEXTURE_MODULATE,
        TEXTURE_FILTER_NEAREST
    };

public:
    PartSettingsPropertyHandler(PropertySet* propSet, ImageResources* imageResources);
    ~PartSettingsPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    bool                    decodeContext(size_t* geoIdx, int* partId) const;
    void                    encodeContext(size_t geoIdx, int partId);
    void                    applyToPartSettings(PartSettings* partSettings) const;
    void                    buildProperties(const PartSettings& partSettings);

private:
    CEE_PRIVATE_IMPL(PartSettingsPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(PartSettingsPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT CuttingPlanePropertyHandler
{
public:
    enum PropertyKey
    {
        VISIBLE,
        POINT,
        NORMAL,
        MAP_SCALAR_RESULT_ID,
        MAP_VECTOR_RESULT_ID,
        VECTOR_GRID_SPACING_MODE,
        VECTOR_GRID_SPACING,
        SURFACE_VISIBLE,
        SURFACE_COLOR,
        CLIP_MODEL,
        COMPUTE_FROM_VISIBLE_PARTS,
        DRAW_STYLE,
        OPACITY,
        SPECULAR_INTENSITY,
        USE_SOURCE_PART_COLOR,
        GRID_LINES_VISIBLE,
        PROJECT_VECTORS_ON_PLANE,
        CRINKLE_CUT_SURFACE
    };

public:
    CuttingPlanePropertyHandler(PropertySet* propSet);
    ~CuttingPlanePropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToCuttingPlane(ug::CuttingPlane* cuttingPlane) const;
    void                    buildProperties(const ug::CuttingPlane& cuttingPlane);

private:
    CEE_PRIVATE_IMPL(CuttingPlanePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(CuttingPlanePropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT IsosurfacePropertyHandler
{
public:
    enum PropertyKey
    {
        VISIBLE,
        ISO_VALUE,
        ISO_SCALAR_RESULT_ID,
        MAP_SCALAR_RESULT_ID,
        MAP_VECTOR_RESULT_ID,
        SURFACE_VISIBLE,
        SURFACE_COLOR,
        COMPUTE_FROM_VISIBLE_PARTS,
        DRAW_STYLE,
        OPACITY,
        SPECULAR_INTENSITY
    };

public:
    IsosurfacePropertyHandler(PropertySet* propSet);
    ~IsosurfacePropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToIsosurface(ug::Isosurface* isosurface) const;
    void                    buildProperties(const ug::Isosurface& isosurface);

private:
    CEE_PRIVATE_IMPL(IsosurfacePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(IsosurfacePropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT IsovolumePropertyHandler
{
public:
    enum PropertyKey
    {
        VISIBLE,
        MINIMUM_ISO_VALUE,
        MAXIMUM_ISO_VALUE,
        ISO_SCALAR_RESULT_ID,
        MAP_SCALAR_RESULT_ID,
        MAP_VECTOR_RESULT_ID,
        VOLUME_VISIBLE,
        VOLUME_COLOR,
        COMPUTE_FROM_VISIBLE_PARTS,
        DRAW_STYLE,
        OPACITY,
        SPECULAR_INTENSITY,
        TEXTURE_IMAGE_NAME,
        TEXTURE_ENVIRONMENT_MAPPING
    };

public:
    IsovolumePropertyHandler(PropertySet* propSet, ImageResources* imageResources);
    ~IsovolumePropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToIsovolume(ug::Isovolume* isovolume) const;
    void                    buildProperties(const ug::Isovolume& isovolume);

private:
    CEE_PRIVATE_IMPL(IsovolumePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(IsovolumePropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ParticleTraceGroupPropertyHandler
{
public:
    enum PropertyKey
    {
        VISIBLE,
        SEED_POINTS,
        VECTOR_RESULT_ID,
        MAP_SCALAR_RESULT_ID,
        VISUALIZATION_STYLE,
        COLOR,
        RADIUS_SCALE_MODE,
        RADIUS_SCALE_FACTOR,
        POINT_SAMPLING_NUM_PULSES,
        COMET_LENGTH,
        RIBBON_RELATIVE_WIDTH,
        RIBBON_RELATIVE_THICKNESS,
        ARROW_LENGTH
    };

public:
    ParticleTraceGroupPropertyHandler(PropertySet* propSet);
    ~ParticleTraceGroupPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToParticleTraceGroup(ug::ParticleTraceGroup* particleTraceGroup) const;
    void                    buildProperties(const ug::ParticleTraceGroup& particleTraceGroup);

private:
    CEE_PRIVATE_IMPL(ParticleTraceGroupPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ParticleTraceGroupPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT MirrorSettingsPropertyHandler
{
public:
    enum PropertyKey
    {
        NUM_MIRROR_PASSES,
        PLANE_FIRST_A,
        PLANE_FIRST_B,
        PLANE_FIRST_C,
        PLANE_FIRST_D,
        PLANE_SECOND_A,
        PLANE_SECOND_B,
        PLANE_SECOND_C,
        PLANE_SECOND_D,
        PLANE_MASTER_A,
        PLANE_MASTER_B,
        PLANE_MASTER_C,
        PLANE_MASTER_D,
        REFLECTIVE
    };

public:
    MirrorSettingsPropertyHandler(PropertySet* propSet);
    ~MirrorSettingsPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToMirrorSettings(ug::MirrorSettings* mirrorSettings) const;
    void                    buildProperties(const ug::MirrorSettings& mirrorSettings);

private:
    CEE_PRIVATE_IMPL(MirrorSettingsPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(MirrorSettingsPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ViewClippingPlanePropertyHandler
{
public:
    enum PropertyKey
    {
        POINT,
        NORMAL
    };

public:
    ViewClippingPlanePropertyHandler(PropertySet* propSet);
    ~ViewClippingPlanePropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToClippingPlane(Plane* plane) const;
    void                    buildProperties(const Plane& plane);

private:
    CEE_PRIVATE_IMPL(ViewClippingPlanePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ViewClippingPlanePropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT CameraPropertyHandler
{
public:
    enum PropertyKey
    {
        EYE,
        VRP,
        VUP,
        PROJECTION_TYPE,
        FIELD_OF_VIEW_Y_DEGREES,
        ORTHO_HEIGHT
    };

public:
    CameraPropertyHandler(PropertySet* propSet);
    ~CameraPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToCamera(vis::Camera* camera, const BoundingBox* viewBoundingBox) const;
    void                    buildProperties(const vis::Camera& camera);

private:
    CEE_PRIVATE_IMPL(CameraPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(CameraPropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT BackgroundPropertyHandler
{
public:
    enum PropertyKey
    {
        SINGLE_BACKGROUND_COLOR,
        TOP_BACKGROUND_COLOR,
        BOTTOM_BACKGROUND_COLOR
    };

public:
    BackgroundPropertyHandler(PropertySet* propSet);
    ~BackgroundPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    void                    applyToBackground(vis::Background* background) const;
    void                    buildProperties(const vis::Background& background);

private:
    CEE_PRIVATE_IMPL(BackgroundPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(BackgroundPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT OverlayImagePropertyHandler
{
public:
    enum PropertyKey
    {
        IMAGE_NAME,
        IMAGE_SCALE_WIDTH,
        IMAGE_SCALE_HEIGHT,
        LAYOUT_CORNER,
        LAYOUT_DIRECTION,
        BLENDING_MODE,
        GLOBAL_ALPHA
    };

public:
    OverlayImagePropertyHandler(PropertySet* propSet, ImageResources* imageResources);
    ~OverlayImagePropertyHandler();

    static Str  propertySetClassType();
    static bool isValidPropertySet(const PropertySet& propSet);
    Variant     propertyValue(PropertyKey key) const;
    void        setPropertyValue(PropertyKey key, Variant data);

    void        applyToOverlayImage(vis::OverlayImage* overlayImage, vis::OverlayItem::LayoutCorner* layoutCorner, vis::OverlayItem::LayoutDirection* layoutDirection) const;
    void        buildProperties(const vis::OverlayImage& overlayImage, vis::OverlayItem::LayoutCorner layoutCorner, vis::OverlayItem::LayoutDirection layoutDirection);

private:
    CEE_PRIVATE_IMPL(OverlayImagePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayImagePropertyHandler);
};



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ViewerPropertyHandler
{
public:
    enum PropertyKey
    {
        ANIMATION_FRAMES_PER_SECOND,
        ANIMATION_FIRST_FRAME_DELAY,
        ANIMATION_LAST_FRAME_DELAY,
        NUM_PARTICLE_TRACE_STEPS,
        START_ANIMATION,
        HALO_COLOR,
        CUTTING_PLANE_INDEX_TO_ANIMATE,
        CUTTING_PLANE_NUM_ANIM_FRAMES,
        CUTTING_PLANE_ANIM_FRAME_DISTANCE,
        VIEW_TITLE
    };

public:
    ViewerPropertyHandler(PropertySet* propSet);
    ~ViewerPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

    double                  defaultFramesPerSecond() const;
    void                    setDefaultFramesPerSecond(double animationFPS);

    int                     firstFrameDelay() const;
    void                    setFirstFrameDelay(int milliSeconds);

    int                     lastFrameDelay() const;
    void                    setLastFrameDelay(int milliSeconds);

    int                     numberOfParticleTraceAnimationSteps() const;
    void                    setNumberOfParticleTraceAnimationSteps(int numAnimationSteps);

    bool                    hasStartAnimation() const;
    void                    setStartAnimation(bool start);

    Color3f                 haloColor() const;
    void                    setHaloColor(const Color3f& color);

    int                     animatedCuttingPlaneIndex() const;
    void                    setAnimatedCuttingPlaneIndex(int index);

    int                     animatedCuttingPlaneNumberOfFrames() const;
    void                    setAnimatedCuttingPlaneNumberOfFrames(int numFrames);

    double                  animatedCuttingPlaneDistancePerFrame() const;
    void                    setAnimatedCuttingPlaneDistancePerFrame(double distance);

    Str                     viewTitle() const;
    void                    setViewTitle(const Str& title);

private:
    CEE_PRIVATE_IMPL(ViewerPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(ViewerPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT OverlayPlotPropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_OVERLAY_PLOT_ID,
        HEIGHT,
        WIDTH,
        TITLE,
        DRAW_BACKGROUND,
        BACKGROUND_COLOR,
        BACKGROUND_OPACITY,
        BORDER_COLOR,
        BORDER_WIDTH,
        DRAW_BORDER,
        X_AXIS_TITLE,
        Y_AXIS_TITLE,
        X_AXIS_RANGE_MIN,
        X_AXIS_RANGE_MAX,
        Y_AXIS_RANGE_MIN,
        Y_AXIS_RANGE_MAX,
        DRAW_LEGEND,
        TEXT_COLOR,
        DRAW_GRID_LINES,
        DRAW_MARKER_LINE,
        MARKER_LINE_VALUE,
        MARKER_LINE_COLOR,
        AXIS_COLOR,
        AXIS_GRID_LINES_COLOR
    };

public:
    OverlayPlotPropertyHandler(PropertySet* propSet);
    ~OverlayPlotPropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

private:
    CEE_PRIVATE_IMPL(OverlayPlotPropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayPlotPropertyHandler);
};


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT OverlayPlotCurvePropertyHandler
{
public:
    enum PropertyKey
    {
        CONTEXT_OVERLAY_PLOT_ID,
        VTFX_PLOT_DATA_INDEX,
        VTFX_PLOT_VARIABLE_INDEX_X,
        VTFX_PLOT_VARIABLE_INDEX_Y,
        TITLE,
        COLOR
    };

public:
    OverlayPlotCurvePropertyHandler(PropertySet* propSet);
    ~OverlayPlotCurvePropertyHandler();

    static Str              propertySetClassType();
    static bool             isValidPropertySet(const PropertySet& propSet);
    Variant                 propertyValue(PropertyKey key) const;
    void                    setPropertyValue(PropertyKey key, Variant data);

private:
    CEE_PRIVATE_IMPL(OverlayPlotCurvePropertyHandler);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayPlotCurvePropertyHandler);
};

}  // namespace ug
}  // namespace cee
