//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/Str.h"
#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/CustomResultGenerator.h"


#include <map>

namespace cee {
namespace ug {

class UnstructGridModel;
class DataState;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ResultInfo
{

public:
    ResultInfo();
    ResultInfo(ResultType type, int resultId, const Str& resultName, ResultMapping resMapping);
    ResultInfo(ResultType type, int resultId, const Str& resultName, ResultMapping resMapping, ResultType baseResultType, int baseResultId, DeriveOperation deriveOperation);
    ResultInfo(ResultType type, int resultId, const Str& resultName, ResultMapping resMapping, std::vector<CustomResultArgument>& customResultArguments, const CustomResultGenerator* customResultGenerator);
    ResultInfo(ResultType type, const Str& resultName, ResultMapping resMapping, std::vector<CustomResultArgument>& customResultArguments, const CustomResultGenerator* customResultGenerator);

    ResultInfo(const ResultInfo& other);
    ~ResultInfo();

    ResultInfo&                         operator=(const ResultInfo& rhs);
    bool                                operator==(const ResultInfo& rhs) const;
    bool                                operator!=(const ResultInfo& rhs) const;

    ResultType                          type() const;
    int                                 id() const;
    
    Str                                 name() const;
    void                                setName(const Str& name);
    
    Str                                 idString() const;
    void                                setIdString(const Str& idString);

    ResultMapping                       resultMapping() const;

    ResultType                          baseResultType() const;
    int				                    baseResultId() const;
    DeriveOperation                     deriveOperation() const;

    StrainType                          strainType() const;
    void                                setStrainType(StrainType type);

    bool                                isValid() const;
    bool                                isCustom() const;
    
    Str                                 resultCalculatorId() const;

    Str                                 nature() const;
    void                                setNature(const Str& nature);

    const std::vector<int>&             sectionIds() const;
    void                                setSectionIds(const std::vector<int>& ids);

    Str                                 attribute(const Str& name) const;
    std::vector<Str>                    attributeNames() const;

    static Str                          deriveOperationName(DeriveOperation deriveOperation);

    void                                setCustomResultGenerator(const CustomResultGenerator* generator, const std::vector<CustomResultArgument>& arguments);
    std::vector<CustomResultArgument>   customResultArguments() const;
    const CustomResultGenerator*        customResultGenerator() const;

    void                                setId(int id);
    static int                          nextId(ResultType resultType);

private:
    void                                setAttribute(const Str& name, const Str& value);

    static int                          updateOrGetNextId(ResultType resultId, int id, bool update);

private:

    CEE_BASE_F(DataSourceDirectory);
    CEE_PRIVATE_IMPL(ResultInfo);
};




}  // namespace ug
}  // namespace cee
