//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2017, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/PtrRef.h"

#include "CeeUnstructGrid/Base.h"
#include "CeeUnstructGrid/ResultsQueryResult.h"
#include "CeeUnstructGrid/ResultsQueryItem.h"
#include "CeeUnstructGrid/ResultsQueryResultPosition.h"

#include <vector>
#include <map>

namespace cee {
namespace ug {

class DataSource;
class ResultsQueryResultPosition;


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT ResultsQuery
{
public:
    ResultsQuery(DataSource* dataSource);
    ~ResultsQuery();

	bool scalars(int scalarId, int stateId, size_t geometryIndex, const std::vector<ResultsQueryItem>* items, std::vector<ResultsQueryResult>* results);
	bool vectors(int vectorId, int stateId, size_t geometryIndex, const std::vector<ResultsQueryItem>* items, std::vector<ResultsQueryResult>* results);
	bool tensors(int tensorId, int stateId, size_t geometryIndex, const std::vector<ResultsQueryItem>* items, std::vector<ResultsQueryResult>* results);


    bool minimumAndMaximumScalar(int scalarId, int stateId, size_t geometryIndex, ResultsQueryResultPosition* minScalarPositionAndResult, ResultsQueryResultPosition* maxScalarPositionAndResult);
    bool minimumScalars(size_t scalarsCount, int scalarId, int stateId, size_t geometryIndex, std::vector<ResultsQueryResultPosition>* minScalarPositionAndResults);
    bool maximumScalars(size_t scalarsCount, int scalarId, int stateId, size_t geometryIndex, std::vector<ResultsQueryResultPosition>* maxScalarPositionAndResults);

    bool minimumAndMaximumVector(int vectorId, int stateId, size_t geometryIndex, ResultsQueryResultPosition* minVectorPositionAndResult, ResultsQueryResultPosition* maxVectorPositionAndResult);
    bool minimumVectors(size_t vectorsCount, int vectorId, int stateId, size_t geometryIndex, std::vector<ResultsQueryResultPosition>* minVectorPositionAndResults);
    bool maximumVectors(size_t vectorsCount, int vectorId, int stateId, size_t geometryIndex, std::vector<ResultsQueryResultPosition>* maxVectorPositionAndResults);
  

private:
    CEE_PRIVATE_IMPL(ResultsQuery);
    CEE_DISALLOW_COPY_AND_ASSIGN(ResultsQuery);

};

}  // namespace ug
}  // namespace cee
