//##################################################################################################
//
//  Ceetron Analyzer
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2018, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################


#pragma once

#include "CeeCore/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Str.h"
#include "CeeCore/Variant.h"

#include "CeeUnstructGrid/SetInfo.h"

#include <map>


namespace cee {
    class Image;
    class PropertySetCollection;

namespace vis { class View; }

namespace ug {
    class DataElementSet;
    class UnstructGridModel;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT Situation : public RefCountedObject
{
public:
    Situation();
    virtual ~Situation();

    const PropertySetCollection*        propertyCollection() const;
    void                                setPropertyCollection(PropertySetCollection* collection);

    void                                setName(const Str& name);
    Str                                 name() const;

    void                                setDescription(const Str& description);
    Str                                 description() const;

    void                                setSourceModelFile(const Str& modelfile);
    Str                                 sourceModelFile() const;

    void                                setSnapshot(Image* image);
    const Image*                        snapshot() const;

    Str                                 date() const;

    std::vector<const DataElementSet*>  elementSets() const;
    std::vector<SetInfo>                elementSetInfos() const;
    void                                addElementSetInfo(SetInfo setInfo);
    void                                addElementSet(const DataElementSet* set);

    size_t                              userDataCount() const;
    Str                                 userData(const Str& name) const;
    void                                setUserData(const Str& name, const Str& value);
    void                                setAllUserData(std::map<Str, Str> ud);
    const std::map<Str, Str>*           allUserData() const;

    void                                capture(const UnstructGridModel& model, const vis::View& view);

    void                                updateModel(UnstructGridModel* model) const;
    void                                updateView(vis::View* view) const;

private:
    void                                setDate(const Str& date);

    CEE_PRIVATE_IMPL(Situation);
    CEE_BASE_F(SituationIo);
    CEE_BASE_F(SituationIoPrivate);
    CEE_DISALLOW_COPY_AND_ASSIGN(Situation);
};

}
}
