//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2013, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Vec3d.h"
#include "CeeUnstructGrid/UnstructGridModel.h"
#include "CeeUnstructGrid/HitItem.h"

#include <vector>

namespace cee {
namespace ug {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT SurfacePathQuery
{
public:
    /// Enum specifying what kind of items that are in use
    enum PointType
    {
        NODE,               ///< The items specify nodes
        ELEMENT,            ///< The items specify elements
        COORDINATE          ///< The items specify coordinates
    };


public:
    SurfacePathQuery(UnstructGridModel* model, size_t frameIndex, PointType pointType, std::vector<cee::ug::HitItem> items);
    ~SurfacePathQuery();

    bool        generatePath(std::vector<Vec3d>* points);
    bool        getScalarResultValues(int scalarResultId, std::vector<double>* length, std::vector<double>* results);
    void        enableSampling(bool enable);

private:
    CEE_PRIVATE_IMPL(SurfacePathQuery);
    CEE_DISALLOW_COPY_AND_ASSIGN(SurfacePathQuery);
};

}  // namespace ug
}  // namespace cee
