//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/PtrRef.h"
#include "CeeCore/Str.h"
#include "CeeCore/PropertySet.h"

class VTDatabase;

namespace cee {

class Image;

namespace ug {

class VTFxMemoryFile;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT VTFxFileBrowser 
{
public:
    VTFxFileBrowser();
    ~VTFxFileBrowser();
    
    static bool     isVTFxFile(const Str& fileName);
    static bool     isEncrypted(const Str& fileName);

    bool            initialize(const Str& fileName);
    bool            initialize(const VTFxMemoryFile* vtfxMemFile);

    void            setPassword(const Str& password);
    
    size_t          caseCount() const;
    Str             caseName(size_t caseIndex) const;
    Str             caseDescriptionHtml(size_t caseIndex) const;
    Image*          caseSnapshot(size_t caseIndex) const;
    int             caseId(size_t caseIndex) const;
    size_t          caseIndex(int caseId) const;

    Str             readTextFile(const Str& filename) const;

    size_t          userTextFileCount() const;
    Str             userTextFileName(size_t userTextFileIndex) const;
    Str             userTextFileContent(size_t userTextFileIndex) const;

    int             vendorId() const;
    bool            hasValidExpressSignature(const Str& vendorCode) const;
    bool            isPasswordValid() const;
    bool            isLegacyFile() const;

    bool            fileInfo(PropertySet* properties);

private:
    CEE_PRIVATE_IMPL(VTFxFileBrowser);
    CEE_DISALLOW_COPY_AND_ASSIGN(VTFxFileBrowser);
};

}  // namespace ug
}  // namespace cee
