//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Color3f.h"

namespace cee {
namespace ug {


//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT VectorSettings : public RefCountedObject
{
public:
    enum ColorMode
    {
        SINGLE,
        COLOR_BY_FRINGES
    };

    enum ScaleMode
    {
        ABSOLUTE_SCALING,
        RELATIVE_SCALING
    };

    enum DrawMode
    {
        NORMAL,
        SIMPLE
    };


public:
    virtual ~VectorSettings();

    ScaleMode   scaleMode() const;
    double      scaleFactor() const;
    void        setScaleFactor(ScaleMode mode, double scaleFactor);

    bool        resultInLocalCoordinates() const;
    void        setResultInLocalCoordinates(bool localCoordinates);

    ColorMode   colorMode() const;
    void        setColorMode(ColorMode colorMode);
    Color3f     singleVectorColor() const;
    void        setSingleVectorColor(const Color3f& color);

    int         simpleVectorLineWidth() const;
    void        setSimpleVectorLineWidth(int width);

    DrawMode    drawMode() const;
    void        setDrawMode(DrawMode mode);

    double      arrowShaftRelativeRadius() const;
    void        setArrowShaftRelativeRadius(double shaftRelativeRadius);
    double      arrowHeadRelativeRadius() const;
    void        setArrowHeadRelativeRadius(double headRelativeRadius);
    double      arrowHeadRelativeLength() const;
    void        setArrowHeadRelativeLength(double headRelativeLength);

    bool        isClampingEnabled() const;
    double      clampingRangeMinimum() const;
    double      clampingRangeMaximum() const;
    void        setClampingRange(double min, double max);
    void        disableClamping();

    bool        isLengthFilteringEnabled() const;
    double      lengthFilteringMinimum() const;
    double      lengthFilteringMaximum() const;
    void        setLengthFilteringRange(double min, double max);
    void        disableLengthFiltering();

    bool        showInteriorVectors() const;
    void        setShowInteriorVectors(bool show);

    unsigned int    drawSkipBy() const;
    void            setDrawSkipBy(unsigned int skipBy);

    size_t          drawMaximumCount() const;
    void            setDrawMaximumCount(size_t maximumCount);

private:
    VectorSettings();
    CEE_PRIVATE_F(UnstructGridModel);
    CEE_PRIVATE_IMPL(VectorSettings);
    CEE_DISALLOW_COPY_AND_ASSIGN(VectorSettings);
};

}  // namespace ug
}  // namespace cee

