//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: UnstructGrid
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeUnstructGrid/Base.h"
#include "CeeCore/Vec3d.h"

#include <vector>

namespace cee {
    class Image;
    class Vec2f;

namespace ug {
class UnstructGridModel;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_UG_EXPORT VisualizationPartQuery
{
public:
    VisualizationPartQuery(const UnstructGridModel* model, size_t frameIndex, size_t geometryIndex, int partId);
    ~VisualizationPartQuery();

    void                visibleNodes(std::vector<size_t>* visibleNodes);
    void                visibleElements(std::vector<size_t>* elementIndices);
    void                visibleElementSurfaces(std::vector<size_t>* elementIndices, std::vector<unsigned char>* surfaceIndices);
    
    void                visibleTriangles(std::vector<unsigned int>* indices);
    void                outlineEdges(std::vector<unsigned int>* edgeSegmentsNodeIndices);
    
    void                visibleIndexedTriangles(std::vector<unsigned int>* indices, std::vector<cee::Vec3d>* nodes);
    void                visibleIndexedLines(std::vector<unsigned int>* indices, std::vector<cee::Vec3d>* nodes);
    void                visiblePoints(std::vector<cee::Vec3d>* points);

    void                visibleNodesNormals(std::vector<cee::Vec3d>* normals);

    void                visibleNodesTextureCoordinates(std::vector<cee::Vec2f>* textureCoordinates);
    void                fringesLegendTextureImage(Image* image);
    
    bool                hasTriangles() const;
    bool                hasLines() const;
    bool                hasPoints() const;

private:
    CEE_PRIVATE_IMPL(VisualizationPartQuery);
    CEE_DISALLOW_COPY_AND_ASSIGN(VisualizationPartQuery);
};

}  // namespace ug
}  // namespace cee
