//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2017, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/MouseEvent.h"
#include "CeeVisualization/MarkupModel.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Plane.h"

namespace cee {
class Vec3d;
class Color3f;

namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT BoxLocator : public MarkupModel
{
public:
	BoxLocator(View* view);

	virtual ~BoxLocator();

	void				setup(const Vec3d& min, const Vec3d& max);

	void				tiltAlongAxis();

	void				setBoxColor(const Color3f& color);
	Color3f				boxColor() const;

	void				setBoxOpacity(float opacity);
	float				boxOpacity() const;

	void				setSphereColor(const Color3f& color);
	Color3f				sphereColor() const;

	void				setSelectedColor(const Color3f& color);
	Color3f				selectedColor();

	std::vector<Plane>	planes() const;
	Vec3d				centerPosition() const;

	void				boxNavigationOnMousePressEvent(MouseEvent* mouseEvent, const Vec3d& rotationPoint);
	void				boxNavigationOnMouseMoveEvent(MouseEvent* mouseEvent);
	void				boxNavigationOnMouseReleaseEvent(MouseEvent* mouseEvent);

	bool				onMousePressEvent(MouseEvent* mouseEvent);
	bool				onMouseMoveEvent(MouseEvent* mouseEvent);
	void				onMouseReleaseEvent();


private:
	CEE_PRIVATE_IMPL(BoxLocator);
	CEE_DISALLOW_COPY_AND_ASSIGN(BoxLocator);
};

} // namespace vis
} // namespace cee
