//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/Color3f.h"
#include <vector>

namespace cee {
namespace vis {

//==================================================================================================
//
// Factory functions for creating color tables for e.g. color legends/scalar mappers
//
//==================================================================================================
class CEE_VIS_EXPORT ColorTableFactory
{
public:
    /// The predefined color schemes supported by the color table factory
    enum ColorTable
    {
        NORMAL,             ///< The standard color map for scalar mapping in CDC. Following the color spectra, but starting on blue and ending on red.
        BLACK_WHITE,        ///< Black to while color map
        BLUE_RED,           ///< Blue to red color map
        BLUE_GREEN,         ///< Blue to green color map
        YELLOW_RED,         ///< Yellow to red color map
        GREEN_YELLOW_RED,   ///< A map from Green through yellow ending up in red
        RED_YELLOW,         ///< A red to yellow color map
        THERMAL_1,          ///< A map suitable for thermal scalars
        THERMAL_2,          ///< A map suitable for thermal scalars
        THERMAL_3,          ///< A map suitable for thermal scalars
        METAL_CASTING       ///< A map suitable for metal casting
    };
    
public:
    static std::vector<Color3f> colorTableArray(ColorTable colorTable, size_t colorCount);
    static std::vector<Color3f> interpolateColorArray(const std::vector<Color3f>& colorArray, size_t targetColorCount);
};

} // namespace vis
} // namespace cee
