//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Model.h"
#include "CeeVisualization/MarkupModelHitItem.h"
#include "CeeVisualization/MarkupModelHitItemCollection.h"

namespace cee {
class Ray;

namespace vis {
class MarkupPart;
class View;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupModel : public Model
{
public:
    MarkupModel();
    virtual ~MarkupModel();

    size_t                  partCount() const;
    size_t                  partIndex(MarkupPart* part) const;
    MarkupPart*             part(size_t index);
    const MarkupPart*       part(size_t index) const;
    void                    addPart(MarkupPart* part);
    void                    removePart(MarkupPart* part);
    void                    removeAllParts();

    virtual BoundingBox     boundingBox();
    virtual Str             modelInfo() const;
    size_t                  rayIntersect(const Ray& ray, const View& view);
    bool                    rayIntersect(const Ray& ray, const View& view, MarkupModelHitItem* hitItem);
    bool                    rayIntersectAllHits(const Ray& ray, const View& view, MarkupModelHitItemCollection* hitItems);
    
    bool                    isViewClippingIgnored() const;
    void                    setIgnoreViewClipping(bool ignore);

private:
    virtual cvf::Model*     internal_cvfModel();
    virtual cvf::Model*     internal_cvfModelForOverlayParts();
    virtual cvf::Model*     internal_cvfModelForOverlayDetectVisibleParts();
    virtual void            internal_setViewCapabilities(const cvf::OpenGLCapabilities& capabilities, bool viewDoesHighlighting);
    virtual bool            internal_canAddToViewWithCapabilities(const cvf::OpenGLCapabilities& viewCapabilities) const;
    virtual NotifyAction    internal_detectAndConfigureViewClippingInModel(const vis::View& triggeringView, cvf::OpenGLContext* cvfContext);
    virtual void            internal_applyViewClippingPlanesBeforeRendering(const cvf::ClipPlaneSet* viewClipPlaneSet, const cvf::Plane* dropShadowClipPlane);
    virtual void            internal_applyCameraTransformationToClippingPlanes(const cvf::Camera& camera);
    virtual NotifyAction    internal_preRenderUpdateModel(const View& triggeringView, cvf::OpenGLContext* cvfContext);

private:
    CEE_PRIVATE_IMPL(MarkupModel);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupModel);
};

} // namespace vis
} // namespace cee
