//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/BoundingBox.h"

namespace cvf { 
class Part; 
class Transform;
class OpenGLContext;
}

namespace cee {
namespace vis {
class MarkupClippingData;


//==================================================================================================
//
// Base class for MarkupModel parts
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPart : public RefCountedObject
{
public:
    /// Enum specifying the type of MarkupPart
    enum PartType
    {
        LABELS,                     ///< Labels part: MarkupPartLabels
        TEXTS_3D,                   ///< 3d text: MarkupPartText3d
        TRIANGLES,                  ///< Triangles part: MarkupPartTriangles
        INSTANCED_GEOMETRY,         ///< Instanced Triangles part: MarkupPartInstancedGeometry
        LINES,                      ///< Lines part: MarkupPartLines
        POINTS,                     ///< Points part: MarkupPartPoints
        ARROWS,                     ///< Arrows part: MarkupPartArrows
        FIXED_SIZE_ARROW,           ///< Fixed size arrow part: MarkupPartFixedSizeArrow
        FIXED_SIZE_SPHERE,          ///< Fixed size sphere part: MarkupPartFixedSizeSphere
        FIXED_SIZE_IMAGE_GLYPH      ///< Fixed size image glyph part: MarkupPartFixedSizeGlyph
    };

public:
    MarkupPart();
    virtual ~MarkupPart();

    virtual PartType            type() const = 0;                       ///< Returns the type of Markup part
    virtual BoundingBox         boundingBox() const = 0;                ///< Returns the current bounding box of the part

    bool                        visible() const;                    ///< Returns true if the part is visible
    void                        setVisible(bool visible);           ///< Sets if the part should be visible or not

    virtual int                 priority() const = 0;                   ///< Returns the render priority of the part
    virtual void                setPriority(int priority) = 0;          ///< Sets render priority of the part
    
    virtual const Mat4d&        transformation() const = 0;                     ///< Returns the current transformation matrix for the part
    virtual void                setTransformation(const Mat4d& matrix) = 0;     ///< Sets the transformation matrix to use for the part

private:
    virtual bool                internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData) = 0;
    virtual cvf::Part*          internal_cvfPart() = 0;
    virtual cvf::Part*          internal_cvfOverlayPart();
    virtual cvf::Part*          internal_cvfOverlayDetectVisiblePart();
    virtual cvf::Transform*     internal_cvfTransform();
    bool                        internal_checkAndClearVisibilityChanged();

private:
    CEE_PRIVATE_F(MarkupModel);
    CEE_PRIVATE_IMPL(MarkupPart);
};

} // namespace vis
} // namespace cee
