//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

#include <vector>

namespace cee {
class BoundingBox;

namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartArrows : public MarkupPart
{
public:
    MarkupPartArrows(); 
    MarkupPartArrows(const Color3f& color); 
    virtual ~MarkupPartArrows();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Color3f&                  color() const;
    void                            setColor(const Color3f& color);

    size_t                          count() const;
    Vec3d                           startPosition(size_t index) const;
    Vec3d                           endPosition(size_t index) const;
    void                            add(const Vec3d& startPosition, const Vec3d& endPosition);
    void                            add(const std::vector<Vec3d>& startPositions, const std::vector<Vec3d>& endPositions);
    void                            set(size_t index, const Vec3d& startPosition, const Vec3d& endPosition);
    void                            remove(size_t index);
    void                            removeAll();

    static PtrRef<MarkupPartArrows> create(const Vec3d& startPosition, const Vec3d& endPosition, const Color3f& color);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();

private:
    CEE_PRIVATE_IMPL(MarkupPartArrows);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartArrows);
};

} // namespace vis
} // namespace cee
