//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2017, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/MarkupPart.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

namespace cee {
namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartFixedSizeArrow : public MarkupPart
{
public:
	MarkupPartFixedSizeArrow();
	MarkupPartFixedSizeArrow(const Vec3d& position, const Vec3d& direction, double lengthInPixels, const Color3f& color);
	virtual ~MarkupPartFixedSizeArrow();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Color3f&                  color() const;
    void                            setColor(const Color3f& color);

    Vec3d                           position() const;
	void							setPosition(const Vec3d& pos);

	Vec3d							direction() const;
	void							setDirection(const Vec3d& dir);

	double                          lengthInPixels() const;
	void                            setLengthInPixels(double length);

	unsigned int                    numberOfSubDivisions() const;
	void                            setNumberOfSubDivisions(unsigned int numSubDivisions);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();
	virtual cvf::Transform*         internal_cvfTransform();

private:
	CEE_PRIVATE_IMPL(MarkupPartFixedSizeArrow);
	CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartFixedSizeArrow);
};

} // namespace vis
} // namespace cee
