//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

#include <vector>

namespace cee {
class BoundingBox;
class Image;

namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartFixedSizeGlyph : public MarkupPart
{
public:
    MarkupPartFixedSizeGlyph(); 
    MarkupPartFixedSizeGlyph(Image* image, const Vec3d& position, unsigned int pixelSizeLongestEdge); 
    virtual ~MarkupPartFixedSizeGlyph();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Image*                    image() const;
    void                            setImage(Image* image);

    Vec3d                           position() const;
    void                            setPosition(const Vec3d& position);

    unsigned int                    pixelSizeLongestEdge() const;
    void                            setPixelSizeLongestEdge(unsigned int pixelSize);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();
    virtual cvf::Part*              internal_cvfOverlayPart();
    virtual cvf::Transform*         internal_cvfTransform();

private:
    CEE_PRIVATE_IMPL(MarkupPartFixedSizeGlyph);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartFixedSizeGlyph);
};

} // namespace vis
} // namespace cee
