//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/PtrRef.h"

#include <vector>

namespace cee {
class BoundingBox;

namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartFixedSizeSphere : public MarkupPart
{
public:
	MarkupPartFixedSizeSphere();
    MarkupPartFixedSizeSphere(const Vec3d& centerPosition, const Color3f& color, double radiusInPixels, unsigned int numSubDivisions); 
    virtual ~MarkupPartFixedSizeSphere();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Vec3d&                    centerPosition() const;
    void                            setCenterPosition(const Vec3d& centerPos);

    const Color3f&                  color() const;
    void                            setColor(const Color3f& color);

    double                          radiusInPixels() const;
    void                            setRadiusInPixels(double radius);

    unsigned int                    numberOfSubDivisions() const;
    void                            setNumberOfSubDivisions(unsigned int numSubDivisions);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();
    virtual cvf::Transform*         internal_cvfTransform();

private:
    CEE_PRIVATE_IMPL(MarkupPartFixedSizeSphere);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartFixedSizeSphere);
};

} // namespace vis
} // namespace cee
