//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

#include <vector>

namespace cee {
class BoundingBox;

namespace vis {
    

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartInstancedGeometry : public MarkupPart
{
public:

	/// Geometry primitive type
	enum PrimitiveType 
	{
		TRIANGLES,
		LINES
	};
	
	MarkupPartInstancedGeometry();
    MarkupPartInstancedGeometry(const Color3f& singleColor, float polygonOffsetFactor, float eyeLiftFactor);
    virtual ~MarkupPartInstancedGeometry();

    virtual PartType                    type() const;
    virtual BoundingBox                 boundingBox() const;

    virtual void                        setPriority(int priority);
    virtual int                         priority() const;

    virtual const Mat4d&                transformation() const;
    virtual void                        setTransformation(const Mat4d& matrix);
    
    void                                setGeometry(PrimitiveType primitiveType, const std::vector<unsigned int>& indices, const std::vector<Vec3d>& vertices);
    const std::vector<unsigned int>&    geometryIndices() const;
    const std::vector<Vec3d>&           geometryVertices() const;
    PrimitiveType                       geometryPrimitiveType() const;

    const Color3f&                      singleColor() const;
    void                                setSingleColor(const Color3f& color);
    
    bool                                fixedPixelSize() const;
    void                                setFixedPixelSize(bool fixedSize);
    
    float                               polygonOffsetFactor() const;
    void                                setPolygonOffsetFactor(float factor);

    float                               eyeLiftFactor() const;
    void                                setEyeLiftFactor(float factor);
    
    void                                setInstanceTransformArray(const std::vector<Mat4d>& transformArr);
    void                                setInstanceColorArray(const std::vector<Color3f>& colorArr);
    
    void                                addInstance(const Mat4d& transform);
    void                                addInstance(const Mat4d& transform, const Color3f& color);
    size_t                              instanceCount() const;
    void                                removeAllInstances();

protected:
    virtual bool                internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*          internal_cvfPart();

private:
    CEE_PRIVATE_IMPL(MarkupPartInstancedGeometry);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartInstancedGeometry);
};

} // namespace vis
} // namespace cee
