//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

#include <vector>

namespace cee {
class BoundingBox;

namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartPoints : public MarkupPart
{
public:
    MarkupPartPoints(); 
    MarkupPartPoints(const Color3f& color, float pointSize, bool alwaysVisible, float eyeLiftFactor); 
    virtual ~MarkupPartPoints();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Color3f&                  color() const;
    void                            setColor(const Color3f& color);

    float                           pointSize() const;
    void                            setPointSize(float pointSize);

    bool                            alwaysVisible() const;
    void                            setAlwaysVisible(bool showAlways);

    float                           eyeLiftFactor() const;
    void                            setEyeLiftFactor(float factor);

    void                            add(const Vec3d& vertex);
    void                            add(const std::vector<Vec3d>& vertices);
    size_t                          count() const;
    void                            removeAll();

    const std::vector<Vec3d>&       vertices() const;

    static PtrRef<MarkupPartPoints> create(const std::vector<Vec3d>& vertices, const Color3f& color, float pointSize = 1.0f, bool alwaysVisible = false, float eyeLiftFactor = 0.0f);
    static PtrRef<MarkupPartPoints> create(const Vec3d& vertex, const Color3f& color, float pointSize = 1.0f, bool alwaysVisible = false, float eyeLiftFactor = 0.0f);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();

private:
    CEE_PRIVATE_IMPL(MarkupPartPoints);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartPoints);
};

} // namespace vis
} // namespace cee
