//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2015, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/MarkupPart.h"
#include "CeeVisualization/MarkupText3dItem.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Str.h"
#include "CeeCore/Vec3d.h"
#include "CeeCore/PtrRef.h"

namespace cee {
class BoundingBox;

namespace vis {
class Font;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT MarkupPartText3d: public MarkupPart
{
public:
    MarkupPartText3d();
    virtual ~MarkupPartText3d();

    virtual PartType                type() const;
    virtual BoundingBox             boundingBox() const;

    virtual void                    setPriority(int priority);
    virtual int                     priority() const;

    virtual const Mat4d&            transformation() const;
    virtual void                    setTransformation(const Mat4d& matrix);
    
    const Font*                     font() const;
    void                            setFont(Font* labelFont);

    const Color3f&                  textColor() const;
    void                            setTextColor(const Color3f& color);
    
    float                           eyeLiftFactor() const;
    void                            setEyeLiftFactor(float factor);
    
    unsigned int                    maximumNumberOfLabelsToDraw() const;
    void                            setMaximumNumberOfLabelsToDraw(unsigned int maximumNumberToDraw);

    size_t                          count() const;
    MarkupText3dItem                textItem(size_t index) const;
    size_t                          add(const MarkupText3dItem& item);
    size_t                          add(const Str& text, const Vec3d& position, const Vec3d& directionVector, const Vec3d& upVector, double textHeight);
    void                            set(size_t index, const MarkupText3dItem& item);
    void                            remove(size_t index);
    void                            removeAll();

    static PtrRef<MarkupPartText3d> create(const Str& text, const Color3f& color, const Vec3d& position, const Vec3d& directionVector, const Vec3d& upVector, double textHeight);

protected:
    virtual bool                    internal_preRenderUpdatePart(bool forceUpdate, cvf::OpenGLContext* cvfContext, MarkupClippingData* clippingData);
    virtual cvf::Part*              internal_cvfPart();

private:
    CEE_PRIVATE_F(MarkupModel);
    CEE_PRIVATE_IMPL(MarkupPartText3d);
    CEE_DISALLOW_COPY_AND_ASSIGN(MarkupPartText3d);
};

} // namespace vis
} // namespace cee
