//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/Camera.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/BoundingBox.h"
#include "CeeCore/Str.h"

#include <vector>

namespace cvf {
class Model;
class OpenGLCapabilities;
class ClipPlaneSet;
class Plane;
class OpenGLContext;
class Camera;
}

namespace cee {
namespace vis {

class OverlayItem;
class View;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT Model : public RefCountedObject
{
public:
    Model();
    virtual ~Model();

    Str                     name() const;
    void                    setName(const Str& name);

    bool                    use2dPixelProjection() const;
    void                    setUse2dPixelProjection(bool use2dMode);

    virtual BoundingBox     boundingBox() = 0;  ///< Returns the model's bounding box
    virtual Str             modelInfo() const;
    
protected:
    enum NotifyAction
    {
        NO_ACTION,             
        MODEL_CHANGED
    };

    void                    notifyObservingViews(NotifyAction notifyAction, const View* exceptView);
    bool                    hasAnyObservingViewsWithClipping(bool* outExplicitViewClippingIsActive = NULL, bool* outDropShadowClippingIsActive = NULL) const;

private:
    void                                            internal_addView(View* view);
    void                                            internal_removeView(View* view);
    virtual cvf::Model*                             internal_cvfModel() = 0;
    virtual cvf::Model*                             internal_cvfModelForOverlayParts();
    virtual cvf::Model*                             internal_cvfModelForOverlayDetectVisibleParts();
    virtual std::vector<OverlayItem*>               internal_overlayItems();
    virtual void                                    internal_setViewCapabilities(const cvf::OpenGLCapabilities& capabilities, bool viewDoesHighlighting);
    virtual bool                                    internal_canAddToViewWithCapabilities(const cvf::OpenGLCapabilities& viewCapabilities) const;
    virtual NotifyAction                            internal_detectAndConfigureViewClippingInModel(const View& triggeringView, cvf::OpenGLContext* cvfContext) = 0;
    virtual void                                    internal_applyViewClippingPlanesBeforeRendering(const cvf::ClipPlaneSet* viewClipPlaneSet, const cvf::Plane* dropShadowClipPlane) = 0;
    virtual void                                    internal_applyCameraTransformationToClippingPlanes(const cvf::Camera& camera) = 0;
    virtual NotifyAction                            internal_preRenderUpdateModel(const View& triggeringView, cvf::OpenGLContext* cvfContext);

private:
    CEE_BASE_F(View);
    CEE_PRIVATE_F(View);
    CEE_PRIVATE_F(Overlay);
    CEE_PRIVATE_IMPL(Model);
    CEE_DISALLOW_COPY_AND_ASSIGN(Model);
};

} // namespace vis
} // namespace cee
