//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"

namespace cee {
namespace vis {


enum MouseButton
{
    NoButton     = 0x00000000,
    LeftButton   = 0x00000001,
    RightButton  = 0x00000002,
    MiddleButton = 0x00000004
};

typedef int MouseButtons;


enum KeyboardModifier
{
    NoModifier      = 0x00000000,
    ShiftModifier   = 0x00000010,
    ControlModifier = 0x00000020
};

typedef int KeyboardModifiers;


//==================================================================================================
//
// Mouse event
//
//==================================================================================================
class CEE_VIS_EXPORT MouseEvent
{
public:
    MouseEvent(int x, int y, MouseButtons buttonsDown, KeyboardModifiers modifiersDown);
    virtual ~MouseEvent();

    int                 x() const;                  // In OpenGL style coordinates with origin in lower left corner
    int                 y() const;                  // In OpenGL style coordinates with origin in lower left corner
    MouseButtons        buttons() const;
    KeyboardModifiers   modifiers() const;

private:
    int                 m_x;                        // In OpenGL style coordinates with origin in lower left corner
    int                 m_y;                        // In OpenGL style coordinates with origin in lower left corner
    MouseButtons        m_mouseButtonsDown;         // Combination of mouse buttons that are down
    KeyboardModifiers   m_keyboardModifiersDown;    // Combination of modifier keys that are down. 
};

} // namespace vis
} // namespace cee
