//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/OverlayItem.h"
#include "CeeCore/Str.h"
#include "CeeCore/Color3f.h"
#include "CeeVisualization/Font.h"

namespace cee {
namespace vis {


//==================================================================================================
//
// A color legend used to indicate the interpretation of the colors in the model
//
//==================================================================================================
class CEE_VIS_EXPORT OverlayCategoryLegend: public OverlayItem
{
public:
    /// Position of the legend tick marks
    enum TickMarksPosition
    {
        LEFT,           ///< Draw tick marks on the left side of the color legend
        RIGHT,          ///< Draw tick marks on the right side of the color legend
        TOP,            ///< Draw tick marks above the color legend
        BOTTOM,         ///< Draw tick marks below the color legend
        FOLLOW_LAYOUT   ///< Automatically position the tick marks left or right based on the color legend's alignment in the view
    };

    /// Direction of color legend bar
    enum Direction
    {
        HORIZONTAL,     ///< Draw horizontal color legend bar
        VERTICAL        ///< Draw vertical color legend bar
    };

public:
    OverlayCategoryLegend();
    OverlayCategoryLegend(Font* titleFont, Font* categoryFont);
    virtual ~OverlayCategoryLegend();

    virtual unsigned int    width() const;
    virtual unsigned int    height() const;

    virtual int             priority() const;
    virtual void            setPriority(int priority);

    void                    setSize(unsigned int width, unsigned int height);
    void                    setWidthToFitText();

    size_t                  categoryCount() const;
    Str                     categoryText(size_t index) const;
    Color3f                 categoryColor(size_t index) const;
    void                    addCategory(const Str& text, Color3f color);
    void                    removeAllCategories();

    Str                     title() const;
    void                    setTitle(const Str& title);

    Color3f                 textColor() const;
    void                    setTextColor(const Color3f& color);

    TickMarksPosition       tickMarksPosition() const;
    void                    setTickMarksPosition(TickMarksPosition position);

    Direction               direction() const;
    void                    setDirection(Direction direction);

private:
    virtual cvf::OverlayItem* internal_cvfOverlayItem();

private:
    CEE_PRIVATE_IMPL(OverlayCategoryLegend);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayCategoryLegend);
};

} // namespace vis
} // namespace cee
