//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/OverlayItem.h"
#include "CeeVisualization/ScalarMapperContinuousDomain.h"
#include "CeeCore/Str.h"
#include "CeeCore/Color3f.h"

#include <vector>

namespace cee {
namespace vis {
class Font;

//==================================================================================================
//
// A color legend used to show a scalar range and the corresponding colors with uniform level sizes
//
//==================================================================================================
class CEE_VIS_EXPORT OverlayColorLegendContinuousDomain: public OverlayItem
{
public:
    OverlayColorLegendContinuousDomain(Font* font);
    virtual ~OverlayColorLegendContinuousDomain();
    
    void                    setupFromScalarMapper(const ScalarMapperContinuousDomain* scalarMapper);
    void                    setupNiceNumbersTickMarks(unsigned int maxNumTickMarks, const ScalarMapperContinuousDomain* scalarMapper);
    void                    setupNiceNumbersTickMarksLogarithmic(unsigned int maxNumTickMarks, const ScalarMapperContinuousDomain* scalarMapper);
    void                    setTickValues(const std::vector<double>& tickValues, const ScalarMapperContinuousDomain* scalarMapper);

    virtual unsigned int    width() const;
    virtual unsigned int    height() const;
    void                    setSize(unsigned int width, unsigned int height);
    void                    setWidthToFitText();

    virtual int             priority() const;
    virtual void            setPriority(int priority);

    Font*                   titleFont();
    void                    setTitleFont(Font* font);

    Font*                   tickMarkFont();
    void                    setTickMarkFont(Font* font);
    
    unsigned int            legendBarWidth() const;
    void                    setLegendBarWidth(unsigned int widthInPixels);

    Str                     title() const;
    void                    setTitle(const Str& title);
    
    Color3f                 textColor() const;
    void                    setTextColor(const Color3f& color);

    char                    numericFormat() const;
    void                    setNumericFormat(char format);

    int                     numericPrecision() const;
    void                    setNumericPrecision(int precision);
    
    void                    setShowTickMarkLabels(bool show);
    bool                    showTickMarkLabels() const;
    
    void                    setForceUniformTickMarkSpacing(bool forceUniformSpacing);
    bool                    forceUniformTickMarkSpacing() const;

    void                    setTickMarksThroughLegendBar(bool drawOverLegendBar);
    bool                    tickMarksThroughLegendBar() const;
    
    void                    setCustomLabels(const std::vector<Str>& labels, const std::vector<double>& relativePositions);
    void                    clearCustomLabels();
    
private:
    virtual cvf::OverlayItem* internal_cvfOverlayItem();

private:
    CEE_PRIVATE_IMPL(OverlayColorLegendContinuousDomain);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayColorLegendContinuousDomain);
};

} // namespace vis
} // namespace cee
