//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"

namespace cvf
{
    class OverlayItem;
}

namespace cee {
namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT OverlayItem : public RefCountedObject
{
public:
    /// Enumerator for the different locations/"corners" of the view
    enum LayoutCorner
    {
        TOP_LEFT,                   ///< Top left corner
        TOP_RIGHT,                  ///< Top right corner
        BOTTOM_LEFT,                ///< Bottom left corner
        BOTTOM_RIGHT,               ///< Bottom right corner
        CENTER,                     ///< Center of the view
        TOP_CENTER,                 ///< Top centered
        BOTTOM_CENTER               ///< Bottom centered
    };

    /// Enumerator for horizontal or vertical orientation of layout items. Describes
    /// which direction additional items should be positioned if more than one exists in the 
    /// same corner.
    enum LayoutDirection
    {
        HORIZONTAL,                 ///< Horizontal layout direction
        VERTICAL,                   ///< Vertical layout direction
        ABSOLUTE_POSITION           ///< Absolute position of the overlay item. Absolute offset from the given LayoutCorner. \sa setAbsolutePosition
    };

public:
    virtual unsigned int    width() const = 0;                                      ///< Returns the width of the overlay item
    virtual unsigned int    height() const = 0;                                     ///< Returns the height of the overlay item
    virtual void            setSize(unsigned int width, unsigned int height) = 0;

    virtual int             priority() const = 0;                                   ///< Returns the render priority of the item
    virtual void            setPriority(int priority) = 0;                          ///< Sets render priority of the item

    bool                    visible() const;
    void                    setVisible(bool visible);

    int                     absolutePositionX() const;
    int                     absolutePositionY() const;
    void                    setAbsolutePosition(int x, int y);


private:
    virtual cvf::OverlayItem* internal_cvfOverlayItem() = 0;

private:
    CEE_PRIVATE_F(Overlay);
};

} // namespace vis
} // namespace cee
