//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/OverlayItem.h"
#include "CeeCore/Str.h"

namespace cee {
class Color3f;

namespace vis {
class Font;

//==================================================================================================
//
// A text box that can be used as an overlay item in a view
//
//==================================================================================================
class CEE_VIS_EXPORT OverlayTextBox : public OverlayItem
{
public:

    /// Alignment of text content within a text box
    enum TextBoxContentAlignment 
    {
        LEFT,       ///< Align left
        CENTER,     ///< Align center
        RIGHT       ///< Align right
    };

    OverlayTextBox(Font* font);
    virtual ~OverlayTextBox();

    virtual unsigned int    width() const;
    virtual unsigned int    height() const;
    void                    setSize(unsigned int width, unsigned int height);

    virtual int             priority() const;
    virtual void            setPriority(int priority);

    void                    setSizeToFitText();

    void                    setFont(Font* font);

    Str                     text() const;
    void                    setText(const Str& text);

    Color3f                 textColor() const;
    void                    setTextColor(const Color3f& color);

    Color3f                 backgroundColor() const;
    void                    setBackgroundColor(const Color3f& color);

    float                   backgroundOpacity() const;
    void                    setBackgroundOpacity(float opacity);

    Color3f                 borderColor() const;
    void                    setBorderColor(const Color3f& color);

    float                   borderWidth() const;
    void                    setBorderWidth(float width);

    bool                    drawBackground() const;
    void                    setDrawBackground(bool drawBackground);

    bool                    drawBorder() const;
    void                    setDrawBorder(bool drawBorder);

    TextBoxContentAlignment textContentAlignment() const;
    void                    setTextContentAlignment(TextBoxContentAlignment alignment);

private:
    virtual cvf::OverlayItem* internal_cvfOverlayItem();

private:
    CEE_PRIVATE_IMPL(OverlayTextBox);
    CEE_DISALLOW_COPY_AND_ASSIGN(OverlayTextBox);
};

} // namespace vis
} // namespace cee
