//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"

namespace cee {
namespace vis {

//==================================================================================================
/// \class PerformanceInfo
/// \ingroup Visualization
///
/// Class for storing performance info stats
//==================================================================================================
class CEE_VIS_EXPORT PerformanceInfo
{
public:
    double  totalDrawTime;              ///< Total amount of time (in seconds) used to draw the last frame
    double  averageTotalDrawTime;       ///< Average over the last 10 renderings

    double  computeVisiblePartsTime;    ///< Time (in seconds) used to create the visible part collection
    double  buildRenderQueueTime;       ///< Time (in seconds) used to build the rendering queue
    double  sortRenderQueueTime;        ///< Time (in seconds) used to sort the render queue
    double  renderEngineTime;           ///< Time (in seconds) used to render the pre-processed rendering queue
    size_t  renderingCount;             ///< Number of renderings

    size_t  visiblePartsCount;          ///< Number of visible parts
    size_t  renderedPartsCount;         ///< Number of parts that was drawn
    size_t  vertexCount;                ///< Number of vertices (nodes, points) used to draw.
    size_t  triangleCount;              ///< Number of triangles (GL_TRIANGELS) 
    size_t  openGLPrimitiveCount;       ///< Total number of GL primitives drawn (lines, points, polygons, etc). For strips the number of resulting triangles.
    size_t  applyRenderStateCount;      ///< Number of render state changes
    size_t  shaderProgramChangesCount;  ///< Number of shader program changes

    size_t  vboCount;                   ///< Number of VBOs in use
    double  vboMemoryUsageMB;           ///< Total memory used by VBOs (in megabytes)
};

} // namespace vis
} // namespace cee
