//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2012, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/MouseEvent.h"
#include "CeeVisualization/MarkupModel.h"

#include "CeeCore/RefCountedObject.h"
#include "CeeCore/Plane.h"

namespace cee {
class Vec3d;
class Color3f;

namespace vis {
class Camera;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT PlaneLocator: public MarkupModel
{
public:
	PlaneLocator(View* view);

    virtual ~PlaneLocator();

	void			setup(const Vec3d& point, const Vec3d& normal);
	Vec3d			point() const;
    void            setPoint(const Vec3d& point);
	Vec3d			normal() const;

    void            setMaximumExtent(double maxExtent);
    double          maximumExtent() const;

	Color3f			planeColor() const;
	void			setPlaneColor(const Color3f& color);

    void		    setPlaneOpacity(float opacity);
    float		    planeOpacity() const;

	Color3f			centerPointColor() const;
	void			setCenterPointColor(const Color3f& color);

	Color3f			arrowColor() const;
	void			setArrowColor(const Color3f& color);

	bool			surfaceVisible() const;
	void			setSurfaceVisible(bool visible);

	bool			borderVisible() const;
	void			setBorderVisible(bool visible);

    void            planeNavigationOnMousePressEvent(MouseEvent* mouseEvent, const Vec3d& rotationPoint);
	void            planeNavigationOnMouseMoveEvent(MouseEvent* mouseEvent);
	void            planeNavigationOnMouseReleaseEvent(MouseEvent* mouseEvent);

private:
    CEE_PRIVATE_IMPL(PlaneLocator);
    CEE_DISALLOW_COPY_AND_ASSIGN(PlaneLocator);
};

} // namespace vis
} // namespace cee
