//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/RefCountedObject.h"
#include "CeeVisualization/Base.h"
#include "CeeCore/Color3f.h"
#include "CeeCore/Vec2f.h"


namespace cee {
class Image;

namespace vis {

//==================================================================================================
///
/// \class cee::vis::ScalarMapper
/// \ingroup Visualization
///
/// Abstract base class for scalar mappers.
///
/// The scalar mapper offers two services:
/// - Map from a scalar value (domain value) to a texture coordinate or a color
/// - Create a texture from the scalar mapper that can be used for rendering and corresponds with
///   the texture coordinates produced.
///
//==================================================================================================

class CEE_VIS_EXPORT ScalarMapper : public RefCountedObject
{
public:
    virtual ~ScalarMapper() {};

    virtual Vec2f           mapToTextureCoordinate(double scalarValue) const = 0;   ///< Maps scalar value to texture coordinate
    virtual Color3f         mapToColor(double scalarValue) const = 0;               ///< Maps scalar value to color

    virtual bool            updateTexture(Image* image) const = 0;                  ///< Updates texture image based on the settings in the scalar mapper
};

} // namespace vis
} // namespace cee
