//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2016, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeCore/Color4f.h"
#include "CeeVisualization/Base.h"
#include "CeeVisualization/ScalarMapper.h"


#include <vector>


namespace cvf {
    class ScalarMapperContinuousDomain;
}

namespace cee {
class Image;

namespace vis {


//==================================================================================================
//
/// Base class for scalar mappers that implement normalizedValue / domainValue
/// 
//==================================================================================================
class CEE_VIS_EXPORT ScalarMapperContinuousDomain : public ScalarMapper
{
public:
    virtual ~ScalarMapperContinuousDomain();

    virtual double              normalizedValue(double domainValue) const = 0;
    virtual double              domainValue(double normalizedPosition) const = 0;
    
    virtual unsigned int        version() const = 0;
    
    virtual std::vector<double> tickValuesHint() const;
    
    Color4f                     undefinedColor() const;
    void                        setUndefinedColor(const Color4f& color);
    
    bool                        aboveRangeColor(Color4f* color) const;
    void                        setAboveRangeColor(const Color4f& color);
    void                        clearAboveRangeColor();
    
    bool                        belowRangeColor(Color4f* color) const;
    void                        setBelowRangeColor(const Color4f& color);
    void                        clearBelowRangeColor();

private:
    virtual cvf::ScalarMapperContinuousDomain*  internal_cvfScalarMapper() = 0;

private:
    CEE_BASE_F(OverlayColorLegendContinuousDomain);
};

} // namespace vis
} // namespace cee
