//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/ScalarMapperContinuousDomain.h"
#include "CeeVisualization/ColorTableFactory.h"

#include <vector>

namespace cee {
namespace vis {

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT ScalarMapperContinuousPiecewise : public ScalarMapperContinuousDomain
{
public:
    ScalarMapperContinuousPiecewise();
    virtual ~ScalarMapperContinuousPiecewise();

    double                  rangeMinimum() const;
    double                  rangeMaximum() const;

    void                    setup(const std::vector<Color3f>& colorsFromToArray, const std::vector<double>& tickValuesArray);

    virtual Vec2f           mapToTextureCoordinate(double scalarValue) const;
    virtual Color3f         mapToColor(double scalarValue) const;

    virtual bool            updateTexture(Image* image) const;

    virtual double          normalizedValue(double domainValue) const;
    virtual double          domainValue(double normalizedPosition) const;

    virtual unsigned int    version() const;

private:
    CEE_PRIVATE_IMPL(ScalarMapperContinuousPiecewise);
    CEE_DISALLOW_COPY_AND_ASSIGN(ScalarMapperContinuousPiecewise);
    
    virtual cvf::ScalarMapperContinuousDomain*  internal_cvfScalarMapper();
};

} // namespace vis
} // namespace cee
