//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2011, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeCore/RefCountedObject.h"
#include "CeeCore/BoundingBox.h"
#include "CeeVisualization/Background.h"
#include "CeeVisualization/TransparencyMode.h"
#include "CeeVisualization/Model.h"

namespace cee {
class Image;

namespace vis {

class Viewer;
class Model;
class Camera;
class InputManager;
class PerformanceInfo;
class Overlay;
class Clipping;

//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT View : public RefCountedObject
{
public:
    View();
    virtual ~View();

    const Viewer*                   attachedViewer() const;
    void                            attachToViewer(Viewer* viewer);
    void                            detachFromViewer();

    size_t                          modelCount() const;
    size_t                          modelIndex(const Model* model) const;
    Model*                          model(size_t modelIndex);
    void                            addModel(Model* model);
    void                            removeAllModels();
    void                            removeModel(Model* model);
    void                            setModelOpenGLCapabilitiesFromView(Model* model) const;
    
    void                            addSecondPassModel(Model* model);

    const Overlay&                  overlay() const;
    Overlay&                        overlay();

    const Clipping&                 clipping() const;
    Clipping&                       clipping();

    const Camera&                   camera() const;
    Camera&                         camera();
    BoundingBox                     boundingBox();

    Background                      background() const;
    void                            setBackground(const Background& background);

    Color3f                         haloColor() const;
    void                            setHaloColor(const Color3f& color);

    bool                            isDropShadowEnabled() const;
    bool                            enableDropShadow(bool enable);

    TransparencyMode                transparencyMode() const;
    void                            setTransparencyMode(const TransparencyMode& transparency);

    bool                            renderToImage(unsigned int width, unsigned int height, Image* image);
    bool                            renderToImage(Image* image);

    void                            renderViewInAttachedViewer();
    void                            lastRenderingPerformanceInfo(PerformanceInfo* info) const;

    void                            requestRedraw();
    void                            requestImmediateRedraw();
    
    bool                            regionClosestVertex(int x, int y, unsigned int width, unsigned int height, Vec3d* vertex);

    void*                           framebufferData();

private:
    void                            internal_handleNotificationFromModel(Model::NotifyAction notifyAction);

private:
    CEE_BASE_F(Camera);
    CEE_BASE_F(Model);
    CEE_PRIVATE_IMPL(View);
    CEE_DISALLOW_COPY_AND_ASSIGN(View);
};

} // namespace vis
} // namespace cee
