//##################################################################################################
//
//  Ceetron Desktop Components
//  Component: Visualization
//  
//  --------------------------------------------------------------------------------------------
//  Copyright (C) 2014, Ceetron AS
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Ceetron AS. The contents of this file may 
//  not be disclosed to third parties, copied or duplicated in any form, in whole or in part,
//  without the prior written permission of Ceetron AS.
//##################################################################################################

#pragma once

#include "CeeVisualization/Base.h"
#include "CeeVisualization/Viewer.h"
#include "CeeVisualization/OpenGLInfo.h"

namespace cee {
namespace vis {

class OpenGLContextGroup;



//==================================================================================================
//
// 
//
//==================================================================================================
class CEE_VIS_EXPORT ViewerUIFramework : public Viewer
{
public:
    ViewerUIFramework(OpenGLContextGroup* contextGroup);
    ~ViewerUIFramework();

protected:
    virtual void            doRequestRedraw() = 0;
    virtual void            doRequestImmediateRedraw() = 0;
    virtual bool            doMakeCurrent() = 0;
    virtual bool            doIsCurrent() = 0;
    virtual unsigned int    doGetDefaultFramebufferObject() = 0;

protected:
    bool                        initializeComponentInternalOpenGL();
    void                        shutdownComponentInternalOpenGL();
    OpenGLInfo                  openGLInfo() const;

private:
    virtual void                internal_requestRedraw();
    virtual void                internal_requestImmediateRedraw();
    virtual bool                internal_makeOpenGLContextCurrent();
    virtual cvf::OpenGLContext* internal_cvfOpenGLContext();

private:
    CEE_BASE_F(ForwardingOpenGLContext);
    CEE_PRIVATE_IMPL(ViewerUIFramework);
    CEE_DISALLOW_COPY_AND_ASSIGN(ViewerUIFramework);
};


//==================================================================================================
//
// 
//
//==================================================================================================
#ifdef CEE_MONO
class CEE_VIS_EXPORT ViewerUIFrameworkMono : public ViewerUIFramework
{
public:
    ViewerUIFrameworkMono (OpenGLContextGroup* contextGroup);
    ~ViewerUIFrameworkMono ();

protected:
    virtual void    doRequestRedraw();
    virtual void    doRequestImmediateRedraw();
    virtual bool    doMakeCurrent();
    virtual bool    doIsCurrent();
};
#endif

} // namespace vis
} // namespace cee


